/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: linkstart.c,v 1.10 92/06/11 15:43:56 rws Exp $
 */

#include	"xtest.h"
#include	"tet_api.h"
#include "stdlib.h"
#include	<stdio.h>
#include	<string.h>

/* For stdlib.h that are not quite standard */
#ifndef EXIT_FAILURE
#define	EXIT_FAILURE 1
#endif

#define MAXTPS	1024
#define TLI(fun, y) fun, y +1

/* Generate 8 ICs */
#define TLI1(fun, y) TLI(fun, y##0), TLI(fun, y##1), TLI(fun, y##2), TLI(fun, y##3), TLI(fun, y##4), TLI(fun, y##5), TLI(fun, y##6), TLI(fun, y##7)

/* Generate 8x8 ICs */
#define TLI2(fun, y) TLI1(fun, y##0), TLI1(fun, y##1), TLI1(fun, y##2), TLI1(fun, y##3), TLI1(fun, y##4), TLI1(fun, y##5), TLI1(fun, y##6), TLI1(fun, y##7)

/* Generate 8x8x8 ICs */
#define TLI3(fun, y) TLI2(fun, y##0), TLI2(fun, y##1), TLI2(fun, y##2), TLI2(fun, y##3), TLI2(fun, y##4), TLI2(fun, y##5), TLI2(fun, y##6), TLI2(fun, y##7)

static void
nullfn()
{
	return;
}

/*
 * Define the TET interface variables here.  This module will be
 * brought into the executable by the references to these variables.
 */
int 	tet_thistest;

struct	tet_testlist tet_testlist[MAXTPS+1]={TLI3(nullfn, 0), TLI3(nullfn, 1), NULL, 0};

void	linkstart();
void	linkclean();

void 	(*tet_startup)() = linkstart;
void 	(*tet_cleanup)() = linkclean;

char	*TestName;
int 	ntests;

extern	char	*tet_pname;

extern	struct	linkinfo	*linktbl[];

/*
 * The startup routine for the linked executables case.  We have
 * to make some assumptions about the TET that are not officialy in
 * the spec.
 *   - It is valid to change the contents of tet_testlist[] in the
 *     startup routine.
 * 
 * Get the name of the routine and set up the interface to
 * point to the tests for that routine.
 */
void
linkstart()
{
struct	linkinfo	*lp;
struct	linkinfo	**lpp;
struct	tet_testlist *tlp;
char	*name;
int 	i;

	lp = (struct linkinfo *)0;

	/*
	 * Get the basename part of tet_pname.
	 */
	name = strrchr(tet_pname, '/');
	if (name)
		name++;
	else
		name = tet_pname;

	for (lpp = linktbl; *lpp; lpp++) {
		if (strcmp((*lpp)->name, name) == 0) {
			lp = *lpp;
			break;
		}
	}

	if (lp == (struct linkinfo *)0) {
		/*
		 * If this happens if probably means that the executable
		 * has been built incorrectly.
		 */
		report("Name (%s) not found in link table, aborting test\n", name);
		fprintf(stderr, "Name (%s) not found in link table, aborting test\n", name);
		exit(EXIT_FAILURE);
	}

	TestName = lp->testname;
	ntests   = *lp->ntests;

	tlp = lp->testlist;
	for (i = 0; tlp[i].testfunc != 0; i++) {
		tet_testlist[i] = tlp[i];
	}
	/* Copy the final null entry */
	tet_testlist[i] = tlp[i];

	if (lp->localstartup)
		(*lp->localstartup)();
	else
		startup();

}

/*
 * Cleanup function for linked executables.
 */
void
linkclean()
{
struct	linkinfo	*lp;
struct	linkinfo	**lpp;
char 	*name;

	lp = (struct linkinfo *)0;

	/*
	 * Get the basename part of tet_pname.
	 */
	name = strrchr(tet_pname, '/');
	if (name)
		name++;
	else
		name = tet_pname;

	for (lpp = linktbl; *lpp; lpp++) {
		if (strcmp((*lpp)->name, name) == 0) {
			lp = *lpp;
			break;
		}
	}

	if (lp == (struct linkinfo *)0) {
		/*
		 * If this happens here, something has gone VERY BADLY WRONG
		 * and I'm not certain what we can do about it.
		 */
		report("Name (%s) not found in link table in linkclean!\n", name);
		fprintf(stderr, "Name (%s) not found in link table in linkclean!\n", name);
		exit(EXIT_FAILURE);
	}

	if (lp->localcleanup)
		(*lp->localcleanup)();
	else
		cleanup();
}
