/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: btmpunt.m,v 1.6 92/06/11 15:56:00 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XBitmapUnit(display)
 *   Display	*display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XBitmapUnit	1
char    *TestName = "XBitmapUnit";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XBitmapUnit function
 */
static Display	*display;


static int 	ValueReturn;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XBitmapUnit-1.(A)
 * A call to XBitmapUnit returns an integer 8, 16, or 32 which
 * is less than the bitmap pad value and which is the size in
 * bits of a scanline unit belonging to bitmaps.
 */
static void
t001()
{
int	bunt;
int	pad;
XImage	*xi;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	bunt = XBitmapUnit(display);	
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( (bunt != 8) && (bunt != 16) && (bunt != 32) ) {
		report("%s() returned illegal value %d.", TestName, bunt);
		FAIL;
	} else
		CHECK;

	pad = XBitmapPad(display);

	if( bunt > pad ) {
		report("%s() returned %d which is not less than the bitmap pad of %d.", TestName, bunt, pad);
		FAIL;
	} else
		CHECK;

	xi = XGetImage(display, DRW(display), 0,0, 1,1, XAllPlanes(), ZPixmap);

	if(xi->bitmap_unit != bunt) {
		report("%s() returned %d instead of %d.", TestName, bunt, xi->bitmap_unit);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	XFree((char *) xi);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
