/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: lstpxmpfrm.m,v 1.9 92/06/11 16:00:19 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XPixmapFormatValues *
 *   XListPixmapFormats(display, count_return)
 *   Display	*display;
 *   int	*count_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XListPixmapFormats	1
char    *TestName = "XListPixmapFormats";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XListPixmapFormats function
 */
static Display	*display;
static int	*count_return;


#include <string.h>
#define SEPS " "

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	count_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XListPixmapFormats-1.(A)
 * A call to XListPixmapFormats returns an array of
 * XPixampFormatValues structures which can be freed with XFree
 * of length returned in the count_return argument that
 * describe the types of Z format images that are supported by
 * the display specified by the display argument.
 */
static void
t001()
{
XPixmapFormatValues	*plst = (XPixmapFormatValues *) NULL;
XPixmapFormatValues	*pt = (XPixmapFormatValues *) NULL;
char			*tok;
char			*dp;
int			*Pdepth;
int			Ndepth;
int			nr=0;
int			i;
int			j;
int			found;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	count_return = &nr;
	startcall(display);
	if (isdeleted())
		return;
	pt = plst = XListPixmapFormats(display, count_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(plst == (XPixmapFormatValues *) NULL) {
		delete("%s() returned NULL", TestName);
		return;
	} else
		CHECK;

	dp = config.pixmap_depths;
	if (dp == (char*)0) {
		delete("XT_PIXMAP_DEPTHS not set");
		return;
	} else
		CHECK;

	/*
	 * The number of pixmap depths in the string is less than the
	 * length of the string.
	 */
	Pdepth = (int*)malloc(strlen(dp) * sizeof(int));
	if (Pdepth == (int*)0) {
		delete("Could not allocate memory for pixmap depth array");
		return;
	} else
		CHECK;

	Ndepth = 0;
	for (tok = strtok(dp, SEPS); tok; tok = strtok((char*)0, SEPS))
		Pdepth[Ndepth++]  = atov(tok);

	if(nr != Ndepth) {
		report("%s() returned %d structures", TestName, nr);
		report("Expected %d structures", Ndepth);
		FAIL;
	} else
		CHECK;

	for(j = 0; j < Ndepth; j++) {
		trace("pixmap depth expected %d", Pdepth[j]);
		found = 0;
		for(pt = plst, i = 0; i < nr; i++, pt++) {
			trace("pixmap depth found %d", pt->depth);
			if(pt->depth == Pdepth[j]) {
				found = 1;
				break;
			}
		}
		if(!found) {
			report("Pixmap depth %d not returned by %s", Pdepth[j],
								TestName);
			FAIL;
		} else
			CHECK;
	}

	for(pt = plst, i = 0; i < nr; pt++) {
		trace("Pixmap format %d : depth %d, bits_per_pixel %d, scanline_pad %d.", ++i,
			pt->depth, pt-> bits_per_pixel, pt->scanline_pad);
	}
	XFree( (char *) plst);
	CHECKPASS(4+Ndepth);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListPixmapFormats-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When insufficient memory is available, then a call to
 * XListPixmapFormats returns NULL.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
