/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: opndsply.m,v 1.16 92/06/11 16:00:58 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Display *
 *   XOpenDisplay(display_name)
 *   char	*display_name;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XOpenDisplay	1
char    *TestName = "XOpenDisplay";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XOpenDisplay function
 */
static char	*display_name;


#include	"ctype.h"
#include	"Xatom.h"
static char *copystring (src, len) /* Courtesy of Xlib. */
    char *src;
    int len;
{
    char *dst = (char *) malloc (len + 1);

    if (dst) {
	strncpy (dst, src, len);
	dst[len] = '\0';
    }

    return dst;
}


extern char **environ;

static void
t005exec()
{
char	*argv[2];
char	*str;
char	*mstr = "DISPLAY=%s";

	if((str = (char *) malloc( strlen(config.display) + strlen(mstr) - 1)) == (char *) NULL) {
		delete("malloc() failed.");
		return;
	}

	sprintf(str, mstr, config.display);

	argv[0] = "./Test1";
	argv[1] = (char *) NULL;

	if (xtest_putenv(str)) {
		delete("xtest_putenv failed");
		return;
	}

	(void) tet_exec("./Test1", argv, environ);

	delete("Exec of file ./Test1 failed");
	free( (char *) str);
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display_name = config.display;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XOpenDisplay-1.(C)
 * If the system is POSIX compliant and supports DECnet: When
 * the display_name argument is a string of the form
 * hostname::number.screen_num, then a call to XOpenDisplay
 * returns a pointer to a display structure and opens a
 * connection to display server number number on hostname with
 * default screen screen_number.
 */
static void
t001()
{
Display	*display;
char	*cptr, *sptr, *dnop, *snop;
int	dno;
int	sno;
char	ssno[9], sdno[9];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0) {
		unsupported("This assertion can only be tested on a POSIX system.");
		return;
	} else
		CHECK;

	if(config.decnet == 0) {
		unsupported("This assertion can only be tested on a system with support for DECnet.");
		return;
	} else
		CHECK;

	for(cptr = config.display; (*cptr) && (*cptr != ':'); cptr++)
		;

	if(*cptr == '\0') {
		delete("XT_DISPLAY does not specify a display.");
		return;
	} else {
		CHECK;
		if(*++cptr != ':') {
			delete("XT_DISPLAY does not contain a valid display name.");
			return;
		} else
			CHECK;

		for(sptr = ++cptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
			;
		if( (sptr == cptr) || (*sptr != '\0' && *sptr != '.') ) {
			delete("Bad display number in XT_DISPLAY.");
			return;
		} else
			CHECK;

		if( (dnop = copystring(cptr, sptr - cptr)) == (char *) 0) {
			delete("malloc failed,");
			return;
		} else
			CHECK;

		dno = atoi(dnop);

		if(*sptr) {
			for(cptr = ++sptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
				;
			if(*sptr) {
				delete("Bad screen number in XT_DISPLAY.");
				return;
			} else
				CHECK;

			if((snop = copystring(cptr, sptr - cptr)) == (char *) NULL) {
				delete("malloc failed.");
				return;
			} else
				CHECK;

			sno = atoi(snop);
			free(snop);

		} else {
			CHECK; CHECK;
			sno = 0; /* No screen number in XT_DISPLAY, assume zero. */
		}
	}

	sprintf(ssno, "%d", sno);
	sprintf(sdno, "%d", dno);

	if((display_name = (char *) malloc(1 + 3 + strlen(ssno) + strlen(sdno) + strlen(config.displayhost))) == (char *) 0) {
		delete("malloc failed.");
		return;
	} else
		CHECK;

	sprintf(display_name, "%s::%s.%s", config.displayhost, sdno, ssno);

	display = XOpenDisplay(display_name);
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else {
		CHECK;
		XNoOp(display);
		XFlush(display);
		XCloseDisplay(display);
	}

	free(display_name);
	CHECKPASS(10);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-2.(C)
 * If the system is POSIX compliant and supports DECnet: When
 * the display_name argument is a string of the form
 * hostname::number, then a call to XOpenDisplay returns a
 * pointer to a display structure and opens a connection to
 * display server number number on hostname with default screen
 * 0.
 */
static void
t002()
{
Display	*display;
char	*cptr, *sptr, *dnop, *snop;
int	rsno;
int	dno;
char	sdno[9];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0) {
		unsupported("This assertion can only be tested on a POSIX system.");
		return;
	} else
		CHECK;

	if(config.decnet == 0) {
		unsupported("This assertion can only be tested on a system with support for DECnet.");
		return;
	} else
		CHECK;

	for(cptr = config.display; (*cptr) && (*cptr != ':'); cptr++)
		;

	if(*cptr == '\0') {
		delete("XT_DISPLAY does not specify a display.");
		return;
	} else {
		CHECK;
		if(*++cptr != ':') {
			delete("XT_DISPLAY does not contain a valid display name.");
			return;
		} else
			CHECK;

		for(sptr = ++cptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
			;
		if( (sptr == cptr) || (*sptr != '\0' && *sptr != '.') ) {
			delete("Bad display number in XT_DISPLAY.");
			return;
		} else
			CHECK;

		if( (dnop = copystring(cptr, sptr - cptr)) == (char *) 0) {
			delete("malloc failed,");
			return;
		} else
			CHECK;

		dno = atoi(dnop);
		free(dnop);

		if(*sptr) {
			for(cptr = ++sptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
				;
			if(*sptr) {
				delete("Bad screen number in XT_DISPLAY.");
				return;
			} else
				CHECK;

			if((snop = copystring(cptr, sptr - cptr)) == (char *) NULL) {
				delete("malloc failed.");
				return;
			} else
				CHECK;

		} else {
			CHECK; CHECK;
		}
	}

	sprintf(sdno, "%d", dno);

	if((display_name = (char *) malloc(1 + 2 + strlen(sdno) + strlen(config.displayhost))) == (char *) 0) {
		delete("malloc failed.");
		return;
	} else
		CHECK;

	sprintf(display_name, "%s::%s", config.displayhost, sdno);
	free(snop);
	free(dnop);

	display = XOpenDisplay(display_name);
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else {
		CHECK;
		if( (rsno=XDefaultScreen(display)) != 0 ) {
			report("%s() with argument \"%s\" opened screen %d instead of screen 0.", TestName, display_name, rsno);
			FAIL;
		} else
			CHECK;

		XNoOp(display);
		XFlush(display);
		XCloseDisplay(display);
	}

	free(display_name);
	CHECKPASS(11);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-3.(C)
 * If the system is POSIX compliant and supports TCP: When the
 * display_name argument is a string of the form
 * hostname:number.screen_num, then a call to XOpenDisplay
 * returns a pointer to a display structure and opens a
 * connection to display server number number on hostname with
 * default screen screen_number.
 */
static void
t003()
{
Display	*display;
char	*cptr, *sptr, *dnop, *snop;
int	dno;
int	sno;
char	ssno[9], sdno[9];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0) {
		unsupported("This assertion can only be tested on a POSIX system.");
		return;
	} else
		CHECK;

	if(config.tcp == 0) {
		unsupported("This assertion can only be tested on a system with support for TCP.");
		return;
	} else
		CHECK;

	for(cptr = config.display; (*cptr) && (*cptr != ':'); cptr++)
		;

	if(*cptr == '\0') {
		delete("XT_DISPLAY does not specify a display.");
		return;
	} else {
		CHECK;
		for(sptr = ++cptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
			;
		if( (sptr == cptr) || (*sptr != '\0' && *sptr != '.') ) {
			delete("Bad display number in XT_DISPLAY.");
			return;
		} else
			CHECK;

		if( (dnop = copystring(cptr, sptr - cptr)) == (char *) 0) {
			delete("malloc failed,");
			return;
		} else
			CHECK;

		dno = atoi(dnop);
		free(dnop);

		if(*sptr) {
			for(cptr = ++sptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
				;
			if(*sptr) {
				delete("Bad screen number in XT_DISPLAY.");
				return;
			} else
				CHECK;

			if((snop = copystring(cptr, sptr - cptr)) == (char *) NULL) {
				delete("malloc failed.");
				return;
			} else
				CHECK;

			sno = atoi(snop);
			free(snop);

		} else {
			CHECK; CHECK;
			sno = 0; /* No screen number in XT_DISPLAY, assume zero. */
		}
	}

	sprintf(ssno, "%d", sno);
	sprintf(sdno, "%d", dno);

	if((display_name = (char *) malloc(1 + 2 + strlen(ssno) + strlen(sdno) + strlen(config.displayhost))) == (char *) 0) {
		delete("malloc failed.");
		return;
	} else
		CHECK;

	sprintf(display_name, "%s:%s.%s", config.displayhost, sdno, ssno);

	display = XOpenDisplay(display_name);
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else {
		CHECK;
		XNoOp(display);
		XFlush(display);
		XCloseDisplay(display);
	}

	free(display_name);
	CHECKPASS(9);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-4.(C)
 * If the system is POSIX compliant and supports TCP: When the
 * display_name argument is a string of the form
 * hostname:number, then a call to XOpenDisplay returns a
 * pointer to a display structure and opens a connection to
 * display server number number on hostname with default screen
 * 0.
 */
static void
t004()
{
Display	*display;
char	*cptr, *sptr, *dnop, *snop;
int	rsno;
int	dno;
char	sdno[9];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0) {
		unsupported("This assertion can only be tested on a POSIX system.");
		return;
	} else
		CHECK;

	if(config.tcp == 0) {
		unsupported("This assertion can only be tested on a system with support for TCP.");
		return;
	} else
		CHECK;

	for(cptr = config.display; (*cptr) && (*cptr != ':'); cptr++)
		;

	if(*cptr == '\0') {
		delete("XT_DISPLAY does not specify a display.");
		return;
	} else {
		CHECK;

		for(sptr = ++cptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
			;
		if( (sptr == cptr) || (*sptr != '\0' && *sptr != '.') ) {
			delete("Bad display number in XT_DISPLAY.");
			return;
		} else
			CHECK;

		if( (dnop = copystring(cptr, sptr - cptr)) == (char *) 0) {
			delete("malloc failed,");
			return;
		} else
			CHECK;

		dno = atoi(dnop);
		free(dnop);

		if(*sptr) {
			for(cptr = ++sptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
				;
			if(*sptr) {
				delete("Bad screen number in XT_DISPLAY.");
				return;
			} else
				CHECK;

			if((snop = copystring(cptr, sptr - cptr)) == (char *) NULL) {
				delete("malloc failed.");
				return;
			} else
				CHECK;

			free(snop);

		} else {
			CHECK; CHECK;
		}
	}

	sprintf(sdno, "%d", dno);

	if((display_name = (char *) malloc(1 + 1 + strlen(sdno) + strlen(config.displayhost))) == (char *) 0) {
		delete("malloc failed.");
		return;
	} else
		CHECK;

	sprintf(display_name, "%s:%s", config.displayhost, sdno);

	display = XOpenDisplay(display_name);
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else {
		CHECK;
		if( (rsno=XDefaultScreen(display)) != 0 ) {
			report("%s() with argument \"%s\" opened screen %d instead of screen 0.", TestName, display_name, rsno);
			FAIL;
		} else
			CHECK;

		XNoOp(display);
		XFlush(display);
		XCloseDisplay(display);
	}

	free(display_name);
	CHECKPASS(10);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-5.(C)
 * If the system is POSIX compliant: When the display_name
 * argument is NULL, then a call to XOpenDisplay opens a
 * connection specified by the value of the DISPLAY environment
 * variable.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0) {
		unsupported("This assertion can only be tested on a POSIX system.");
	} else
		(void) tet_fork(t005exec, TET_NULLFP, 0 , ~0);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-6.(D)
 *   Reason: There is no known portable test method for this assertion
 * If the system is POSIX compliant and supports DECnet and a
 * local display server: When the display_name argument is a
 * string of the form ::number.screen_num or ::number, then a
 * call to XOpenDisplay opens the most efficient transport
 * connection available to the specified display server and
 * default screen on the client machine.
 */
static void
t006()
{
Display	*display;
char	*cptr, *sptr, *dnop, *snop;
int	dno;
int	sno;
char	ssno[9], sdno[9];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0) {
		unsupported("This assertion can only be tested on a POSIX system.");
		return;
	} else
		CHECK;

	if(config.decnet == 0) {
		unsupported("This assertion can only be tested on a system with support for DECnet.");
		return;
	} else
		CHECK;

	if(config.local == 0) {
		unsupported("This assertion can only be tested on a system with a local display server.");
		return;
	} else
		CHECK;

	for(cptr = config.display; (*cptr) && (*cptr != ':'); cptr++)
		;

	if(*cptr == '\0') {
		delete("XT_DISPLAY does not specify a display.");
		return;
	} else {
		CHECK;
		if(*++cptr != ':') {
			delete("XT_DISPLAY does not contain a valid display name.");
			return;
		} else
			CHECK;

		for(sptr = ++cptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
			;
		if( (sptr == cptr) || (*sptr != '\0' && *sptr != '.') ) {
			delete("Bad display number in XT_DISPLAY.");
			return;
		} else
			CHECK;

		if( (dnop = copystring(cptr, sptr - cptr)) == (char *) 0) {
			delete("malloc failed,");
			return;
		} else
			CHECK;

		dno = atoi(dnop);
		free(dnop);

		if(*sptr) {
			for(cptr = ++sptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
				;
			if(*sptr) {
				delete("Bad screen number in XT_DISPLAY.");
				return;
			} else
				CHECK;

			if((snop = copystring(cptr, sptr - cptr)) == (char *) NULL) {
				delete("malloc failed.");
				return;
			} else
				CHECK;

			sno = atoi(snop);
			free(snop);

		} else {
			CHECK; CHECK;
			sno = 0; /* No screen number in XT_DISPLAY, assume zero. */
		}
	}

	sprintf(ssno, "%d", sno);
	sprintf(sdno, "%d", dno);

	if((display_name = (char *) malloc(1 + 3 + strlen(ssno) + strlen(sdno))) == (char *) 0) {
		delete("malloc failed.");
		return;
	} else
		CHECK;

	sprintf(display_name, "::%s.%s", sdno, ssno);

	display = XOpenDisplay(display_name);
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else {
		CHECK;
		XNoOp(display);
		XFlush(display);
		XCloseDisplay(display);
	}

	free(display_name);
/* added :: */

	if((display_name = (char *) malloc(1 + 2 + strlen(sdno))) == (char *) 0) {
		delete("malloc failed.");
		return;
	} else
		CHECK;

	sprintf(display_name, "::%s", sdno);

	display = XOpenDisplay(display_name);
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else {
		CHECK;
		XNoOp(display);
		XFlush(display);
		XCloseDisplay(display);
	}

	free(display_name);

	CHECKUNTESTED(13);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-7.(D)
 *   Reason: There is no known portable test method for this assertion
 * If the system is POSIX compliant and supports TCP and a
 * local display server: When the display_name argument is a
 * string of the form :number.screen_num or :number, then a
 * call to XOpenDisplay opens the most efficient transport
 * connection available to the specified display server and
 * default screen on the client machine.
 */
static void
t007()
{
Display	*display;
char	*cptr, *sptr, *dnop, *snop;
int	dno;
int	sno;
char	ssno[9], sdno[9];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0) {
		unsupported("This assertion can only be tested on a POSIX system.");
		return;
	} else
		CHECK;

	if(config.tcp == 0) {
		unsupported("This assertion can only be tested on a system with support for TCP.");
		return;
	} else
		CHECK;

	if(config.local == 0) {
		unsupported("This assertion can only be tested on a system with a local display server.");
		return;
	} else
		CHECK;

	for(cptr = config.display; (*cptr) && (*cptr != ':'); cptr++)
		;

	if(*cptr == '\0') {
		delete("XT_DISPLAY does not specify a display.");
		return;
	} else {
		CHECK;

		for(sptr = ++cptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
			;
		if( (sptr == cptr) || (*sptr != '\0' && *sptr != '.') ) {
			delete("Bad display number in XT_DISPLAY.");
			return;
		} else
			CHECK;

		if( (dnop = copystring(cptr, sptr - cptr)) == (char *) 0) {
			delete("malloc failed,");
			return;
		} else
			CHECK;

		dno = atoi(dnop);
		free(dnop);

		if(*sptr) {
			for(cptr = ++sptr; *sptr && isascii(*sptr) && isdigit(*sptr); sptr++)
				;
			if(*sptr) {
				delete("Bad screen number in XT_DISPLAY.");
				return;
			} else
				CHECK;

			if((snop = copystring(cptr, sptr - cptr)) == (char *) NULL) {
				delete("malloc failed.");
				return;
			} else
				CHECK;

			sno = atoi(snop);
			free(snop);

		} else {
			CHECK; CHECK;
			sno = 0; /* No screen number in XT_DISPLAY, assume zero. */
		}
	}

	sprintf(ssno, "%d", sno);
	sprintf(sdno, "%d", dno);

	if((display_name = (char *) malloc(1 + 2 + strlen(ssno) + strlen(sdno))) == (char *) 0) {
		delete("malloc failed.");
		return;
	} else
		CHECK;

	sprintf(display_name, ":%s.%s", sdno, ssno);

	display = XOpenDisplay(display_name);
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else {
		CHECK;
		XNoOp(display);
		XFlush(display);
		XCloseDisplay(display);
	}

	free(display_name);
/* added : */

	if((display_name = (char *) malloc(1 + 1 + strlen(sdno))) == (char *) 0) {
		delete("malloc failed.");
		return;
	} else
		CHECK;

	sprintf(display_name, ":%s", sdno);

	display = XOpenDisplay(display_name);
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else {
		CHECK;
		XNoOp(display);
		XFlush(display);
		XCloseDisplay(display);
	}

	free(display_name);

	CHECKUNTESTED(12);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-8.(A)
 * When a call to XOpenDisplay is successful, then all of the
 * screens in the display can be used by the client.
 */
static void
t008()
{
int			scount;
int			i;
Window			aroot;
XWindowAttributes	atts;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	scount = ScreenCount(Dsp);

	for(i=0; i< scount; i++) {

		aroot = RootWindow(Dsp, i);
	
		if(XGetWindowAttributes(Dsp, aroot, &atts) == 0) {
			report("Unable to access the attributes of the root window of screen %d.", i);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(scount);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-9.(A)
 * On a call to XOpenDisplay the RESOURCE_MANAGER property of
 * the root window of screen 0 is read and stored.
 */
static void
t009()
{
Window	root0;
unsigned char	*rmstr;
char	*rstr;
Display	*display;
Atom	type_ret;
int	format_ret;
unsigned long	nitems_ret;
unsigned long	bytes_after_ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	root0 = XRootWindow(Dsp, 0);

	if(XGetWindowProperty(Dsp, root0, XA_RESOURCE_MANAGER, 0L, -1L, False, 
			XA_STRING, &type_ret, &format_ret, &nitems_ret,
				&bytes_after_ret, &rmstr) != Success) {
		delete("XGetWindowProperty did not return Success");
		return;
	}
	if(type_ret != XA_STRING || format_ret != 8 ||
	  nitems_ret == 0 || rmstr == (unsigned char *)NULL) {
		trace("nitems was %ld", nitems_ret);
		trace("type was  %d", type_ret);
		trace("format was  %d", format_ret);
		rmstr = (unsigned char *)NULL;
	}
	trace("Config.display is %s", config.display);
	display = XOpenDisplay(config.display);
	
	if(display == (Display *) NULL) {
		report("%s() returned NULL with argument %s.", TestName, config.display);
		FAIL;
		return;
	} else 
		CHECK;

	rstr = XResourceManagerString(display);

	if(rmstr == (unsigned char *)NULL) {
		if(rstr != (char *)NULL && *rstr != '\0') {
			report("After calling %s", TestName);
			report("XResourceManagerString returned a value :");
			report("  \"%s\",", rstr);
			report("although the RESOURCE_MANAGER property of the");
			report("root window of screen zero was unset.");
			FAIL;
		} else
			CHECK;
	} else {
		if((rstr == (char *) NULL) || (strcmp(rstr, rmstr) != 0)) {
			report("After calling %s", TestName);
			report("XResourceManagerString returned a value :");
			report("  \"%s\",", rstr == (char *) NULL ? "<NULL_POINTER>" : rstr);
			report("instead of:");
			report("  \"%s\".", rmstr);
			FAIL;
		} else
			CHECK;
	}

	XCloseDisplay(display);

	CHECKPASS(2);	

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XOpenDisplay-10.(A)
 * When a call to XOpenDisplay is not successful, then it
 * returns NULL.
 */
static void
t010()
{
Display *display;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	display_name = config.displayhost;
	startcall(Dsp);
	if (isdeleted())
		return;
	display = XOpenDisplay(display_name);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(display != (Display *) NULL) {
		report("%s() did not return NULL with argument \"%s\".", TestName, display_name);
		FAIL;
	} else
		PASS;		
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
