/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: vndrrls.m,v 1.6 92/06/11 16:02:08 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char *
 *   XVendorRelease(display)
 *   Display	*display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XVendorRelease	1
char    *TestName = "XVendorRelease";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XVendorRelease function
 */
static Display	*display;


#include <string.h>
/*
 * extract fields from version
 */
#define GET_MAJOR(ver)	((ver) / 10000000)
#define GET_MINOR(ver)	(((ver) % 10000000) / 100000)
#define GET_PATCH(ver)	(((ver) % 100000) / 1000)
#define GET_SNAP(ver)	(((ver) % 1000))
#define EXTRACT_VERSION(name, ver) \
			ver_##name##_major = GET_MAJOR(ver); \
			ver_##name##_minor = GET_MINOR(ver); \
			ver_##name##_patch = GET_PATCH(ver); \
			ver_##name##_snap = GET_SNAP(ver)
#define SEPS	":"


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XVendorRelease-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XVendorRelease returns a number related to a
 * vendor's release of the X server.
 */
static void
t001()
{
int	vr, cfgvr;
int	ver_ret_major, ver_ret_minor, ver_ret_patch, ver_ret_snap;
int	ver_cfg_major, ver_cfg_minor, ver_cfg_patch, ver_cfg_snap;
Bool	matched = False;
char	*Prelease;
char	*tok;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	vr = XVendorRelease(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* convert versions to fields  and then check with config.vendor_lease */
	EXTRACT_VERSION(ret, vr);

	Prelease = (char *)malloc(strlen(config.vendor_release) * sizeof(char));
	if (Prelease == (char *)NULL) {
		delete("Could not allocate memory for vendor release string");
		return;
	} else
		CHECK;

	strcpy(Prelease, config.vendor_release);
	while ((tok = strsep(&Prelease, SEPS)) != NULL) {
		cfgvr = atov(tok);
		EXTRACT_VERSION(cfg, cfgvr);

		if ((ver_ret_major == ver_cfg_major) &&
		    ((ver_ret_minor >= ver_cfg_minor) ||
		     (ver_ret_patch >= ver_cfg_patch) ||
		     (ver_ret_snap >= ver_cfg_snap))) {
			matched = True;
			break;
		} 
	}

	if (matched == False) {
		report("%s() returned %d instead one of %s.", TestName, vr, config.vendor_release);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
