/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: trnsltcrdn.m,v 1.7 92/06/11 16:11:39 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Bool
 *   XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, dest_y_return, child_return)
 *   Display	*display;
 *   Window	src_w;
 *   Window	dest_w;
 *   int	src_x;
 *   int	src_y;
 *   int	*dest_x_return	;
 *   int	*dest_y_return	;
 *   Window	*child_return	;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XTranslateCoordinates	1
char    *TestName = "XTranslateCoordinates";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW src_w
#define A_DRAWABLE src_w
#define A_WINDOW2 dest_w
#define A_DRAWABLE2 dest_w


/*
 * Arguments to the XTranslateCoordinates function
 */
static Display	*display;
static Window	src_w;
static Window	dest_w;
static int	src_x;
static int	src_y;
static int	*dest_x_return	;
static int	*dest_y_return	;
static Window	*child_return	;


static	struct	area	c_ap;
static	int	dest_x;
static	int	dest_y;
static	Window	child;

void transinit()
{
	tpstartup();

/* struct area of the child window */
	c_ap.x = 50;
	c_ap.y = 60;
	c_ap.width = 20;
	c_ap.height= 17;

	dest_x = -1;
	dest_y = -1;
	child  = ~0;
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	src_w = 0;
	dest_w = 0;
	src_x = c_ap.x + 10;
	src_y = c_ap.y + 8;
	dest_x_return = &dest_x;
	dest_y_return = &dest_y;
	child_return = &child;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	src_w = ErrdefWindow;
	dest_w = ErrdefWindow;
}

/*
 * Assertion XTranslateCoordinates-1.(A)
 * A call to XTranslateCoordinates takes the src_x and src_y
 * coordinates relative to the source window's origin and
 * converts them to be relative to the destination window's
 * origin and returns these translated coordinates in
 * dest_x_return and dest_y_return.
 */
static void
t001()
{
Bool	ret;
int	exp_x, exp_y;
int	border_width;
int 	pass = 0, fail = 0;

	transinit();
	setargs();

/* Set up child window position coordinates */

	for(border_width = 0; border_width < 5; border_width++) {
		src_w = defdraw(display, VI_WIN);
		XMapWindow(display, src_w);

	/* Child window: visual = CopyFromParent, and is mapped */
		dest_w = crechild(display, src_w, &c_ap);
		XSetWindowBorderWidth(display, dest_w, border_width);
		XSetWindowBorder(display, dest_w, W_FG);
		XMapWindow(display, dest_w);

/* Calculate expected return values */
		exp_x = src_x - c_ap.x - border_width;
		exp_y = src_y - c_ap.y - border_width;

		startcall(display);
		if (isdeleted())
			return;
		ret = XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, dest_y_return, child_return);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (ret == False)
		{
			report("%s returned False when expecting True.",
				TestName);
			report("src_x=%d", src_x); delete("src_y=%d", src_y);
			report("exp_x=%d", exp_x); delete("exp_y=%d", exp_y);
			report("border_width=%d", border_width);
			FAIL;
		} else
		if ((dest_x != exp_x) || (dest_y != exp_y)) {
			report("%s did not return expected coordinates",
				TestName);
			report("Expected: (*dest_x_return)=%d, (*dest_y_return)=%d",
				exp_x, exp_y);
			report("Got: (*dest_x_return)=%d, (*dest_y_return)=%d",
				dest_x, dest_y);
			if( (dest_x == -1) || (dest_y == -1) )
				report("(*dest_x_return) and (*dest_y_return) were probably not set");

			FAIL;
		} else
			CHECK;

	}

	CHECKPASS(5);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTranslateCoordinates-2.(C)
 * When src_w and dest_w were not created on the same screen,
 * then a call to XTranslateCoordinates returns zero and
 * dest_x_return and dest_y_return are set to zero.
 */
static void
t002()
{
int	ret;
int 	pass = 0, fail = 0;

	transinit();
	setargs();

        if (config.alt_screen == -1) {
                unsupported("No alternate screen supported");
                return;
        }

	src_w  = defdraw(display, VI_WIN);
	XMapWindow(display, src_w);
	dest_w = defdraw(display, VI_ALT_WIN);
	XMapWindow(display, dest_w);

	startcall(display);
	if (isdeleted())
		return;
	ret = XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, dest_y_return, child_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ret != 0)
	{
		report("%s did not return zero when", TestName);
		report("src_w and dest_w were on different screens.");
		report("Expecting: 0");
		report("Returned : %d", ret);
		FAIL;
	}
	else
		CHECK;

	if ( dest_x != 0  || dest_y != 0 )
	{
		report("%s did not set the dest_x and dest_y as expected.",
				TestName);
		report("Expecting: (*dest_x_return)=0, (*dest_y_return)=0");
		report("Returned : (*dest_x_return)=%d, (*dest_y_return)=%d",
			dest_x, dest_y);
		FAIL;
	}
	else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTranslateCoordinates-3.(A)
 * When the coordinates are contained in a mapped child of
 * dest_w, then that child is returned in child_return.
 */
static void
t003()
{
Window	exp_w;
Bool	ret;
int	border_width;
int 	pass = 0, fail = 0;

	transinit();
	setargs();

/* Set up child window position coordinates */

	for(border_width = 0; border_width < 5; border_width++) {
		src_w = defdraw(display, VI_WIN);
		XMapWindow(display, src_w);

		dest_w = src_w;

		exp_w = crechild(display, src_w, &c_ap);

		XSetWindowBorderWidth(display, exp_w, border_width); 
		XSetWindowBorder(display, exp_w, W_FG);
		XMapWindow(display, exp_w);

/* Calculate expected return values */
		startcall(display);
		if (isdeleted())
			return;
		ret = XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, dest_y_return, child_return);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (ret == False)
		{
			report("%s returned False when expecting True.",
				TestName);
			report("border_width=%d", border_width);
			FAIL;
		} else
		if (child != exp_w)
		{
			report("%s did not return the expected window in (*child_return)",
				TestName);
			report("(*child_return) = %0x", child);
			report("expected window = %0x", exp_w);
			report("border_width=%d", border_width);
			FAIL;
		}
		else
			CHECK;

	}

	CHECKPASS(5);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTranslateCoordinates-4.(A)
 * When the coordinates are not contained in a mapped child of
 * dest_w, then child_return is set to None.
 */
static void
t004()
{
XVisualInfo 	*vp;
XWindowAttributes	winatt;
Window	exp_w;
Bool	ret;
int 	pass = 0, fail = 0;

	transinit();
	setargs();

	src_w = defdraw(display, VI_WIN);
	XMapWindow(display, src_w);
	dest_w = src_w;

/* Get the root window relative postion of the first window */
	XGetWindowAttributes(display, src_w, &winatt);

/* Set the coordinates of the sibling window */
	c_ap.x += winatt.x;
	c_ap.y += winatt.y;

/* Make sibling window: mapped */
	resetvinf(VI_WIN); nextvinf(&vp);
	exp_w = makewinpos(display, vp,  c_ap.x, c_ap.y);

	startcall(display);
	if (isdeleted())
		return;
	ret = XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, dest_y_return, child_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ret == False)
	{
		report("%s returned False when expecting True.",
			TestName);
		FAIL;
	} else
	if (child != None)
	{
		report("When the source coordinates refered to a sibling window");
		report("%s did not set child_return to None",
			TestName);
		report("Window id returned: %0x", child);
		report("Source/Dest window: %0x", src_w);
		report("Sibling window    : %0x", exp_w);
		FAIL;
	}
	else
		CHECK;

/* Now test for a point on dest_w itself */
	src_x = 5;
	src_y = 5;
	child = ~0;	/* We must make sure xname actually sets this */

	startcall(display);
	if (isdeleted())
		return;
	ret = XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, dest_y_return, child_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ret == False)
	{
		report("%s returned False when expecting True.",
			TestName);
		FAIL;
	} else
	if (child != None)
	{
		report("When the source coordinates refered to the dest_w");
		report("%s did not set child_return to None",
			TestName);
		report("child_return      : %0x", child);
		report("Source/Dest window: %0x", src_w);
		report("Sibling window    : %0x", exp_w);
		FAIL;
	}
	else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTranslateCoordinates-5.(A)
 * When a window argument does not name a valid window, then a
 * BadWindow error occurs.
 */
static void
t005()
{
Window	goodwin;
int 	pass = 0, fail = 0;

	transinit();
	setargs();

	seterrdef();
	
	goodwin = defdraw(display, VI_WIN);

	src_w	= badwin(display);
	dest_w	= goodwin;
	startcall(display);
	if (isdeleted())
		return;
	XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, dest_y_return, child_return);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadWindow)
		CHECK;
	else
		FAIL;

	src_w	= goodwin;
	dest_w	= badwin(display);
	startcall(display);
	if (isdeleted())
		return;
	XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, dest_y_return, child_return);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadWindow)
		CHECK;
	else
		FAIL;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
