/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: unmpwdw.m,v 1.6 92/06/11 16:11:54 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XUnmapWindow(display, w)
 *   Display	*display;
 *   Window	w;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XUnmapWindow	1
char    *TestName = "XUnmapWindow";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XUnmapWindow function
 */
static Display	*display;
static Window	w;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
}

/*
 * Assertion XUnmapWindow-1.(A)
 * A call to XUnmapWindow unmaps the specified window.
 */
static void
t001()
{
Window	base;
struct	area	area;
XWindowAttributes	atts;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	setarea(&area, 10, 10, 50, 50);
	w = crechild(display, base, &area);
	pattern(display, w);

	startcall(display);
	if (isdeleted())
		return;
	XUnmapWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetWindowAttributes(display, w, &atts);
	if (atts.map_state != IsUnmapped) {
		report("After unmap map-state was %s, expecting IsUnmapped",
			mapstatename(atts.map_state));
		FAIL;
	} else
		CHECK;

	if (checkclear(display, base))
		CHECK;
	else {
		report("Window did not disappear after unmapping");
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUnmapWindow-2.(A)
 * When the specified window is mapped, then a call to
 * XUnmapWindow generates an UnmapNotify event.
 */
static void
t002()
{
Window	base;
struct	area	area;
XWindowAttributes	atts;
XEvent	ev;
XUnmapEvent	*ump;
XUnmapEvent	good;
int 	got_parent = 0;
int 	got_window = 0;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	setarea(&area, 10, 10, 50, 50);
	w = crechild(display, base, &area);
	pattern(display, w);

	XSelectInput(display, base, SubstructureNotifyMask);
	XSelectInput(display, w, StructureNotifyMask);

	startcall(display);
	if (isdeleted())
		return;
	XUnmapWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSelectInput(display, base, NoEventMask);
	XSelectInput(display, w, NoEventMask);

	n = XPending(display);
	if (n != 2) {
		report("Wrong number of events received got %d, expecting 2", n);
		FAIL;
	} else
		CHECK;

	got_window = 0;
	got_parent = 0;
	while (getevent(display, &ev)) {
		if (ev.type != UnmapNotify) {
			report("Event of type %s was received unexpectedly", eventname(ev.type));
			FAIL;
			continue;
		} else
			CHECK;

		ump = (XUnmapEvent*)&ev;

		if (ump->event == ump->window) {
			got_window++;
		} else if (ump->event != base) {
			report("Event received on a window other than the parent");
			FAIL;
		} else {
			got_parent++;
		}
		good.type = UnmapNotify;
		good.serial = 0L;
		good.send_event = False;
		good.display = display;
		good.event = ump->event;
		good.window = w;
		good.from_configure = False;

		if (checkevent((XEvent*)&good, &ev))
			FAIL;
		else
			CHECK;
	}

	if (got_window > 1) {
		report("Repeat unmap notify event on window");
		FAIL;
	} else if (got_window == 0) {
		report("UnmapNotify event was not received on window");
		FAIL;
	} else
		CHECK;

	if (got_parent > 1) {
		report("Repeat unmap notify event on parent window");
		FAIL;
	} else if (got_parent == 0) {
		report("UnmapNotify event was not received on parent window");
		FAIL;
	} else
		CHECK;

	XGetWindowAttributes(display, w, &atts);
	if (atts.map_state != IsUnmapped) {
		report("After unmap map-state was %s, expecting IsUnmapped",
			mapstatename(atts.map_state));
		FAIL;
	} else
		CHECK;

	if (checkclear(display, base))
		CHECK;
	else {
		report("Window did not disappear after unmapping");
		FAIL;
	}

	CHECKPASS(1+2*2+4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUnmapWindow-3.(A)
 * When the specified window is already unmapped, then a call
 * to XUnmapWindow has no effect.
 */
static void
t003()
{
Window	base;
struct	area	area;
XWindowAttributes	atts;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	setarea(&area, 10, 10, 50, 50);
	w = creunmapchild(display, base, &area);
	pattern(display, w);

	XSelectInput(display, w, StructureNotifyMask);

	startcall(display);
	if (isdeleted())
		return;
	XUnmapWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSelectInput(display, w, NoEventMask);

	n = XPending(display);
	if (n != 0) {
		report("Received event when window already unmapped");
		FAIL;
	} else
		CHECK;

	XGetWindowAttributes(display, w, &atts);
	if (atts.map_state != IsUnmapped) {
		report("After unmap map-state was %s, expecting IsUnmapped",
			mapstatename(atts.map_state));
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUnmapWindow-4.(A)
 * When there is a viewable child of window, then after a
 * call to XUnmapWindow the child window is no longer viewable.
 */
static void
t004()
{
Window	base;
Window	ch;
struct	area	area;
XWindowAttributes	atts;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	setarea(&area, 10, 10, 50, 50);
	w = crechild(display, base, &area);
	pattern(display, w);

	setarea(&area, 5, 5, 10, 10);
	ch = crechild(display, w, &area);

	(void) XGetWindowAttributes(display, ch, &atts);
	if (atts.map_state != IsViewable) {
		delete("Before unmapping parent map-state of child was %s, expecting IsViewable", mapstatename(atts.map_state));
		return;
	} else
		CHECK;

	startcall(display);
	if (isdeleted())
		return;
	XUnmapWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) XGetWindowAttributes(display, ch, &atts);
	if (atts.map_state != IsUnviewable) {
		report("After unmap map-state was %s, expecting IsUnviewable",
			mapstatename(atts.map_state));
		FAIL;
	} else
		CHECK;

	if (checkclear(display, base))
		CHECK;
	else {
		report("Window did not disappear after unmapping");
		FAIL;
	}

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUnmapWindow-5.(A)
 * When a call to XUnmapWindow uncovers part of any window
 * that was formerly obscured, then either Expose events are
 * generated or the contents are restored from backing store.
 */
static void
t005()
{
Window	base;
struct	area	area;
XWindowAttributes	atts;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);

	setarea(&area, 10, 10, 50, 50);
	w = crechild(display, base, &area);
	pattern(display, w);

	setforexpose(display, base);
	XSelectInput(display, base, ExposureMask);

	startcall(display);
	if (isdeleted())
		return;
	XUnmapWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetWindowAttributes(display, w, &atts);
	if (atts.map_state != IsUnmapped) {
		report("After unmap map-state was %s, expecting IsUnmapped",
			mapstatename(atts.map_state));
		FAIL;
	} else
		CHECK;

	if (exposecheck(display, base))
		CHECK;
	else {
		report("Neither Expose events or backing store processing");
		report("could correctly restore the window contents.");
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XUnmapWindow-6.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XUnmapWindow(display, w);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
