/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: cpyclrmpan.m,v 1.11 92/06/11 16:15:11 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Colormap
 *   XCopyColormapAndFree(display, colormap)
 *   Display *display;
 *   Colormap colormap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCopyColormapAndFree	1
char    *TestName = "XCopyColormapAndFree";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XCopyColormapAndFree function
 */
static Display *display;
static Colormap colormap;


static Bool check_rgb(dpy, cmap, xcp)
	Display *dpy;
	Colormap cmap;
	XColor *xcp;
{
	XColor ncol;

	ncol.pixel = xcp->pixel;
	ncol.flags = DoRed | DoGreen | DoBlue;
	XQueryColor(dpy, cmap, &ncol);

	if ( ncol.red != xcp->red || ncol.green != xcp->green || ncol.blue != xcp->blue ) {
			report("mismatch, got rgb = 0x%x,0x%x,0x%x instead of 0x%x,0x%x,0x%x for pixel %lu",
				ncol.red, ncol.green, ncol.blue,
				xcp->red, xcp->green, xcp->blue, xcp->pixel);
			return False;
	} else
		return True;
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XCopyColormapAndFree-1.(A)
 * A call to XCopyColormapAndFree creates a colourmap of the
 * same visual type and for the same screen as the colormap
 * argument and returns the new colourmap ID.
 */
static void
t001()
{
Colormap	testcmap;
XVisualInfo	*vp;
unsigned long 	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
XColor		*cellmap, *cellptr;
XColor		cell;
unsigned long	l;
unsigned int	i;
int		pathcnt = 0, size;
XColor		color, color1, color2, ncol1, ncol2;
Display		*disp2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	color1.pixel = 0xffff;
	color1.red = 0x0ff0;
	color1.green = 0xf0f0;
	color1.blue = 0x0f0f;
	color1.flags = DoRed|DoGreen|DoBlue;

	color2.pixel = 1;
	color2.red = 0xffff;
	color2.green = 0xffff;
	color2.blue = 0xffff;
	color2.flags = DoRed|DoGreen|DoBlue;

	for(resetsupvis(vmask);  nextsupvis(&vp); ) {

		colormap = makecolmap(display, vp->visual, AllocNone);
		size = maxsize(vp);		

		cellmap = (XColor *) malloc(size * sizeof(XColor));

		if (cellmap == (XColor *)NULL) {
			delete("malloc failure");
			return;
		} else
			CHECK;

		*cellmap = color1;

		if( XAllocColor(display, colormap, cellmap) == False ) {
			delete("XAllocColor() failed to allocate a r/o colourcell");
			return;
		}
		XQueryColor(display, colormap, cellmap);

		for(i=1, cellptr = cellmap+1; i < size; i++, cellptr++) {
			if(XAllocColorCells(display, colormap, False, 0, 0L, &(cellptr->pixel), 1) == False ) {
				delete("XAllocColorCells() failed to allocate a r/w colourcell (%d)",i);
				return;
			}
			cellptr->red = 0xffff;
			cellptr->green = 0xf00f;
			cellptr->blue = 0x0ff0;
			cellptr->flags = DoRed|DoGreen|DoBlue;
			XStoreColor(display, colormap, cellptr);
			XQueryColor(display, colormap, cellptr);
		}
		if(i == size)
			CHECK;

		startcall(display);
		if (isdeleted())
			return;
		testcmap = XCopyColormapAndFree(display, colormap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
			
		for(i=0, cellptr = cellmap; i< size; i++, cellptr++) {
			cell.pixel = cellptr->pixel;
			cell.flags = DoRed|DoGreen|DoBlue;
			XQueryColor(display, testcmap, &cell);
			if( (cell.red != cellptr->red) ||
			    (cell.green != cellptr->green) || (cell.blue != cellptr->blue) ){
				report("Copied colourmap cell pixel %ld had r %u g %u b %u instead of pixel %ld r %u g %u b %u",
					cell.pixel, cell.red, cell.green, cell.blue,
					cellptr->pixel, cellptr->red, cellptr->green, cellptr->blue);
				FAIL;
			}

			if(i==size)
				CHECK;
		}
	}
	
	CHECKPASS(2*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCopyColormapAndFree-2.(C)
 * When the colormap argument was created by the client with
 * alloc set to AllocAll, then all entries from the argument
 * colormap are moved to the new colourmap with the same colour
 * values and are freed in colormap.
 */
static void
t002()
{
Colormap	testcmap;
XVisualInfo	*vp;
unsigned long 	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
XColor		*cellmap, *cellptr;
XColor		cell;
unsigned long	l;
int		i;
int		pathcnt = 0;
int 	size;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	for (resetsupvis(vmask);  nextsupvis(&vp); ) {

		colormap = makecolmap(display, vp->visual, AllocAll);
		size = maxsize(vp);
		cellmap = (XColor *) malloc( (size * sizeof(XColor)));
		if(cellmap == (XColor *) 0) {
			delete("malloc() failed to allocate memory for XColor array.");
			return;
		}

/* fill colour map with some recognisable and distinguishable values */
		for(l=0, cellptr=cellmap; l< size; l++, cellptr++) {
			cellptr->pixel = l;
			cellptr->green = cellptr->blue = cellptr->red = l << 8;
			cellptr->flags = DoRed | DoGreen | DoBlue;
		}
		XStoreColors(display, colormap, cellmap, size);

/* server sets to "closest available values" so find what they were */
		XQueryColors(display, colormap, cellmap, size);

		startcall(display);
		if (isdeleted())
			return;
		testcmap = XCopyColormapAndFree(display, colormap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		pathcnt += size;

		for(l=0, cellptr=cellmap; l < size; l++, cellptr++) {
			cell.pixel = l;
			XQueryColor(display, testcmap, &cell);
			if((cell.pixel != cellptr->pixel) || (cell.red != cellptr->red) ||
			   (cell.green != cellptr->green) || (cell.blue != cellptr->blue)) { 
				report("Cell %lu had pixelvalue %lu (r %u g %u b %u) instead of pixelvalue %lu (r %u g %u b %u)",
						l,
						cell.pixel, cell.red, cell.green, cell.blue,
						cellptr->pixel, cellptr->red, cellptr->green, cellptr->blue);
				FAIL;
			} else 
				CHECK;
		}

		for(l=0; l < size; l++) {
			if (XAllocColorCells(display, colormap, False, NULL, 0, &cell.pixel, 1) == False) {
				report("Cell %u was not deallocated.", l);
				FAIL;
			} else
				CHECK;
		}

		free(cellmap);
	}

	CHECKPASS(pathcnt*2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCopyColormapAndFree-3.(C)
 * If any of the visual classes PseudoColor, GrayScale, or
 * DirectColor are supported: When the colormap argument was
 * created by the client with AllocNone, then all of the
 * entries from the colormap argument that have been allocated
 * by the client using XAllocColor, XAllocNamedColor,
 * XAllocColorCells, or XAllocColorPlanes and not freed since
 * they were allocated are moved to the new colourmap with the
 * same colour values and the same read-only or writable
 * characteristics and are freed in colormap.
 */
static void
t003()
{
Colormap	testcmap;
XVisualInfo	*vp;
unsigned long 	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
XColor		*cellmap, *cellptr;
XColor		cell;
unsigned long	l;
int	i;
int		pathcnt = 0, size;
XColor		color, ncol1, ncol2, cols[4];
Display		*disp2;
unsigned long	pix[2];
unsigned long	*copypix;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	cols[0].pixel = 0xffff;
	cols[0].red = 0x0ff0;
	cols[0].green = 0xf0f0;
	cols[0].blue = 0x0f0f;
	cols[0].flags = DoRed|DoGreen|DoBlue;

	cols[1].pixel = 1;
	cols[1].red = 0xffff;
	cols[1].green = 0xffff;
	cols[1].blue = 0xffff;
	cols[1].flags = DoRed|DoGreen|DoBlue;

	for(resetsupvis(vmask);  nextsupvis(&vp); ) {

		disp2 = opendisplay();
		colormap = makecolmap(display, vp->visual, AllocNone);
/* Allocate 2 r/o cells for a new client */
/* set them and find what they ended up as */
		if (XAllocColor(display, colormap, &cols[0]) == False) {
			delete("failed to allocate first r/o cell.");
			return;
		} else
			CHECK;
		trace("r/o Pixel value allocated was %lu", cols[0].pixel);
		XQueryColor(display, colormap, &cols[0]);

		if (XAllocColor(display, colormap, &cols[1]) == False) {
			delete("failed to allocate second r/o cell.");
			return;
		} else
			CHECK;
		trace("r/o Pixel value allocated was %lu", cols[1].pixel);
		XQueryColor(display, colormap, &cols[1]);
		
/* Allocate 2 r/w cells for a new client */
		if(XAllocColorCells(display, colormap, False, 0L, 0, pix, 2) == False) {
			delete("XAllocColorCells() failed.");
			return;
		}
/* set the 2 r/w cells to the same values as the first two. */
		cols[2] = cols[0];
		cols[2].pixel = pix[0];
		cols[3] = cols[1];
		cols[3].pixel = pix[1];
		trace("r/w Pixel values allocated were %lu & %lu",
			cols[2].pixel, cols[3].pixel);
		XStoreColors(display, colormap, &cols[2], 2);
		XQueryColors(display, colormap, &cols[2], 2);

/* Allocate remaining cells to the other client */
		size = maxsize(vp) - 4;
		debug(1, "size=%d, maxsize=%d", size, maxsize(vp));
		copypix = (unsigned long *) malloc( maxsize(vp) * sizeof(unsigned long));

		for(i=0; i < size; i++) {
			if( XAllocColorCells(disp2, colormap, False, 0L, 0, &copypix[i], 1) == False) {
				delete("XAllocColorCells failed after %d cells", i);
				return;
			}
		}

		if( XAllocColorCells(disp2, colormap, False, 0L, 0, &copypix[i], 1) != False) {
			trace("Did not fail as expected");
		} else
			CHECK;

/* do copy and free with first client */
		startcall(display);
		if (isdeleted())
			return;
		testcmap = XCopyColormapAndFree(display, colormap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* check remaining ones are free in copy */
		for ( i=0; XAllocColorCells(disp2, testcmap, False, 0L, 0, &copypix[i], 1) != False; i++) {
			; /* do nothing */
		}
		if ( i != size ) {
			report("only allocated %d cells instead of %d, in copy",
					i, size);
			FAIL;
		} else
			CHECK;

/* now check the first four cells of copy are as we set them in original. */

		if (!check_rgb(disp2, testcmap, &cols[0]) |
			!check_rgb(disp2, testcmap, &cols[1]) |
			!check_rgb(disp2, testcmap, &cols[2]) |
			!check_rgb(disp2, testcmap, &cols[3]) ) {
				report("RGB values not the same!");
				FAIL;
		} else
			CHECK;
/* should check that can now allocate 4 more in old cmap */
		for(i=0; i < 4; i++) {
			if( XAllocColorCells(disp2, colormap, False, 0L, 0, &copypix[i], 1) == False) {
				report("Could not allocate the supposedly freed cell (%d) in original cmap", i);
				FAIL;
			} else {
				trace("allocated cell %d, pixel %lu", i, copypix[i]);
				CHECK;
			}
		}

		if( XAllocColorCells(disp2, colormap, False, 0L, 0, &copypix[i], 1) != False) {
			report("Allocated one more cell in original cmap than expected");
			FAIL;
		} else
			CHECK;
/* chould check that the r/w or r/o characteristics are preserved in copy */
		startcall(disp2);
		XStoreColors(disp2, testcmap, cols, 2);
		endcall(disp2);
		if (geterr() != BadAccess) {
			report("Trying to update 2 r/o cells. Got %s, Expecting BadAccess", errorname(geterr()));
			FAIL;
		} else
			CHECK;

		startcall(disp2);
		XStoreColors(disp2, testcmap, &cols[2], 2);
		endcall(disp2);
		if (geterr() != Success) {
			report("Trying to update 2 r/w cells. Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		} else
			CHECK;
	}
	
	CHECKPASS(12*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XCopyColormapAndFree-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/ECol.mc */
/*
 * Assertion XCopyColormapAndFree-5.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	XCopyColormapAndFree(display, colormap);
	endcall(display);
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
