/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: crtpxmp.m,v 1.9 92/06/11 16:15:43 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XCreatePixmap(display, d, width, height, depth)
 *   Display *display;
 *   Drawable d;
 *   unsigned int width;
 *   unsigned int height;
 *   unsigned int depth;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCreatePixmap	1
char    *TestName = "XCreatePixmap";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE d


/*
 * Arguments to the XCreatePixmap function
 */
static Display *display;
static Drawable d;
static unsigned int width;
static unsigned int height;
static unsigned int depth;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	d = DRW(display);
	width = 13;
	height = 17;
	depth = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XCreatePixmap-1.(A)
 * A call to XCreatePixmap creates a pixmap of width width,
 * height height, and depth depth on the same screen as the
 * drawable argument d, and returns a pixmap ID.
 */
static void
t001()
{
XVisualInfo	*vp;
Status 		gstat;
Window		root_ret;
int		x_ret, y_ret;
unsigned int	w_ret, h_ret, bw_ret, bh_ret, dep_ret;
Pixmap		pmap;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	for(resetvinf(VI_PIX); nextvinf(&vp); ) {
		depth = vp->depth;
		d = DRW(display);
		startcall(display);
		if (isdeleted())
			return;
		pmap = XCreatePixmap(display, d, width, height, depth);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		gstat = XGetGeometry(display, pmap, &root_ret, &x_ret, &y_ret, &w_ret, &h_ret, &bw_ret, &dep_ret);

		if(gstat != True) {
			delete("XGetGeometry did not return True for pixmap ID 0x%x",pmap);
			return;
		} else
			CHECK;

		if(w_ret != width) {
			report("XGetGeometry returned width of %u instead of %u for pixmap of depth %u", w_ret, width, depth);
			FAIL;
		} else 
			CHECK;

		if(h_ret != height) {
			report("XGetGeometry returned height of %u instead of %u for pixmap of depth %u", h_ret, height, depth);
			FAIL;
		} else 
			CHECK;

		if(dep_ret != depth) {
			report("XGetGeometry returned depth of %u instead of depth %u", dep_ret, depth);
			FAIL;
		} else 
			CHECK;

 	}

	CHECKPASS(4 * nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmap-2.(A)
 * When the drawable argument d is an InputOnly window, then
 * no error occurs.
 */
static void
t002()
{
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_PIX); nextvinf(&vp); ) {
		d = iponlywin(display);
		depth = vp->depth;
		startcall(display);
		if (isdeleted())
			return;
		XCreatePixmap(display, d, width, height, depth);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(geterr() != Success) {
			report("XCreatePixmap() produced error %s with an InputOnly Window as the drawable",errorname(geterr()));
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmap-3.(A)
 * When either the width argument or height argument is zero,
 * then a BadValue error occurs.
 */
static void
t003()
{
XVisualInfo	*vp;
unsigned int	depth;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_PIX); nextvinf(&vp); ) {
		depth = vp->depth;

		d = DRW(display); 
		height = 17;
		width = 0;
		startcall(display);
		if (isdeleted())
			return;
		XCreatePixmap(display, d, width, height, depth);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() != BadValue) {
			report("When XCreatePixmap called with zero width");
			report("Got %s, Expecting BadValue error", 
							errorname(geterr()));
			FAIL;
		} else
			CHECK;

		width = 19;
		height = 0;
		startcall(display);
		if (isdeleted())
			return;
		XCreatePixmap(display, d, width, height, depth);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() != BadValue) {
			report("When XCreatePixmap called with zero height");
			report("Got %s, Expecting BadValue error", 
							errorname(geterr()));
			FAIL;
		} else
			CHECK;
	}	

	CHECKPASS(nvinf() * 2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmap-4.(A)
 * When the depth depth is not supported by the screen of the
 * drawable d, then a BadValue error occurs.
 */
static void
t004()
{
XVisualInfo	*vp;
unsigned int	depth, nondepth;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	nondepth = 1;
	for(resetvinf(VI_PIX); nextvinf(&vp); )
		nondepth += vp->depth;	

	trace("Selected unsupported depth is %u", nondepth);
	d = DRW(display);
	depth = nondepth;
	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmap(display, d, width, height, depth);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}
	if(geterr() != BadValue) {
		report("When XCreatePixmap called with unsupported depth %d",
								depth);
		report("Got %s, Expecting BadValue error", errorname(geterr()));
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XCreatePixmap-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EDra.mc */
/*
 * Assertion XCreatePixmap-6.(A)
 * When a drawable argument does not name a valid Drawable,
 * then a BadDrawable error occurs.
 */
/* End of included file error/EDra.mc */

/* Including from file error/EDra.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmap(display, d, width, height, depth);
	endcall(display);
	if (geterr() != BadDrawable) {
		report("Got %s, Expecting BadDrawable", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadDrawable)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EDra.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
