/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: frclrs.m,v 1.13 92/06/11 16:16:00 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XFreeColors(display, colormap, pixels, npixels, planes)
 *   Display *display;
 *   Colormap colormap;
 *   unsigned long *pixels;
 *   int npixels;
 *   unsigned long planes;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFreeColors	1
char    *TestName = "XFreeColors";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XFreeColors function
 */
static Display *display;
static Colormap colormap;
static unsigned long *pixels;
static int npixels;
static unsigned long planes;


unsigned long dummypix;

#define ENOUGH_TIME	100

static
Bool munch(size_in_out, pixels, npix)
	int	*size_in_out;
	unsigned long *pixels;
	int	npix;
{
	int i;
	int sz = 0;
	unsigned long junk;
	int max_sz = (*size_in_out) * ENOUGH_TIME;

	for(i=0; i < npix; i++, pixels++) {
		if(XAllocColorCells(display, colormap, False, 0L, 0, pixels, 1) == False) {
			delete("Could not allocate %d pixels with AllocColorCells (colormap size >= %d, done %d)", npix, *size_in_out, i);
			return False;
		}
	}

	sz = i;

	for(i=0; XAllocColorCells(display, colormap, False, 0L, 0, &junk, 1); i++) {
		if (i > max_sz) {
			delete("Still allocating after %d cells allocated in a colormap of size %d cells.",
				sz + i, *size_in_out);
			return False;
		}
	}

	trace("Rest of colormap allocated (%d + %d = %d cells. Notional size = %d cells).",
		sz, i, sz + i, *size_in_out);
	sz += i;
	*size_in_out = sz;
	return True;
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
	pixels = &dummypix;
	npixels = 1;
	planes = 0L;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XFreeColors-1.(C)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: A call to XFreeColors function frees
 * the colourmap entries obtained by ORing the npixels pixel
 * values specified in the pixels argument with zero or more of
 * the planes specified in the planes argument that have been
 * allocated by the client using XAllocColor, XAllocNamedColor,
 * XAllocColorCells or XAllocColorPlanes.
 */
static void
t001()
{
XVisualInfo 	*vp;
XColor		color1, exactcol;
char		*goodname;
int		size;
unsigned long	pixel;
unsigned long	*pixels;
unsigned long	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	if((vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	if((goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0L) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp);) {

		trace("XAllocNamedColor().");
		colormap = makecolmap(display, vp->visual, AllocNone);

		if(XAllocNamedColor(display, colormap, goodname,  &color1, &exactcol) == False) {
			delete("XAllocNamedColor() failed.");
			return;
		}

		size = maxsize(vp);

		if (!munch(&size, &pixel, 1)) {
			return; /* delete() already called in munch */
		} else
			CHECK;

		pixels = &color1.pixel;
		npixels = 1;
		planes = 0L;

		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(geterr() == Success)
			CHECK;

		if(XAllocColorCells(display, colormap, False, 0L, 0, &pixel, 1) == False) {
			report("XFreeColors() did not free a cell allocated by XAllocNamedColor().");
			FAIL;
		} else
			CHECK;

		freereg();
	}	



	for(resetsupvis(vmask); nextsupvis(&vp);) {

		trace("XAllocColor().");
		colormap = makecolmap(display, vp->visual, AllocNone);

		if(XAllocColor(display, colormap, &color1) == False) {
			delete("XAllocColor() failed.");
			return;
		}

		size = maxsize(vp);

		if (!munch(&size, &pixel, 1)) {
			return; /* delete() already called in munch */
		} else
			CHECK;

		pixels = &color1.pixel;
		npixels = 1;
		planes = 0L;

		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(geterr() == Success)
			CHECK;

		if(XAllocColorCells(display, colormap, False, 0L, 0, &pixel, 1) == False) {
			report("XFreeColors() did not free a cell allocated by XAllocColor().");
			FAIL;
		} else
			CHECK;

		freereg();
	}	


	for(resetsupvis(vmask); nextsupvis(&vp);) {
		trace("XAllocColorCells().");
		colormap = makecolmap(display, vp->visual, AllocNone);

		size = maxsize(vp);

		if (!munch(&size, &pixel, 1)) {
			return; /* delete() already called in munch */
		} else
			CHECK;

		pixels = &pixel;
		npixels = 1;
		planes = 0L;

		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(geterr() == Success)
			CHECK;

		if(XAllocColorCells(display, colormap, False, 0L, 0, &pixel, 1) == False) {
			report("XFreeColors() did not free a cell allocated by XAllocColorCells().");
			FAIL;
		} else
			CHECK;

		freereg();
	}	



	CHECKPASS(nsupvis() * 3 * 3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeColors-2.(C)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: When a read-only colourmap entry has
 * been allocated by another client, then the colourmap entry
 * is not freed on a call to XFreeColors.
 */
static void
t002()
{
Display		*disp2;
XVisualInfo 	*vp;
XColor		color1;
int		size;
unsigned long	pixel;
unsigned long	*pixels;
unsigned long	vmask = (1<<PseudoColor)|(1<<GrayScale)|(1<<DirectColor);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	if((vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp);) {

		colormap = makecolmap(display, vp->visual, AllocNone);

		if((disp2 = opendisplay()) == (Display *) 0) {
			delete("Could not open the display.");
			return;
		}

		if(XAllocColor(Dsp, colormap, &color1) == False) {
			delete("XAllocColor() failed for second client.");
			return;
		}

		if(XAllocColor(disp2, colormap, &color1) == False) {
			delete("XAllocColor() failed for first client.");
			return;
		}

		size = maxsize(vp);

		if (!munch(&size, &pixel, 1)) {
			return; /* delete() already called in munch */
		} else
			CHECK;

		display = Dsp;
		pixels = &color1.pixel;
		npixels = 1;
		planes = 0L;

		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if(geterr() == Success)
			CHECK;

		if(XAllocColorCells(Dsp, colormap, False, 0L, 0, &pixel, 1) != False) {
			report("Shared cell was freed while allocted to another client.");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(3*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeColors-3.(A)
 * If the visual class DirectColor, PseudoColor or GrayScale
 * is supported: When a read-only colourmap entry has been
 * allocated more than once by the client, and XFreeColors has
 * been called one less time than the colormap entry was
 * allocated, then a call to XFreeColors frees the colormap
 * entry.
 */
static void
t003()
{
XVisualInfo 	*vp;
XColor		color1;
int		size, i, refs;
unsigned long	pixel;
unsigned long	*pixels;
unsigned long	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	if((vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp);) {

		colormap = makecolmap(display, vp->visual, AllocNone);

		size = maxsize(vp);

		refs = size-1;
		color1.red = color1.green = color1.blue = 0;
		for(i=0; i < refs; i++)
			if(XAllocColor(display, colormap, &color1) == False) {
				delete("XAllocColor() failed (Iteration %d).", i);
				return;
			}

		if (!munch(&size, &pixel, 1)) {
			return; /* delete() already called in munch */
		} else
			CHECK;

		for(i=0; i < refs-1; i++) {
			pixels = &color1.pixel;
			npixels = 1;
			planes = 0L;
			startcall(display);
			if (isdeleted())
				return;
			XFreeColors(display, colormap, pixels, npixels, planes);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
		}

		if(XAllocColorCells(display, colormap, False, 0L, 0, &pixel, 1) != False) {
			delete("XAllocColorCells() did not fail with a full colourmap.");
			return;
		} else
			CHECK;

		pixels = &color1.pixel;
		npixels = 1;
		planes = 0L;
		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(XAllocColorCells(display, colormap, False, 0L, 0, &pixel, 1) == False) {
			report("XFreeColors() did not free a cell allocated by XAllocColor().");
			FAIL;
		} else
			CHECK;

		freereg();
	}	

	CHECKPASS(3*nsupvis());
  
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeColors-4.(A)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: When one or more pixels cannot be
 * freed, and one or more pixels can be freed, then the pixels
 * that are allocated by the client in the colourmap that can
 * be freed are freed.
 */
static void
t004()
{
Display		*disp2;
XVisualInfo 	*vp;
XColor		color;
int		size;
unsigned long	pixel;
unsigned long	*pixels, *cptr, *ptr;
unsigned long	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	if((vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp);) {

		disp2 = opendisplay();
		colormap = makecolmap(disp2, vp->visual, AllocNone);

		size = maxsize(vp);

		if((cptr = (unsigned long *) malloc(size * sizeof(unsigned long))) == (unsigned long *)0) {
			delete("malloc(%u) failed.", size * sizeof(unsigned long));
			return;
		}

		ptr = cptr;
		color.red = color.green = color.blue = 0;
		if(XAllocColor(disp2, colormap, &color) == False) {
			delete("XAllocColor() failed for client 2");
			return;
		}

		*ptr++ = color.pixel;
		
		if (!munch(&size, ptr, size-1)) {
			return; /* delete() already called in munch */
		} else
			CHECK;

		pixels = cptr;
		npixels = size;
		planes = 0L;

		startcall(disp2);
		XFreeColors(disp2, colormap, pixels, npixels, planes);
		endcall(disp2);
		/* should be BadAccess! Often not on R4 */
		if (geterr() != BadAccess) {
			report("Got %s, expecting BadAccess", errorname(geterr()));
			FAIL;
		} else
			CHECK;

		if(XAllocColorCells(display, colormap, False, 0L, 0, &pixel, 1) == False) {
			report("XFreeColors() did not free a cell allocated by XAllocColor().");
			FAIL;
		} else
			CHECK;

		if(XAllocColorCells(display, colormap, False, 0L, 0, &pixel, 1) != False) {
			report("Colormap was not completely allocated.");
			FAIL;
		}
		
		free(cptr);
		freereg();
	}

	CHECKPASS(3*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeColors-5.(A)
 * If the visual class DirectColor is supported: When all
 * related colormap entries are already freed, then a call to
 * XFreeColors with a particular pixel value allows that pixel
 * value to be allocated by a subsequent call to
 * XAllocColorPlanes.
 */
static void
t005()
{
XVisualInfo	*vp;
int		nreds, ngreens, nblues;
int		reds, greens, blues;
unsigned long	pr[2], rr, gr, br;
unsigned long	tpr[2], trr, tgr, tbr;
unsigned long	vmask = (1<<DirectColor);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}
	resetsupvis(vmask);
	nextsupvis(&vp);

	nreds = bitcount(vp->red_mask);
	ngreens = bitcount(vp->green_mask);
	nblues = bitcount(vp->blue_mask);

	reds = nreds - 1;
	greens = ngreens - 1;
	blues = nblues - 1;

	colormap = makecolmap(display, vp->visual, AllocNone);

	if((XAllocColorPlanes(display, colormap, False, pr, 2, reds, greens, blues, &rr, &gr, &br)) == False) {
		delete("XAllocColorPlanes failed.");
		return;
	} else
		CHECK;

	if((XAllocColorPlanes(display, colormap, False, tpr, 1, 0, 0, 0, &trr, &tgr, &tbr)) != False) {
		delete("Allocated colormap was not completely filled");
		return;
	} else
		CHECK;

	pixels = pr+1;
	npixels = 1;
	planes = rr | gr | br;
	startcall(display);
	if (isdeleted())
		return;
	XFreeColors(display, colormap, pixels, npixels, planes);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if((XAllocColorPlanes(display, colormap, False, pr, 1, reds, greens, blues, &rr, &gr, &br)) == False) {
		trace("Freed colormap cells could not be re-allocated.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeColors-6.(A)
 * When a specified pixel is not a valid entry in the colormap
 * argument, then a BadValue error occurs.
 */
static void
t006()
{
XVisualInfo	*vp;
XColor		color;
unsigned long	pixel[2];
unsigned long	vmask = 0L;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vmask = visualsupported(display, vmask)) == 0L) {
		delete("No visuals reported as supported");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {

		colormap = makecolmap(display, vp->visual, AllocNone);
		pixel[0] = ~0L;
		pixels = pixel;
		npixels = 1;
		planes = 0L;
		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadValue)
			CHECK;

		colormap = makecolmap(display, vp->visual, AllocNone);
		if(XAllocColor(display, colormap, &color) == False) {
			delete("XAllocColor Failed.");
			return;
		}

		pixel[0] = color.pixel;
		pixel[1] = ~0L;
		pixels = pixel;
		npixels = 2;
		planes = 0L;
		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadValue)
			CHECK;

	}	

	CHECKPASS(2*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAcc2.mc */
/*
 * Assertion XFreeColors-7.(A)
 * When an attempt to free a colormap entry not allocated by
 * the client is made, then a BadAccess error occurs.
 */
/* End of included file error/EAcc2.mc */

static void
t007()
{
XVisualInfo	*vp;
unsigned long	pixel[1];
unsigned long	vmask = 0L;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vmask = visualsupported(display, vmask)) == 0L) {
		delete("No visuals reported as supported");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {

		colormap = makecolmap(display, vp->visual, AllocNone);
		pixel[0] = 0;
		pixels = pixel;
		npixels = 1;
		planes = 0L;
		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != BadAccess) {
			report("Got %s, Expecting BadAccess", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadAccess)
			CHECK;
	}	

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XFreeColors-8.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t008()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	XFreeColors(display, colormap, pixels, npixels, planes);
	endcall(display);
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/*
 * Assertion XFreeColors-9.(A)
 * When more than one pixel value is not a valid entry in the
 * colormap argument, then a BadValue error occurs which will
 * report any one of the invalid pixel values.
 */
static void
t009()
{
XVisualInfo	*vp;
XColor		color;
int		errval;
unsigned long	pixel[3];
unsigned long	vmask = 0L;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vmask = visualsupported(display, vmask)) == 0L) {
		delete("No visuals reported as supported");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {

		colormap = makecolmap(display, vp->visual, AllocNone);
		if(XAllocColor(display, colormap, &color) == False) {
			delete("XAllocColor Failed.");
			return;
		}
		pixel[0] = ~0L;
		pixel[1] = color.pixel;
		pixel[2] = ~0L;
		pixels = pixel;
		npixels = 3;
		planes = 0L;
		startcall(display);
		if (isdeleted())
			return;
		XFreeColors(display, colormap, pixels, npixels, planes);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadValue) {
			errval = getbadvalue();
			if( (errval != -1) && (errval != -2) ) {
				report("Erroneous value was reported as %d, instead of -1 or -2", errval);
				FAIL;
			} else
				CHECK;
		}

	}
	CHECKPASS(nsupvis());


	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
