/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stclprctng.m,v 1.6 92/06/11 16:17:56 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering)
 *   Display *display;
 *   GC gc;
 *   int clip_x_origin;
 *   int clip_y_origin;
 *   XRectangle *rectangles;
 *   int n;
 *   int ordering;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetClipRectangles	1
char    *TestName = "XSetClipRectangles";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_GC gc


/*
 * Arguments to the XSetClipRectangles function
 */
static Display *display;
static GC gc;
static int clip_x_origin;
static int clip_y_origin;
static XRectangle *rectangles;
static int n;
static int ordering;


XRectangle	crect;

int
bad_orderA(order_arg)
int	order_arg;
{
XVisualInfo *vp;
Window	win;
XGCValues	values;
XRectangle 	drects[2];
int		i, j;
int		pass = 0, fail = 0;

	resetvinf(VI_WIN);
	nextvinf(&vp);

	win = makewin(display, vp);
	values.foreground = W_FG;
	values.background = W_BG;


	drects[0].x = 0;
	drects[0].y = 10;
	drects[0].width = drects[0].height = 5;

	drects[1].x = 10;
	drects[1].y = 5;
	drects[1].width = drects[1].height = 5;

	gc = XCreateGC(display, win, GCForeground | GCBackground, &values);


	rectangles = drects;
	n = 2;
	ordering = order_arg;

	/* 
	 startcall(display);
	 if (isdeleted())
	 	return;
	 * XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering) can't be used at the moment - 
	 endcall(display);
	 if (geterr() != Success) {
	 	report("Got %s, Expecting Success", errorname(geterr()));
	 	FAIL;
	 }
	 * it expects geterr() to have one correct return type.  
	 */

	startcall(Dsp);

	if (isdeleted()) {
		return;
	}
	XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering);

	endcall(Dsp);

	if (geterr() == BadMatch) {
		trace("XSetClipRectangles returned BadMatch error.");
		CHECK;
	} else if (geterr() == Success) {
		trace("XSetClipRectangles did not return an error.");
		CHECK;
	} else {
		report("Got %s, Expecting BadMatch or Success.", errorname(geterr()));
		FAIL;
	}
	return(pass);
}

int
bad_orderB(order_arg)
int	order_arg;
{
XVisualInfo *vp;
Window	win;
XGCValues	values;
XRectangle 	drects[3];
int		i, j;
int		pass = 0, fail = 0;

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);
	values.foreground = W_FG;
	values.background = W_BG;


	drects[0].x = 20;
	drects[0].y = 5;
	drects[0].width = drects[0].height = 5;

	drects[1].x = 10;
	drects[1].y = 5;
	drects[1].width = drects[1].height = 5;

	gc = XCreateGC(display, win, GCForeground | GCBackground, &values);

	rectangles = drects;
	n = 2;
	ordering = order_arg;

	/* 
	 startcall(display);
	 if (isdeleted())
	 	return;
	 * XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering) can't be used at the moment - 
	 endcall(display);
	 if (geterr() != Success) {
	 	report("Got %s, Expecting Success", errorname(geterr()));
	 	FAIL;
	 }
	 * it expects geterr() to have one correct return type.
	 */

	startcall(Dsp);

	if (isdeleted()) {
		return;
	}
	XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering);

	endcall(Dsp);

	if (geterr() == BadMatch) {
		trace("XSetClipRectangles returned BadMatch error.");
		CHECK;
	} else if (geterr() == Success) {
		trace("XSetClipRectangles did not return an error.");
		CHECK;
	} else {
		report("Got %s, Expecting BadMatch or Success.", errorname(geterr()));
		FAIL;
	}
	return(pass);
}

int
bad_orderC(order_arg)
int	order_arg;
{
XVisualInfo *vp;
Window	win;
XGCValues	values;
XRectangle 	drects[3];
int		i, j;
int		pass = 0, fail = 0;

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);
	values.foreground = W_FG;
	values.background = W_BG;


	drects[0].x = 20;
	drects[0].y = 5;
	drects[0].width = drects[0].height = 5;

	drects[1].x = 30;
	drects[1].y = 5;
	drects[1].width = drects[1].height = 4;

	gc = XCreateGC(display, win, GCForeground | GCBackground, &values);

	rectangles = drects;
	n = 2;
	ordering = order_arg;

	/* 
	 startcall(display);
	 if (isdeleted())
	 	return;
	 * XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering) can't be used at the moment - 
	 endcall(display);
	 if (geterr() != Success) {
	 	report("Got %s, Expecting Success", errorname(geterr()));
	 	FAIL;
	 }
	 * it expects geterr() to have one correct return type.
	 */

	startcall(Dsp);

	if (isdeleted()) {
		return;
	}
	XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering);

	endcall(Dsp);

	if (geterr() == BadMatch) {
		trace("XSetClipRectangles returned BadMatch error.");
		CHECK;
	} else if (geterr() == Success) {
		trace("XSetClipRectangles did not return an error.");
		CHECK;
	} else {
		report("Got %s, Expecting BadMatch or Success.", errorname(geterr()));
		FAIL;
	}
	return(pass);
}



/* Value list for use in test t009 */
static int 	orderingvallist[] = {
	Unsorted,
	YSorted,
	YXSorted,
	YXBanded,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	gc = 0;
	clip_x_origin = 1;
	clip_y_origin = 1;
	rectangles = &crect;
	n = 1;
	ordering = Unsorted;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	gc = ErrdefGC;
}

/*
 * Assertion XSetClipRectangles-1.(A)
 * A call to XSetClipRectangles sets the clip_mask component
 * of the specified GC to the specified list of rectangles and
 * sets the clip_x_origin and clip_y_origin components of the
 * specified GC to the arguments clip_x_origin and
 * clip_y_origin.
 */
static void
t001()
{
XVisualInfo *vp;
Window	win;
XGCValues	values;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);
	values.foreground = W_FG;
	values.background = W_BG;
	values.function = GXxor;

	gc = XCreateGC(display, win, GCForeground | GCBackground | GCFunction, &values);

	XDrawPoint(display, win, gc, 0, 0);

	if( ! checkpixel(display, win, 0, 0, W_FG)) {
		delete("Pixel at (0, 0) was not set to foreground.");
		return;
	} else 
		CHECK;

	crect.x = crect.y = 1;
	crect.width = crect.height = 1;

	startcall(display);
	if (isdeleted())
		return;
	XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawPoint(display, win, gc, 2, 2);

	if( ! checkpixel(display, win, 2, 2, W_FG)) {
		delete("Pixel at (2, 2) was not set to foreground.");
		return;
	} else 
		CHECK;

	XDrawPoint(display, win, gc, 0, 0);

	if( ! checkpixel(display, win, 0, 0, W_FG)) {
		report("Pixel at (0, 0) was not left as foreground.");
		FAIL;
	} else 
		CHECK;

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetClipRectangles-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When two or more of the rectangles intersect, then no error
 * occurs.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSetClipRectangles-3.(A)
 * When the number of rectangles specified by the argument n
 * is zero, then the clip_mask component of the specified gc is
 * set to disable output on any subsequent graphics operation.
 */
static void
t003()
{
XVisualInfo	*vp;
Window		win;
XGCValues	values;
GC		pgc;
int	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	values.foreground = W_FG;
	values.background = W_BG;

	crect.x = 0;
	crect.y = 0;
	crect.width = 10;
	crect.height = 20;

	for(resetvinf(VI_WIN);	nextvinf(&vp); ) {
		win = makewin(display, vp);
		gc = XCreateGC(display, win, GCForeground | GCBackground, &values);

		if( checkarea(display, win, (struct area *) 0, W_BG, 0L, 0) == False) {
			delete("Window contents were not initialised.");
			return;
		} else
			CHECK;

		n = 0;
		startcall(display);
		if (isdeleted())
			return;
		XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		for(j=0; j<W_STDHEIGHT; j++)
			for(i=0; i< W_STDWIDTH; i++)
				XDrawPoint(display, win, gc, i, j);

		if( checkarea(display, win, (struct area *) 0, W_BG, 0L, 0) == False) {
			report("Drawing was not disabled.");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(2 * nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetClipRectangles-4.(A)
 * When the ordering argument is YSorted, and the specified
 * list of rectangles is not non-decreasing in the Y origin,
 * then either a BadMatch error occurs or no error occurs.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(bad_orderA(YSorted) == 1)
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetClipRectangles-5.(A)
 * When the ordering argument is YXSorted and the specified
 * list of rectangles is either not non-decreasing in the Y
 * origin or rectangles with the same Y origin are not
 * non-decreasing in the X origin, then either a BadMatch error
 * occurs or no error occurs.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(bad_orderA(YXSorted) == 1 &&
	   bad_orderB(YXSorted) == 1)
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetClipRectangles-6.(A)
 * When the ordering argument is YXBanded and the specified
 * list of rectangles is either not non-decreasing in the Y
 * origin or rectangles with the same Y origin are not
 * non-decreasing in the X origin or there is a Y scanline
 * included by rectangles with different Y origins or extents,
 * then either a BadMatch error occurs or no error occurs.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(bad_orderA(YXBanded) == 1 &&
	   bad_orderB(YXBanded) == 1 &&
	   bad_orderC(YXBanded) == 1)
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetClipRectangles-7.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XSetClipRectangles-8.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t008()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/*
 * Assertion XSetClipRectangles-9.(A)
 * When the value of ordering is other than Unsorted, YSorted,
 * YXSorted or YXBanded, then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t009()
{
#undef	VALUE_ARG
#define	VALUE_ARG ordering
#undef	VALUE_LIST
#define	VALUE_LIST orderingvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin, rectangles, n, ordering);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
