/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stlnattrbt.m,v 1.5 92/06/11 16:18:59 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style)
 *   Display *display;
 *   GC gc;
 *   unsigned int line_width;
 *   int line_style;
 *   int cap_style;
 *   int join_style;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetLineAttributes	1
char    *TestName = "XSetLineAttributes";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_GC gc


/*
 * Arguments to the XSetLineAttributes function
 */
static Display *display;
static GC gc;
static unsigned int line_width;
static int line_style;
static int cap_style;
static int join_style;


XPoint Jpoints[] = { {0, 1}, {2, 1}, {2, 3} };


/* Value list for use in test t004 */
static int 	line_stylevallist[] = {
	LineSolid,
	LineOnOffDash,
	LineDoubleDash,
};



/* Value list for use in test t005 */
static int 	cap_stylevallist[] = {
	CapNotLast,
	CapButt,
	CapRound,
	CapProjecting,
};



/* Value list for use in test t006 */
static int 	join_stylevallist[] = {
	JoinMiter,
	JoinRound,
	JoinBevel,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	gc = 0;
	line_width = 0;
	line_style = 0;
	cap_style = 0;
	join_style = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	gc = ErrdefGC;
}

/*
 * Assertion XSetLineAttributes-1.(A)
 * A call to XSetLineAttributes sets the line_width,
 * line_style, cap_style and join_style components of the
 * specified GC to the values of the line_width, line_style,
 * cap_style and join_style arguments.
 */
static void
t001()
{
XVisualInfo *vp;
XGCValues values;
GC gc;
Window win;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);
	values.foreground = W_FG;
	values.background = W_BG;
	values.line_width = 1;


	gc = XCreateGC(display, win, (GCLineWidth | GCForeground | GCBackground), &values);

	XDrawLine(display, win, gc, 0, 1, 2, 1);

	if( ! checkpixel(display, win, 0, 2, W_BG)) {
		delete("Pixel at (0, 2) was not set to background.");
		return;
	} else
		CHECK;

	line_width = 10;
	startcall(display);
	if (isdeleted())
		return;
	XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawLine(display, win, gc, 0, 1, 2, 1);

	if( ! checkpixel(display, win, 0, 2, W_FG)) {
		report("Pixel at (0, 2) was not set to foreground.");
		FAIL;
	} else
		CHECK;

/* Line Style */

	values.line_style = LineDoubleDash;
	values.dashes = 1;

	gc =  XCreateGC(display, win, (GCLineWidth | GCForeground | GCBackground | GCLineStyle | GCDashList), &values);

	win = makewin(display, vp);

	XDrawLine(display, win, gc, 0, 0, 10, 0);

	if( ! checkpixel(display, win, 0, 1, W_BG)) {
		delete("Pixel at (0, 1) was not set to background.");
		return;
	} else
		CHECK;

	line_style = LineSolid;
	startcall(display);
	if (isdeleted())
		return;
	XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawLine(display, win, gc, 0, 0, 10, 0);

	if( ! checkpixel(display, win, 0, 1, W_FG)) {
		report("Pixel at (0, 1) was not set to foreground.");
		FAIL;
	} else
		CHECK;

/* Cap Style */


	values.cap_style = CapNotLast;
	values.line_width = 0;

	gc =  XCreateGC(display, win, (GCLineWidth | GCForeground | GCBackground | GCCapStyle), &values);
	win = makewin(display, vp);
	XDrawLine(display, win, gc, 0, 0, 2, 0);

	if( ! checkpixel(display, win, 2, 0, W_BG)) {
		delete("Pixel at (2, 0) was not set to background.");
		return;
	} else
		CHECK;
	
	line_width = 0;
	cap_style = CapButt;
	startcall(display);
	if (isdeleted())
		return;
	XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XDrawLine(display, win, gc, 0, 0, 2, 0);

	if( ! checkpixel(display, win, 2, 0, W_FG)) {
		report("Pixel at (2, 0) was not set to foreground.");
		FAIL;
	} else
		CHECK;

/* Join Style */

	values.join_style = JoinBevel;
	values.line_width = 3;

	gc =  XCreateGC(display, win, (GCLineWidth | GCForeground | GCBackground | GCJoinStyle), &values);
	win = makewin(display, vp);
	XDrawLines(display, win, gc, Jpoints, sizeof(Jpoints) / sizeof(XPoint), CoordModeOrigin);

	if( ! checkpixel(display, win, 3, 0, W_BG)) {
		delete("Pixel at (3, 0) was not set to background.");
		return;
	} else
		CHECK;

	line_width = 3;
	join_style = JoinMiter;
	startcall(display);
	if (isdeleted())
		return;
	XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XDrawLines(display, win, gc, Jpoints, sizeof(Jpoints) / sizeof(XPoint), CoordModeOrigin);

	if( ! checkpixel(display, win, 3, 0, W_FG)) {
		report("Pixel at (3, 0) was not set to foreground.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(8);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetLineAttributes-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XSetLineAttributes-3.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/*
 * Assertion XSetLineAttributes-4.(A)
 * When the value of line_style is other than LineSolid,
 * LineOnOffDash or LineDoubleDash, then a BadValue error
 * occurs.
 */
/* Including from file error/EVal.mc */
static void
t004()
{
#undef	VALUE_ARG
#define	VALUE_ARG line_style
#undef	VALUE_LIST
#define	VALUE_LIST line_stylevallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style);
		gcflush(A_DISPLAY, A_GC);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/*
 * Assertion XSetLineAttributes-5.(A)
 * When the value of cap_style is other than CapNotLast,
 * CapButt, CapRound or CapProjecting, then a BadValue error
 * occurs.
 */
/* Including from file error/EVal.mc */
static void
t005()
{
#undef	VALUE_ARG
#define	VALUE_ARG cap_style
#undef	VALUE_LIST
#define	VALUE_LIST cap_stylevallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style);
		gcflush(A_DISPLAY, A_GC);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/*
 * Assertion XSetLineAttributes-6.(A)
 * When the value of join_style is other than JoinMiter,
 * JoinRound or JoinBevel, then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t006()
{
#undef	VALUE_ARG
#define	VALUE_ARG join_style
#undef	VALUE_LIST
#define	VALUE_LIST join_stylevallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XSetLineAttributes(display, gc, line_width, line_style, cap_style, join_style);
		gcflush(A_DISPLAY, A_GC);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
