/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: strclrs.m,v 1.12 92/06/11 16:19:23 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XStoreColors(display, colormap, color, ncolors)
 *   Display	*display;
 *   Colormap colormap;
 *   XColor *color;
 *   int ncolors;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XStoreColors	1
char    *TestName = "XStoreColors";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XStoreColors function
 */
static Display	*display;
static Colormap colormap;
static XColor *color;
static int ncolors;


XColor dummycol = { 0L, 0,0,0, DoRed|DoGreen|DoBlue, 0 };

#define lowbit(x)		((x) & (~(x) + 1))
#define bitcontig(x)	((((x) + lowbit(x)) & (x)) == 0)
#define bitsubset(a,b)	(((a) & (b)) == (a))

static int
maskshift(mask)
register unsigned long mask;
{
register int i;

	for (i = 0; mask; i++) {
		if (mask & 0x1)
			return i;
		mask >>= 1;
	}
	return i;
}

static
Bool set_one_col(refp, s, flags, mask)
	unsigned long *refp;
	char *s;
	char flags;
	unsigned long mask;
{
	unsigned long i;
	XColor *cp1;
	unsigned long *lrefp = refp;
	int fail = 0; /* used in the FAIL in XCall (spelt wrong intentionally) */
	/* expects to find color and ncolors in-scope */

	for(i=0, cp1 = color; i < ncolors; i++, cp1++, lrefp++) {
		unsigned long subfield_ix = i << maskshift(mask);

		if (!bitsubset(subfield_ix, mask)) {
			delete("Inconsistent maxsize() result: size of %d is too big to fit into %s mask 0x%lx (detected at index %lu, giving pixel 0x%lx)",
				ncolors, s, mask, i, subfield_ix);
			return False;
		}
		*lrefp = cp1->pixel = subfield_ix;
		cp1->flags = flags;
		switch (flags) {
		case DoRed:
			cp1->red = i;
			break;
		case DoGreen:
			cp1->green = i;
			break;
		case DoBlue:
			cp1->blue = i;
			break;
		default:
			delete("Flags 0x%x has more than one colour in it.", (unsigned int)flags);
			return False;
		}
	}
	/* The following may give a warning about the return; */
	startcall(display);
	if (isdeleted())
		return;
	XStoreColors(display, colormap, color, ncolors);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XQueryColors(display, colormap, color, ncolors);
	for(i=0,cp1=color,lrefp=refp; i < ncolors; i++, lrefp++, cp1++) {
		*lrefp = (flags==DoRed) ? cp1->red :
				((flags==DoGreen) ? cp1->green : cp1->blue);
	}
	return (geterr() == Success);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
	color = &dummycol;
	ncolors = 1;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XStoreColors-1.(C)
 * If any of the visual classes DirectColor, PseudoColor or
 * Grayscale is supported: A call to XStoreColors obtains the
 * closest available RGB values for the colormap argument to
 * those specified in the red, green and blue components of
 * the XColor structures named by the color argument, and
 * stores those values in the read-write colourmap entries
 * specified by the pixel components of the XColor structures.
 */
static void
t001()
{
XVisualInfo 	*vp;
XColor		*colp, *cp, *testcol;
unsigned long 	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
unsigned long	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		colormap = makecolmap(display, vp->visual, AllocAll);
		ncolors = maxsize(vp);

		color = (XColor *) malloc(ncolors * sizeof(XColor));
		if(color == (XColor *) 0) {
			delete("malloc() couldn't allocate %d cells for the XColor array.", ncolors);
			return;
		} else
			CHECK;

		for(i=0, colp = color; i < ncolors; i++, colp++) {
			colp->pixel = i;
			colp->red = 0xffff ^ (i*4);
			colp->green =  0xf1f0 ^ (i*8);
			colp->blue =  0x0ff8 ^ (i*6);
			colp->flags = (char) (DoRed|DoGreen|DoBlue);
		}

		startcall(display);
		if (isdeleted())
			return;
		XStoreColors(display, colormap, color, ncolors);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if(geterr() == Success)
			CHECK;

		/* Write colormap stored values back over the original array */
		XQueryColors(display, colormap, color, ncolors);

		startcall(display);
		if (isdeleted())
			return;
		XStoreColors(display, colormap, color, ncolors);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if(geterr() == Success)
			CHECK;

		testcol = (XColor *) malloc(ncolors * sizeof(XColor));
		if(testcol == (XColor *) 0) {
			delete("malloc() couldn't allocate %d cells for the XColor array.", ncolors);
			return;
		} else
			CHECK;

		for(i=0; i<ncolors; i++)
			testcol[i].pixel = i;

		XQueryColors(display, colormap, testcol, ncolors);

		for(i=0, colp = testcol, cp = color; i < ncolors; i++, colp++, cp++)
			if( (cp->red != colp->red) || (cp->green != colp->green) || (cp->blue != colp->blue) ) {
				report("cell %ld:  r %u g %u b %u instead of r %u g %u b %u",
						i,
						cp->red, cp->green, cp->blue,
						colp->red, colp->green, colp->blue);
				FAIL;
			}

		if( i == ncolors)
			CHECK;
		free((char *)color);
		free((char *)testcol);
	}
	CHECKPASS(5*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XStoreColors-2.(C)
 * If any of the visual classes DirectColor, PseudoColor or
 * Grayscale is supported: A call to XStoreColors changes the
 * red, green and blue values in each of the ncolors read-write
 * colourmap entries in accordance with the flags components of
 * the XColor structures named by the color argument.
 */
static void
t002()
{
XVisualInfo 	*vp;
XColor		*colp, *cp, *testcol, *refcol, *quercol;
unsigned long 	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
unsigned long	i,j;
char		flags;
XColor		*redp, *greenp, *bluep;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		colormap = makecolmap(display, vp->visual, AllocAll);
		ncolors = maxsize(vp);

		refcol = (XColor *) malloc(ncolors * sizeof(XColor));
		testcol = (XColor *) malloc(ncolors * sizeof(XColor));
		quercol = (XColor *) malloc(ncolors * sizeof(XColor));

		if((quercol == (XColor *) 0) ||  (refcol == (XColor *) 0) ||  (testcol == (XColor *) 0)){
			delete("malloc() couldn't allocate %d cells for the XColor array.", ncolors);
			return;
		} else
			CHECK;

		for(i=0, cp = testcol, colp = refcol; i < ncolors; i++, cp++, colp++) {
			colp->pixel = i;
			colp->red = 0;
			colp->green = 0;
			colp->blue = 0;
			colp->flags = (DoRed|DoGreen|DoBlue);

			cp->pixel = i;
			cp->red = 0xffff;
			cp->green = 0xffff;
			cp->blue = 0xffff;
			cp->flags = (DoRed|DoGreen|DoBlue);

			quercol[(int)i].pixel = i;
		}

		color = refcol;
		startcall(display);
		if (isdeleted())
			return;
		XStoreColors(display, colormap, color, ncolors);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if(geterr() == Success)
			CHECK;

		/* Write colormap stored values back over the original array */
		XQueryColors(display, colormap, refcol, ncolors);


		color = testcol;
		startcall(display);
		if (isdeleted())
			return;
		XStoreColors(display, colormap, color, ncolors);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if(geterr() == Success)
			CHECK;
		/* Write colormap stored values back over the original array */
		XQueryColors(display, colormap, testcol, ncolors);


		for(i=0; i<8; i++) {

			flags = 0;

			if(i&DoRed) {
				flags |= DoRed;
				redp = testcol;
			} else
				redp = refcol;

			if(i&DoGreen) {
				flags |= DoGreen;
				greenp = testcol;
			} else
				greenp = refcol;

			if(i&DoBlue) {
				flags |= DoBlue;
				bluep = testcol;
			} else
				bluep = refcol;

			color = refcol;
			startcall(display);
			if (isdeleted())
				return;
			XStoreColors(display, colormap, color, ncolors);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			if(geterr() == Success)
				CHECK;

			for(j=0, cp=testcol; j < ncolors; cp++, j++)
				cp->flags = flags;

			color = testcol;
			startcall(display);
			if (isdeleted())
				return;
			XStoreColors(display, colormap, color, ncolors);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			if(geterr() == Success)
				CHECK;


			XQueryColors(display, colormap, quercol, ncolors);

			for(j=0, cp = quercol; j < ncolors; j++, cp++, redp++, greenp++, bluep++)
				if( (cp->red != redp->red) || (cp->green != greenp->green) || (cp->blue != bluep->blue) ) {
					report("cell %ld)  r %u g %u b %u instead of r %u g %u b %u",
							i,  cp->red, cp->green, cp->blue,  redp->red, greenp->green, bluep->blue);
					FAIL;
				}
			if (j == ncolors)
				CHECK;
		}

		free((char*)quercol);
		free((char*)testcol);
		free((char*)refcol);
	}
	CHECKPASS((3 + 3*i) * nsupvis());


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XStoreColors-3.(D)
 *   Reason: There is no known portable test method for this assertion
 * When the colormap is an installed colourmap for its screen,
 * then any changes made by a call to XStoreColors are visible
 * immediately.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XStoreColors-4.(C)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: When one or more colourmap entries
 * cannot be changed, then all the specified colourmap entries
 * that are allocated writable by any client are changed.
 */
static void
t004()
{
XVisualInfo 	*vp;
XColor		*cp1;
XColor		*cp2;
XColor		colors[2];
unsigned long 	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
unsigned long	i;
unsigned long	junk;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

/* If visual class is DirectColor, PseudoColor or GrayScale: */
	for(resetsupvis(vmask); nextsupvis(&vp); ) {

/*    Create a colormap with AllocNone */
		colormap = makecolmap(display, vp->visual, AllocNone);
		ncolors = maxsize(vp);
/*    Check >= 2 cells in colormap */
		if (ncolors < 2) {
			delete("Need at least two cells in colormap to test this assertion, only had %d available.", ncolors);
			return;
		} else
			CHECK;

		ncolors = 2;
		color = colors;

/*    Allocate first cell as r/o, second as read write */
		for(i=0, cp1 = color; i < 2; i++, cp1++) {
			cp1->pixel = i;
			cp1->green = cp1->blue = cp1->red = 0xffff ^ (i*32767);
			cp1->flags = (DoRed|DoGreen|DoBlue);
		}
		if (XAllocColor(display, colormap, &color[0]) == False) {
			delete("Could not allocate first cell as r/o.");
			return;
		} else
			CHECK;

		if (XAllocColorCells(display, colormap, True, &junk, 0,
					&color[1].pixel, 1) == False) {
			delete("Could not allocate second cell as r/w.");
			return;
		} else
			CHECK;

/*    Update r/w cell to known value with XStoreColor. */
		XStoreColor(display, colormap, &color[1]);
/*    Discover what it was really set to, and remember. */
		XQueryColor(display, colormap, &color[0]);

/*    Update both cells, the r/w one to an opposite value, with xname */
		color[1].red = color[1].green = color[1].blue ^= 0xffff;
		startcall(display);
		if (isdeleted())
			return;
		XStoreColors(display, colormap, color, ncolors);
		endcall(display);
		if (geterr() != BadAccess) {
			report("Got %s, Expecting BadAccess", errorname(geterr()));
			FAIL;
		}
/*    Verify BadAccess occurred, will have Failed if not -- see CODE arg. */
		if (geterr() == BadAccess)
			CHECK;
		else
			report("Updating 2 cells, 1st r/o & 2nd r/w, did not give expected BadAccess error.");

/*    Check that despite error the 2nd, r/w, cell was changed in value */
		XQueryColor(display, colormap, cp2 = &color[1]);
		cp1 = cp2 - 1;
		/* now cp1 == remembered (different), cp2 == actual */

		if (cp1->red == cp2->red && cp1->green == cp2->green &&
					cp1->blue == cp2->blue) {
			report("After error the r/w cell was not updated: got r,g,b = %lu,%lu,%lu",
				cp2->red, cp2->green, cp2->blue);
			FAIL;
		} else
			CHECK;

/*    Repeat for next matching visual. */
	}

	CHECKPASS(5*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XStoreColors-5.(C)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: When the pixel component in more
 * than one of the XColor structures named by the color
 * argument is not a valid entry in the colormap argument, then
 * a BadValue error occurs which will report any one of the
 * invalid pixel values.
 */
static void
t005()
{
int		errval;
XVisualInfo 	*vp;
XColor		*cp1;
unsigned long 	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
XColor		colors[3];
unsigned long	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {

		colormap = makecolmap(display, vp->visual, AllocAll);
		for(i=0, cp1 = colors; i < 3; i++, cp1++) {
			cp1->pixel = 0;
			cp1->red = 0xffff;
			cp1->green = 0;
			cp1->blue = 0;
			cp1->flags = (DoRed|DoGreen|DoBlue);
		}

		colors[0].pixel = -1;
		colors[1].pixel = 0;
		colors[2].pixel = -2;
		color = colors;
		ncolors = 3;
		startcall(display);
		if (isdeleted())
			return;
		XStoreColors(display, colormap, color, ncolors);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadValue) {
 			errval = getbadvalue();
                        if( (errval != -1) && (errval != -2) ) {
                                report("Erroneous value was reported as %d, instead of -1 or -2", errval);
                                FAIL;
                        } else {
                                CHECK;
				trace("Reported BadValue was %d", errval);
			}
		}
	}

	CHECKPASS(nsupvis());

		
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAcc3.mc */
/*
 * Assertion XStoreColors-6.(A)
 * When an attempt to store into a read-only or an unallocated
 * colormap entry is made, then a BadAccess error occurs.
 */
/* End of included file error/EAcc3.mc */

static void
t006()
{
XVisualInfo	*vp;
XColor          *cp, *testcol;
Display		*disp2;
unsigned long	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_WIN); nextvinf(&vp); ) {
		colormap = makecolmap(display, vp->visual, AllocNone);
		ncolors = maxsize(vp);
		testcol = (XColor *) malloc(ncolors * sizeof(XColor));

		if(testcol == (XColor *) 0) {
			delete("malloc() couldn't allocate %d cells for the XColor array.", ncolors);
			return;
		} else
			CHECK;

		for(i=0, cp = testcol; i < ncolors; i++, cp++) {
			cp->pixel = i;
			cp->green = cp->blue = cp->red = 0xffff ^ (i*3);
			cp->flags = (DoRed|DoGreen|DoBlue);
		}

		if (vp->class == TrueColor || vp->class == StaticColor ||
		    vp->class == StaticGray) { /* read-only map */

			trace("Testing a colormap with just r-o cells.");

			color = testcol;
			startcall(display);
			if (isdeleted())
				return;
			XStoreColors(display, colormap, color, ncolors);
			endcall(display);
			if (geterr() != BadAccess) {
				report("Got %s, Expecting BadAccess", errorname(geterr()));
				FAIL;
			}
			if(geterr() == BadAccess)
				CHECK;
		} else {
			trace("Testing a colormap with unallocated cells.");
			XSync(display, False);
			CHECK;
		}

		trace("Open second client connection.");
		disp2 = opendisplay();

		if(XAllocColor(disp2, colormap, &testcol[0]) == False) {
			delete("XAllocColor() failed to allocate a r/o cell for a second client.");
			return;
		} else
			CHECK;

		trace("Trying to write into a r/o cell allocated by another client.");
		color = &testcol[0];
		ncolors = 1;
		startcall(display);
		if (isdeleted())
			return;
		XStoreColors(display, colormap, color, ncolors);
		endcall(display);
		if (geterr() != BadAccess) {
			report("Got %s, Expecting BadAccess", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadAccess)
			CHECK;

		if(XAllocColor(display, colormap, &testcol[1]) == False) {
			delete("XAllocColor() failed to allocate a r/o cell");
			return;
		} else
			CHECK;

		trace("Trying to write into a r/o cell allocated by self.");
		color = &testcol[1];
		ncolors = 1;
		startcall(display);
		if (isdeleted())
			return;
		XStoreColors(display, colormap, color, ncolors);
		endcall(display);
		if (geterr() != BadAccess) {
			report("Got %s, Expecting BadAccess", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadAccess)
			CHECK;
	}

	CHECKPASS(6*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XStoreColors-7.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	XStoreColors(display, colormap, color, ncolors);
	endcall(display);
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/*
 * Assertion XStoreColors-8.(D)
 *   Reason: There is no known reliable test method for this assertion
 * When rmask_return, gmask_return and bmask_return have been
 * returned by a previous call to XAllocColorPlanes and a
 * read-write colourmap entry for a pixel value is changed by a
 * call to XStoreColors, then the pixel value is decomposed
 * into three components using the masks and the independent
 * colourmap entries are updated.
 */
static void
t008()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XStoreColors-9.(C)
 * If the visual class DirectColor is supported: When the
 * colormap argument was created with visual type DirectColor
 * and with alloc set to AllocAll, and a read-write colourmap
 * entry for a pixel value is changed by a call to XStoreColors,
 * then the pixel value is decomposed into three components
 * using the red_mask, green_mask and blue_mask in the visual
 * and the independent colourmap entries are updated.
 */
static void
t009()
{
XVisualInfo 	*vp;
unsigned long 	vmask = (1<<DirectColor);
XColor		*colors;
unsigned long	i;
XColor		*refp;
unsigned long	*reds, *greens, *blues;
int		pathcnt = 0;
static XColor triples[] = {
		(unsigned long)~0L, 0,0,0, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 1,0,0, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 0,1,0, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 0,0,1, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 3,2,1, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 1,3,0, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 7,3,1, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 7,8,5, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 15,11,12, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 42,99,13, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 112,127,64, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 255,64,33, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 64,255,33, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 64,33,255, DoRed|DoGreen|DoBlue, 0,
		(unsigned long)~0L, 255,255,255, DoRed|DoGreen|DoBlue, 0
	};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* If visual class DirectColor is supported: */
	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {

/*    Create a colourmap using makecolmap and AlocAll */
		colormap = makecolmap(display, vp->visual, AllocAll);
		ncolors = maxsize(vp);
/*    Allocate an array of XColor's and one for each of reds, greens & blues */
		colors = (XColor *) malloc(ncolors * sizeof(XColor));
		color = colors;
		reds = (unsigned long *) malloc(ncolors * sizeof(unsigned long));
		greens = (unsigned long *) malloc(ncolors * sizeof(unsigned long));
		blues = (unsigned long *) malloc(ncolors * sizeof(unsigned long));
		if ((colors == (XColor *) 0) || (reds == (unsigned long *) 0) ||
				(greens == (unsigned long *) 0) ||
				(blues == (unsigned long *) 0)) {
			delete("malloc() couldn't allocate %d cells for the XColor array.", ncolors);
			return;
		} else
			CHECK;

/*    Set up each of reds, greens & blues to have a distinguishable value, */
/*      using xname, and make sure that's what's in the equivalent colourmaps. */
		/*
		   kept in reds, greens or blues. This routine expects
		   color and ncolors to be set right. It uses XCall as well.
		*/
		if (!set_one_col(reds, "red", DoRed, vp->red_mask) ||
		    !set_one_col(greens, "green", DoGreen, vp->green_mask) ||
		    !set_one_col(blues, "blue", DoBlue, vp->blue_mask)
		   ) {
			delete("Failed to set up colourmap for test.");
			return;
		} else
			CHECK;

/*    Read back some suitably "random" triples and check their components */
		for (i=0, refp=triples; i < NELEM(triples); i++, refp++) {
			XColor testcol;
			unsigned long r,g,b;

			if (refp->red >= (unsigned)ncolors ||
					refp->green >= (unsigned)ncolors ||
					refp->blue >= (unsigned)ncolors)
				continue;

			r = refp->red << maskshift(vp->red_mask);
			if (!bitsubset(r, vp->red_mask))
				continue;
			g = refp->green << maskshift(vp->green_mask);
			if (!bitsubset(g, vp->green_mask))
				continue;
			b = refp->blue << maskshift(vp->blue_mask);
			if (!bitsubset(b, vp->blue_mask))
				continue;
			testcol.pixel = r | g | b;

			XQueryColor(display, colormap, &testcol);

/*    Verify that each component is the same as the corresponding entry in */
/*      reds, greens & blues. */
			if (testcol.flags != (DoRed | DoGreen | DoBlue) ||
			    testcol.red != reds[refp->red] ||
			    testcol.green != greens[refp->green] ||
			    testcol.blue != blues[refp->blue]) {
				report("Expected r,g,b = %lu, %lu, %lu but got %lu, %lu, %lu for pixel 0x%lx with r-,g-,b-masks 0x%lx, 0x%lx, 0x%lx",
					reds[refp->red], greens[refp->green], blues[refp->blue],
					testcol.red, testcol.green, testcol.blue,
					testcol.pixel, vp->red_mask,
					vp->green_mask, vp->blue_mask);
				FAIL;
			} else
				CHECK;

			pathcnt++;
		}

/*    Free the allocated storage: colors, reds, greens & blues. */
		free((char*)colors);
		free((char*)reds);
		free((char*)greens);
		free((char*)blues);
	}

	CHECKPASS(pathcnt+2*nsupvis());
			

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
