/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: strnmdclr.m,v 1.9 92/06/11 16:19:31 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XStoreNamedColor(display, colormap, color, pixel, flags)
 *   Display *display;
 *   Colormap colormap;
 *   char *color;
 *   unsigned long pixel;
 *   int flags;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XStoreNamedColor	1
char    *TestName = "XStoreNamedColor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XStoreNamedColor function
 */
static Display *display;
static Colormap colormap;
static char *color;
static unsigned long pixel;
static int flags;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
	color = "";
	pixel = 0;
	flags = DoRed|DoGreen|DoBlue;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XStoreNamedColor-1.(C)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: A call to XStoreNamedColor obtains
 * the closest available RGB values for the colormap argument
 * to those specified for the colour named color in the
 * database, and stores those values in the read-write
 * colourmap entry specified by the pixel component of the
 * XColor structure.
 */
static void
t001()
{
XVisualInfo *vp;
char *goodname;
XColor namedcol, testcol, storedcol;
unsigned long vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	if((goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}

	flags = DoRed|DoGreen|DoBlue;
	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting XStoreNamedColor() for class %s, color %s", displayclassname(vp->class), goodname);
		colormap = makecolmap(display, vp -> visual, AllocAll);
		color = goodname;
		pixel = 0L;

		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		namedcol.pixel = 0L;
		XQueryColor(display, colormap, &namedcol);
		testcol = namedcol;
		testcol.pixel = 1L;
		testcol.flags = flags;
		XStoreColor(display, colormap, &testcol);
		storedcol.pixel = 1L;
		storedcol.flags = flags;
		XQueryColor(display, colormap, &storedcol);

		trace("Named Color : pixel %lu, r %u g %u b %u", namedcol.pixel , namedcol.red, namedcol.green, namedcol.blue);
		trace("Stored Color: pixel %lu, r %u g %u b %u", storedcol.pixel, storedcol.red, storedcol.green, storedcol.blue);


		if( (namedcol.red != storedcol.red) || (namedcol.green != storedcol.green) || (namedcol.blue != storedcol.blue) ) {
			report("XStoreNamedColor() return RGB values r %u g %u b %u instead of r %u g %u b %u.", 
				namedcol.red, namedcol.green, namedcol.blue, storedcol.red, storedcol.green, storedcol.blue);
				FAIL;
		} else
			CHECK;
	}

	CHECKPASS(nsupvis());


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XStoreNamedColor-2.(A)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: A call to XStoreNamedColor changes
 * the red, green and blue values in the read-write colourmap
 * entry in accordance with the flags argument.
 */
static void
t002()
{
XVisualInfo *vp;
int 		i;
char 		*goodname;
XColor		namedcol, testcol, storedcol;
unsigned long 	vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
unsigned short	redval, greenval, blueval;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	if((goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}

	flags = DoRed|DoGreen|DoBlue;
	color = goodname;
	pixel = 0L;
	storedcol.pixel = 0L;
	testcol.pixel = 0L;
	namedcol.pixel = 0L;

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting XStoreNamedColor() for class %s, color %s", displayclassname(vp->class), goodname);
		colormap = makecolmap(display, vp -> visual, AllocAll);
		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		XQueryColor(display, colormap, &namedcol);

		testcol = namedcol;
		testcol.red ^= 0xffff;
		testcol.green ^= 0xffff;
		testcol.blue ^= 0xffff;
		testcol.flags = DoRed | DoGreen | DoBlue;

		for(i=0; i<8; i++) {

			flags = 0;

			if(i&DoRed) {
				flags |= DoRed;
				redval = namedcol.red;
			} else
				redval = testcol.red;

			if(i&DoGreen) {
				flags |= DoGreen;
				greenval = namedcol.green;
			} else
				greenval = testcol.green;

			if(i&DoBlue) {
				flags |= DoBlue;
				blueval = namedcol.blue;
			} else
				blueval = testcol.green;

			XStoreColor(display, colormap, &testcol);
			startcall(display);
			if (isdeleted())
				return;
			XStoreNamedColor(display, colormap, color, pixel, flags);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			XQueryColor(display, colormap, &storedcol);

			if( (redval != storedcol.red) || (greenval != storedcol.green) || (blueval != storedcol.blue) ){
				report("XStoreNamedColor() RGB value r %u g %u b %u instead of r %u g %u b %u.", 
					storedcol.red, storedcol.green, storedcol.blue, redval, greenval, blueval);
				FAIL;
				}
		}

		if(i == 8)
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XStoreNamedColor-3.(A)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: Upper and lower case characters in
 * the color argument refer to the same color.
 */
static void
t003()
{
XVisualInfo *vp;
char *cp, *goodname, *casename;
XColor casecol, testcol;
unsigned long vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
unsigned short trunc;
int		i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	if( (goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}


	casename = (char *) malloc( strlen(goodname) + 1);
	strcpy(casename, goodname);

	for(i=0, cp=casename; *cp; i++)
		if(i&1)
			*cp++ = tolower(*cp);
		else
			*cp++ = toupper(*cp);


	testcol.pixel = 0L;
	casecol.pixel = 0L;
	pixel = 0L;
	flags = DoRed|DoGreen|DoBlue;

	for(resetsupvis(vmask); nextsupvis(&vp); ) {

		colormap = makecolmap(display, vp -> visual, AllocAll);
		color = goodname;

		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
			
		XQueryColor(display, colormap, &testcol);

	
		color = casename;
		trace("Testing colourname %s", color);
		
		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		
		XQueryColor(display, colormap, &casecol);

		if((casecol.pixel != testcol.pixel) || (casecol.red != testcol.red) || (casecol.green != testcol.green) || (casecol.blue != testcol.blue)) {
			report("Colour name %s pixel %lu ( r %u g %u b %u) instead of pixel %lu ( r %u g %u b %u)",
				goodname, casecol.pixel, casecol.red, casecol.green, casecol.blue,
					  testcol.pixel, testcol.red, testcol.green, testcol.blue);
			FAIL;
		} else
			CHECK;

	}
	CHECKPASS(nsupvis());


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XStoreNamedColor-4.(A)
 * If any of the visual classes DirectColor, PseudoColor or
 * GrayScale is supported: When the pixel argument is not a
 * valid entry in the colormap argument, then a BadValue error
 * occurs.
 */
static void
t004()
{
XVisualInfo *vp;
char *goodname;
XColor namedcol, testcol, storedcol;
unsigned long vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	if((goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}

	flags = DoRed|DoGreen|DoBlue;
	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting XStoreNamedColor() for class %s, color %s", displayclassname(vp->class), goodname);
		colormap = makecolmap(display, vp -> visual, AllocAll);
		color = goodname;
		pixel = maxsize(vp) + 1;
		if (vp->class == DirectColor) {
			pixel = (vp->red_mask | vp->green_mask | vp->blue_mask);
			pixel |= pixel << 1;
		}
		trace("Pixel set to %ld", pixel);
		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadValue)
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAcc3.mc */
/*
 * Assertion XStoreNamedColor-5.(A)
 * When an attempt to store into a read-only or an unallocated
 * colormap entry is made, then a BadAccess error occurs.
 */
/* End of included file error/EAcc3.mc */

static void
t005()
{
XVisualInfo	*vp;
XColor		testcol;
char 		*goodname;
Display		*disp2;
unsigned long vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		UNSUPPORTED;
		return;
	}

	if((goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}

	pixel = 0L;
	color = goodname;
	for(resetsupvis(vmask); nextsupvis(&vp); ) {		

		colormap = makecolmap(display, vp->visual, AllocNone);
		trace("Testing a colormap with no allocated cells.");
		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);
		endcall(display);
		if (geterr() != BadAccess) {
			report("Got %s, Expecting BadAccess", errorname(geterr()));
			FAIL;
		}

		if(geterr() == BadAccess)
			CHECK;

		disp2 = opendisplay();

		testcol.flags = DoRed;
		testcol.red = 65535;
		if(XAllocColor(disp2, colormap, &testcol) == False) {
			delete("XAllocColor() failed to allocate a full red r/o cell for a second client.");
			return;
		} else
			CHECK;

		trace("Trying to write into a r/o cell allocated by another client.");
		pixel = testcol.pixel;
		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);
		endcall(display);
		if (geterr() != BadAccess) {
			report("Got %s, Expecting BadAccess", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadAccess)
			CHECK;

		testcol.red ^= 0xffff;
		testcol.green ^= 0xffff;
		testcol.blue ^= 0xffff;
		testcol.flags = flags;

		if(XAllocColor(display, colormap, &testcol) == False) {
			delete("XAllocColor() failed to allocate a r/o cell with r,g,b = 0x%x,0x%x,0x%x", testcol.red, testcol.green, testcol.blue);
			return;
		} else
			CHECK;

		trace("Trying to write into a r/o cell allocated by self.");
		pixel = testcol.pixel;
		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);
		endcall(display);
		if (geterr() != BadAccess) {
			report("Got %s, Expecting BadAccess", errorname(geterr()));
			FAIL;
		}

		if(geterr() == BadAccess)
			CHECK;
	}

	CHECKPASS(5*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XStoreNamedColor-6.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	XStoreNamedColor(display, colormap, color, pixel, flags);
	endcall(display);
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/* Including from file error/ENam2.mc */
/*
 * Assertion XStoreNamedColor-7.(A)
 * When the specified colour does not exist, then a BadName
 * error occurs.
 */
/* End of included file error/ENam2.mc */

static void
t007()
{
XVisualInfo *vp;
char *badname;
Status status;
unsigned long vmask = (1<<DirectColor)|(1<<PseudoColor)|(1<<GrayScale);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		unsupported("DirectColor, PseudoColor visual classes are not supported.");
		return;
	}

	if( (badname = tet_getvar("XT_BAD_COLORNAME")) == (char *) 0) {
		delete("XT_BAD_COLORNAME is not defined.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		colormap = makecolmap(display, vp -> visual, AllocAll);
		color = badname;		
		startcall(display);
		if (isdeleted())
			return;
		XStoreNamedColor(display, colormap, color, pixel, flags);		
		endcall(display);
		if (geterr() != BadName) {
			report("Got %s, Expecting BadName", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadName)
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XStoreNamedColor-8.(D)
 *   Reason: There is no known reliable test method for this assertion
 * When rmask_return, gmask_return and bmask_return have been
 * returned by a previous call to XAllocColorPlanes and a
 * read-write colourmap entry for a pixel value is changed by a
 * call to XStoreNamedColor, then the pixel value is
 * decomposed into three components using the masks and the
 * independent colourmap entries are updated.
 */
static void
t008()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XStoreNamedColor-9.(D)
 *   Reason: There is no known reliable test method for this assertion
 * If the visual class DirectColor is supported: When the
 * colormap argument was created with visual type DirectColor
 * and with alloc set to AllocAll, and a read-write colourmap
 * entry for a pixel value is changed by a call to
 * XStoreNamedColor, then the pixel value is decomposed into
 * three components using the red_mask, green_mask and
 * blue_mask in the visual and the independent colourmap
 * entries are updated.
 */
static void
t009()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
