/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: frcrsr.m,v 1.8 92/06/11 17:06:09 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XFreeCursor(display, cursor)
 *   Display *display;
 *   Cursor cursor;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFreeCursor	1
char    *TestName = "XFreeCursor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_CURSOR cursor


/*
 * Arguments to the XFreeCursor function
 */
static Display *display;
static Cursor cursor;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	cursor = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	cursor = ErrdefCursor;
}

/*
 * Assertion XFreeCursor-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * When another resource ID references the cursor, then a
 * call to XFreeCursor deletes the association between the
 * cursor resource ID and the specified cursor.
 */
static void
t001()
{
Cursor cursor2;
Window w;
XVisualInfo *vp;
unsigned int shape;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	/* UNSUPPORTED is not allowed */
	shape = config.fontcursor_good;
	if (shape == -1) {
		delete("A value of UNSUPPORTED is not allowed for XT_FONTCURSOR_GOOD");
		return;
	}

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
/* Create cursor and cursor2 as same cursors. */
		cursor = XCreateFontCursor(display, shape);
		cursor2 = XCreateFontCursor(display, shape);

/* Create window. */
		w = makewin(display, vp);

/* Define cursor for window. */
		XDefineCursor(display, w, cursor);

/* Call XFreeCursor with cursor. */
		startcall(display);
		if (isdeleted())
			return;
		XFreeCursor(display, cursor);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (geterr() == Success)
			CHECK;
		else
			FAIL;

/* Call XFreeCursor with cursor2. */
		cursor = cursor2;
		startcall(display);
		if (isdeleted())
			return;
		XFreeCursor(display, cursor);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (geterr() == Success)
			CHECK;
		else
			FAIL;
	}
	
	CHECKUNTESTED(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeCursor-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When no other resource ID references the cursor, then a
 * call to XFreeCursor deletes the association between the
 * cursor resource ID and the specified cursor, and the cursor
 * storage is freed.
 */
static void
t002()
{
Cursor cursor2;
unsigned int shape;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/* UNSUPPORTED is not allowed */
	shape = config.fontcursor_good;
	if (shape == -1) {
		delete("A value of UNSUPPORTED is not allowed for XT_FONTCURSOR_GOOD");
		return;
	}

/* Create cursor and cursor2 as same cursors. */
	cursor = XCreateFontCursor(display, shape);
	cursor2 = XCreateFontCursor(display, shape);

/* Call XFreeCursor with cursor. */
	startcall(display);
	if (isdeleted())
		return;
	XFreeCursor(display, cursor);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (geterr() == Success)
		CHECK;
	else
		FAIL;

/* Call XFreeCursor with cursor2. */
	cursor = cursor2;
	startcall(display);
	if (isdeleted())
		return;
	XFreeCursor(display, cursor);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (geterr() == Success)
		CHECK;
	else
		FAIL;
	
	CHECKUNTESTED(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECur.mc */
/*
 * Assertion XFreeCursor-3.(A)
 * When a cursor argument does not name a valid Cursor, then a
 * BadCursor error occurs.
 */
/* End of included file error/ECur.mc */

/* Including from file error/ECur.mc */
static void
t003()
{
unsigned int shape;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	/* UNSUPPORTED is not allowed */
	shape = config.fontcursor_good;
	if (config.fontcursor_good == -1) {
		delete("A value of UNSUPPORTED is not allowed for XT_FONTCURSOR_GOOD");
		return;
	}

	A_CURSOR = XCreateFontCursor(A_DISPLAY, shape);
	XFreeCursor(A_DISPLAY, A_CURSOR);

	startcall(display);
	if (isdeleted())
		return;
	XFreeCursor(display, cursor);
	endcall(display);
	if (geterr() != BadCursor) {
		report("Got %s, Expecting BadCursor", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadCursor)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECur.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
