/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: frfntinf.m,v 1.7 92/06/11 17:06:24 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XFreeFontInfo(names, free_info, actual_count)
 *   char	**names;
 *   XFontStruct	*free_info;
 *   int 	actual_count;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFreeFontInfo	1
char    *TestName = "XFreeFontInfo";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XFreeFontInfo function
 */
static char	**names;
static XFontStruct	*free_info;
static int 	actual_count;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	names = 0;
	free_info = 0;
	actual_count = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XFreeFontInfo-1.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When names is a list of font names and free_info is a
 * pointer to the font information both returned by a call to
 * XListFontsWithInfo, then a call to XFreeFontInfo frees
 * names, the font names specified by names and free_info.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	names = XListFontsWithInfo(Dsp, "xtfont?", 4, &actual_count, &free_info);
	if (names == NULL) {
		delete("No xtest fonts found, check that they have been installed");
		return;
	} else
		CHECK;

	startcall(Dsp);
	if (isdeleted())
		return;
	XFreeFontInfo(names, free_info, actual_count);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeFontInfo-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When names is NULL and free_info is a pointer to the font
 * information structure returned by an Xlib routine, then a
 * call to XFreeFontInfo frees the storage associated with
 * free_info.
 */
static void
t002()
{
XFontStruct	*fsp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	fsp = XLoadQueryFont(Dsp, "xtfont0");
	if (fsp == NULL) {
		delete("Could not open xtfont0, check that xtest fonts are installed");
		return;
	}

	names = NULL;
	free_info = fsp;
	actual_count = 1;
	startcall(Dsp);
	if (isdeleted())
		return;
	XFreeFontInfo(names, free_info, actual_count);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
