/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ldfnt.m,v 1.8 92/06/11 17:07:22 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Font
 *   XLoadFont(display, name)
 *   Display	*display;
 *   char	*name;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XLoadFont	1
char    *TestName = "XLoadFont";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XLoadFont function
 */
static Display	*display;
static char	*name;


#define TEXTSTRING "AbyZ%~"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	name = "xtfont1";
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XLoadFont-1.(A)
 * When name is a NULL terminated string in ISO Latin-1
 * encoding, then a call to XLoadFont loads the specified font
 * and returns its font ID.
 */
static void
t001()
{
Font	font;
int 	i;
char	fname[64];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (i = 0; i < XT_NFONTS; i++) {
		sprintf(fname, "xtfont%d", i);
		trace("Loading font %s", fname);
		name = fname;

		startcall(display);
		if (isdeleted())
			return;
		font = XLoadFont(display, name);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (geterr() != Success) {
			report("font %s could not be loaded", fname);
			report("Check that xtest fonts have been installed");
			FAIL;
		} else if (font == 0 || (font & 0xc0000000)) {
			report("An invalid font ID was returned");
			FAIL;
		} else
			CHECK;
	}
	CHECKPASS(XT_NFONTS);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XLoadFont-2.(A)
 * The font ID returned by a call to XLoadFont is usable on
 * any GC created for any screen of the display.
 */
static void
t002()
{
XVisualInfo	*vp;
Font	font;
char	*fname = "xtfont1";
Drawable d;
GC	gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		name = fname;

		startcall(display);
		if (isdeleted())
			return;
		font = XLoadFont(display, name);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (geterr() != Success) {
			/* Already done after calling XLoadFont */
			report("font %s could not be loaded", fname);
			report("Check that xtest fonts have been installed");
			FAIL;
		} else {
			d = makewin(display, vp);
			gc = makegc(display, d);
			XSetFont(display, gc, font);
			XDrawString(display, d, gc, 20, 20, "z", 1);
			PIXCHECK(display, d);
			dclear(display, d);
			XUnloadFont(display, font);
		}
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XLoadFont-3.(A)
 * Upper and lower case characters in the name argument refer
 * to the same font.
 */
static void
t003()
{
Font	font;
static char	*names[] = {
	"xtfont1",
	"Xtfont1",
	"XTFONT1",
	"xTfOnT1",
	"XtFoNt1",
};
int 	i;
Drawable	draw;
XImage	*sav;
GC		gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	name = "xtfont1";
	startcall(display);
	if (isdeleted())
		return;
	font = XLoadFont(display, name);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (geterr() != Success) {
		/* Test has already failed */
		delete("Font could not be loaded");
		return;
	}

	draw = defdraw(display, VI_WIN_PIX);
	gc = makegc(display, draw);
	XSetFont(display, gc, font);

	XDrawString(display, draw, gc, 30, 30, TEXTSTRING, strlen(TEXTSTRING));
	sav = savimage(display, draw);

	if (isdeleted())
		return;

	for (i = 0; i < NELEM(names); i++) {

		dclear(display, draw);
		name = names[i];
		startcall(display);
		if (isdeleted())
			return;
		font = XLoadFont(display, name);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (geterr() == Success) {
			XSetFont(display, gc, font);
			XDrawString(display, draw, gc, 30, 30, TEXTSTRING, strlen(TEXTSTRING));
			if (compsavimage(display, draw, sav))
				CHECK;
			else {
				report("Font %s was different to xtfont1", name);
				FAIL;
			}
		} else {
			report("Font load failed with name %s", name);
			FAIL;	/* Has been done already */
		}
	}
	CHECKPASS(NELEM(names));
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ENam1.mc */
/*
 * Assertion XLoadFont-4.(A)
 * When the specified font does not exist, then a BadName
 * error occurs.
 */
/* End of included file error/ENam1.mc */

/* Including from file error/ENam1.mc */
static void
t004()
{
Font	font;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/* Note:  the argument has to be called 'name' */
	name = config.bad_font_name;

	startcall(display);
	if (isdeleted())
		return;
	font = XLoadFont(display, name);
	endcall(display);
	if (geterr() != BadName) {
		report("Got %s, Expecting BadName", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadName)
		PASS;
	else
		FAIL;	/* For safty, this should already have been issued */
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ENam1.mc */

/* Including from file error/EAll.mc */
/*
 * Assertion XLoadFont-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
