/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: lstfntswth.m,v 1.10 92/06/11 17:07:59 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char **
 *   XListFontsWithInfo(display, patternarg, maxnames, count_return, info_return)
 *   Display	*display;
 *   char	*patternarg;
 *   int 	maxnames;
 *   int 	*count_return;
 *   XFontStruct	**info_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XListFontsWithInfo	1
char    *TestName = "XListFontsWithInfo";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XListFontsWithInfo function
 */
static Display	*display;
static char	*patternarg;
static int 	maxnames;
static int 	*count_return;
static XFontStruct	**info_return;


#include    <ctype.h>
#include    <string.h>

static	int 	count;
static	XFontStruct *finfo;

extern	XFontStruct	xtfont0;
extern	char	*xtfont0cpright;
extern	struct	fontinfo	fontinfo[];
extern	int 	nfontinfo;

static lowerstring(str)
char    *str;
{
	for (; *str; str++) {
		if (isupper(*str))
			tolower(*str);
	}
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	patternarg = 0;
	maxnames = 9999;
	count_return = &count;
	info_return = &finfo;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XListFontsWithInfo-1.(A)
 * When the pattern argument is a string in ISO Latin-1
 * encoding terminated with ASCII nul, then a call to
 * XListFontsWithInfo returns an array of strings terminated
 * with ASCII nul which are available font names that match the
 * pattern argument and returns the number of fonts in the
 * count_return argument and their associated font information
 * excluding the per-character metrics in the info_return
 * argument.
 */
static void
t001()
{
char    **names;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	patternarg = "xtfont0";
	startcall(display);
	if (isdeleted())
		return;
	names = XListFontsWithInfo(display, patternarg, maxnames, count_return, info_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (count != 1) {
		report("count_return was %d, expecting 1", count);
		FAIL;
	} else
		CHECK;

	if (names == NULL) {
		report("NULL was returned");
		FAIL;
		return;
	} else
		CHECK;

	lowerstring(*names);
	if (strcmp(*names, patternarg) != 0) {
		report("returned name was '%s', expecting '%s'", *names, patternarg);
		FAIL;
	} else
		CHECK;

	xtfont0.per_char = NULL;
	if (checkfsp(finfo, &xtfont0, xtfont0cpright))
		CHECK;
	else {
		report("font information was incorrect");
		FAIL;
	}

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListFontsWithInfo-2.(A)
 * Upper and lower case character in the pattern argument
 * refer to the same font.
 */
static void
t002()
{
char	**names;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	patternarg = "XtFoNt0";
	startcall(display);
	if (isdeleted())
		return;
	names = XListFontsWithInfo(display, patternarg, maxnames, count_return, info_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (count != 1) {
		report("count_return was %d, expecting 1", count);
		FAIL;
	} else
		CHECK;

	if (names == NULL) {
		report("NULL was returned");
		FAIL;
		return;
	} else
		CHECK;

	lowerstring(*names);
	patternarg = "xtfont0";
	if (strcmp(*names, patternarg) != 0) {
		report("returned name was '%s', expecting '%s'", *names, patternarg);
		FAIL;
	} else
		CHECK;

	xtfont0.per_char = NULL;
	if (checkfsp(finfo, &xtfont0, xtfont0cpright))
		CHECK;
	else {
		report("font infomation was incorrect");
		FAIL;
	}

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListFontsWithInfo-3.(A)
 * Each asterisk (*) in the string is a wildcard for any
 * number of characters.
 */
static void
t003()
{
char	**names;
char	*curname;
int 	i;
int 	j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	patternarg = "x*t*t*";
	startcall(display);
	if (isdeleted())
		return;
	names = XListFontsWithInfo(display, patternarg, maxnames, count_return, info_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (count < XT_NFONTS) {
		report("returned count was %d, expecting at least %d", count,XT_NFONTS);
		FAIL;
	} else
		CHECK;

	if (names == NULL) {
		report("returned name pointer was NULL");
		FAIL;
		return;
	} else
		CHECK;

	for (i = 0; i < count; i++) {
		curname = names[i];
		lowerstring(curname);
		for (j = 0; j < nfontinfo; j++) {
			if (strcmp(curname, fontinfo[j].name) == 0) {
				/* Check if we have already seen this one */
				if (fontinfo[j].flag) {
					report("name %s repeated in list", curname);
					FAIL;
				} else {
					fontinfo[j].flag = 1;

					/* Check font infomation */
					trace("checking font %s", curname);
					fontinfo[j].fontstruct->per_char = NULL;
					if (checkfsp(&finfo[i], fontinfo[j].fontstruct,
							*fontinfo[j].string)) {
						CHECK;
					} else {
						report("font information was incorrect");
						FAIL;
					}
				}
				break;
			}
		}
		if (j == nfontinfo) {
		char	*cp;

			/* It wasn't one of ours... */
			trace("extra font name %s found", curname);
			cp = strchr(curname, 'x');
			if (cp == NULL) {
				report("Found name %s that did not match patternarg");
				FAIL;
			} else if ((cp = strchr(cp, 't')) == NULL) {
				report("Found name %s that did not match patternarg");
				FAIL;
			} else if ((cp = strchr(cp, 't')) == NULL) {
				report("Found name %s that did not match patternarg");
				FAIL;
			} else {
				CHECK;
			}
		}
	}
	for (i = 0; i < nfontinfo; i++) {
		if (fontinfo[i].flag == 0) {
			report("xtest font '%s' was not found", fontinfo[i].name);
			FAIL;
		} else {
			/* reset to zero for next test */
			fontinfo[i].flag = 0;
			CHECK;
		}
	}

	CHECKPASS(2+count+nfontinfo);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListFontsWithInfo-4.(A)
 * Each question mark (?) in the string is a wildcard for a
 * single character.
 */
static void
t004()
{
char	**names;
char	*curname;
int 	i;
int 	j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	patternarg = "x?f?nt?";
	startcall(display);
	if (isdeleted())
		return;
	names = XListFontsWithInfo(display, patternarg, maxnames, count_return, info_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (count < XT_NFONTS) {
		report("returned count was %d, expecting at least %d", count,XT_NFONTS);
		FAIL;
	} else
		CHECK;

	if (names == NULL) {
		report("returned name pointer was NULL");
		FAIL;
		return;
	} else
		CHECK;

	for (i = 0; i < count; i++) {
		curname = names[i];
		lowerstring(curname);
		for (j = 0; j < nfontinfo; j++) {
			if (strcmp(curname, fontinfo[j].name) == 0) {
				/* Check if we have already seen this one */
				if (fontinfo[j].flag) {
					report("name %s repeated in list", curname);
					FAIL;
				} else {
					fontinfo[j].flag++;

					trace("checking font %s", curname);
					fontinfo[j].fontstruct->per_char = NULL;
					if (checkfsp(&finfo[i], fontinfo[j].fontstruct,
							*fontinfo[j].string)) {
						CHECK;
					} else {
						report("font information was incorrect");
						FAIL;
					}
				}
				break;
			}
		}
		if (j == nfontinfo) {
		int 	len;

			/* It wasn't one of ours... */
			trace("extra font name %s found", curname);
			len = strlen(patternarg);
			if (strlen(curname) != len) {
				report("name '%s' did not match patternarg");
				FAIL;
			} else {
				for (j = 0; j < len; j++) {
					if (patternarg[j] == '?')
						continue;
					if (curname[j] != patternarg[j]) {
						report("name '%s' did not match patternarg");
						FAIL;
						break;
					}
				}
				if (j == len)
					CHECK;
			}
		}
	}
	for (i = 0; i < nfontinfo; i++) {
		if (fontinfo[i].flag == 0) {
			report("xtest font '%s' was not found", fontinfo[i].name);
			FAIL;
		} else {
			/* reset to zero for next test */
			fontinfo[i].flag = 0;
			CHECK;
		}
	}
	CHECKPASS(2+count+nfontinfo);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListFontsWithInfo-5.(A)
 * The number of fonts returned in the count_return argument
 * will not exceed ma XListFontsWithInfo s.
 */
static void
t005()
{
char	**names;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	maxnames = 1;
	patternarg = "*";

	startcall(display);
	if (isdeleted())
		return;
	names = XListFontsWithInfo(display, patternarg, maxnames, count_return, info_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (count > maxnames) {
		report("returned count was %d, not expecting it to exceed %d", count, maxnames);
		FAIL;
	} else {
		PASS;
	}

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListFontsWithInfo-6.(A)
 * When there are no available font names that match the
 * pattern argument, then a call to XListFontsWithInfo returns
 * NULL.
 */
static void
t006()
{
char	**names;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	patternarg = "xtfont non existant name";

	startcall(display);
	if (isdeleted())
		return;
	names = XListFontsWithInfo(display, patternarg, maxnames, count_return, info_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (names != NULL) {
		report("returned pointer was not NULL");
		FAIL;
	} else
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XListFontsWithInfo-7.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
