/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: qryfnt.m,v 1.8 92/06/11 17:08:52 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XFontStruct *
 *   XQueryFont(display, font_ID)
 *   Display	*display;
 *   XID 	font_ID;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XQueryFont	1
char    *TestName = "XQueryFont";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XQueryFont function
 */
static Display	*display;
static XID 	font_ID;


extern	int 	checkfsp();
extern	struct	fontinfo fontinfo[];
extern	int 	nfontinfo;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	font_ID = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XQueryFont-1.(A)
 * When the font_ID argument is a valid Font resource, then a
 * call to XQueryFont returns a pointer to an XFontStruct
 * structure which contains information on the font with font
 * ID font_ID.
 */
static void
t001()
{
XFontStruct	*fsp;
Font	font;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (i = 0; i < nfontinfo; i++) {
		trace("Loading font %s", fontinfo[i].name);
		font = XLoadFont(Dsp, fontinfo[i].name);
		if (isdeleted())
			return;

		font_ID = font;
		startcall(display);
		if (isdeleted())
			return;
		fsp = XQueryFont(display, font_ID);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkfsp(fsp, fontinfo[i].fontstruct, *fontinfo[i].string))
			CHECK;
		else {
			report("Returned XFontStruct was incorrect");
			FAIL;
		}
	}
	CHECKPASS(nfontinfo);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XQueryFont-2.(A)
 * When the font_ID argument specifies a GContext, then a
 * call to XQueryFont returns a pointer to an XFontStruct
 * structure which contains information on the font in the
 * corresponding GC's font field.
 */
static void
t002()
{
XFontStruct	*fsp;
Font	font;
GC		gc;
Drawable	d;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	d = defdraw(Dsp, VI_WIN_PIX);
	gc = makegc(Dsp, d);
	if (isdeleted())
		return;

	for (i = 0; i < nfontinfo; i++) {
		trace("Loading font %s", fontinfo[i].name);
		font = XLoadFont(Dsp, fontinfo[i].name);
		if (isdeleted())
			return;

		XSetFont(Dsp, gc, font);

		font_ID = XGContextFromGC(gc);
		startcall(display);
		if (isdeleted())
			return;
		fsp = XQueryFont(display, font_ID);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkfsp(fsp, fontinfo[i].fontstruct, *fontinfo[i].string))
			CHECK;
		else {
			report("Returned XFontStruct was incorrect");
			FAIL;
		}
	}
	CHECKPASS(nfontinfo);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XQueryFont-3.(A)
 * When the font_ID argument does not name a valid GContext or
 * Font resource, then a call to XQueryFont returns NULL.
 */
static void
t003()
{
Font	font;
XFontStruct	*fsp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	font_ID = badfont(Dsp);
	startcall(display);
	if (isdeleted())
		return;
	fsp = XQueryFont(display, font_ID);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (fsp != NULL) {
		report("A non-NULL pointer was returned");
		FAIL;
	} else
		PASS;
		
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
