/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: txtwdth16.m,v 1.5 92/06/11 17:09:52 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XTextWidth16(font_struct, string, count)
 *   XFontStruct	*font_struct;
 *   XChar2b	*string;
 *   int 	count;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XTextWidth16	1
char    *TestName = "XTextWidth16";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XTextWidth16 function
 */
static XFontStruct	*font_struct;
static XChar2b	*string;
static int 	count;


static int 	ValueReturn;


static	XChar2b	ch1[] = {
	{0, 34}, {0, 54}, {0, 89}, {0, 100}, {0, 130}, {0, 201}, {0, 234}, {0, 255},
	{1, 34}, {1, 54}, {1, 89}, {1, 100}, {1, 130}, {1, 201}, {1, 234}, {1, 255},
	{2, 34}, {2, 54}, {2, 89}, {2, 100}, {2, 130}, {2, 201}, {2, 234}, {2, 255},
	{33, 34}, {33, 54}, {33, 89}, {33, 100}, {33, 130}, {33, 201}, {33, 234}, {33, 255},
	{36, 34}, {36, 54}, {36, 89}, {36, 100}, {36, 130}, {36, 201}, {36, 234}, {36, 255},
	{38, 34}, {38, 54}, {38, 89}, {38, 100}, {38, 130}, {38, 201}, {38, 234}, {38, 255},
	{48, 34}, {48, 54}, {48, 89}, {48, 100}, {48, 130}, {48, 201}, {48, 234}, {48, 255},
	{120, 34}, {120, 54}, {120, 89}, {120, 100}, {120, 130}, {120, 201}, {120, 234}, {120, 255},
};

extern	struct	fontinfo	fontinfo[];
extern	int 	nfontinfo;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	font_struct = 0;
	string = ch1;
	count = NELEM(ch1);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XTextWidth16-1.(A)
 * A call to XTextWidth16 returns the sum of the
 * character-width metrics of all characters in the 16-bit or
 * 2-byte character string, string, as rendered in the font
 * referenced by font_struct.
 */
static void
t001()
{
int 	i;
int 	width;
int 	calcwidth;
int 	n16;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	n16 = 0;

	for (i = 0; i < nfontinfo; i++) {
		font_struct = fontinfo[i].fontstruct;
		if (font_struct->max_byte1 == 0)
			continue;

		n16++;

		startcall(Dsp);
		if (isdeleted())
			return;
		width = XTextWidth16(font_struct, string, count);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		calcwidth = txtwidth16(font_struct, ch1, count);

		if (calcwidth != width) {
			report("Font %s - width was %d, expecting %d", fontinfo[i].name,
					width, calcwidth);
			FAIL;
		} else
			CHECK;
	}
	CHECKPASS(n16);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTextWidth16-2.(B)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * When the font is defined with linear indexing rather than
 * 2-byte matrix indexing, then each XChar2b structure is
 * interpreted as a 16-bit number with byte1 as the
 * most-significant byte.
 */
static void
t002()
{
	report("The statement in the X11 specification is not specific enough to write a test");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
