/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: chngactvpn.m,v 1.6 92/06/11 17:11:34 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XChangeActivePointerGrab(display, event_mask, cursor, test_time)
 *   Display	*display;
 *   unsigned int 	event_mask;
 *   Cursor	cursor;
 *   Time	test_time;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XChangeActivePointerGrab	1
char    *TestName = "XChangeActivePointerGrab";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_CURSOR cursor


/*
 * Arguments to the XChangeActivePointerGrab function
 */
static Display	*display;
static unsigned int 	event_mask;
static Cursor	cursor;
static Time	test_time;


/* check to see if pointer is grabbed. Use existing win and another client */
/* Depending on owner_events in xname or when setting grab may cause
 * events to appear on grabbing clients event queue. Before checking for
 * events in the queue the grabbing client should have flushed any junk
 * left by this function.
 */
static int pgrabbed(win)
Window win;
{
	PointerPlace *p;
	XEvent ev;
	Display *client2 = opendisplay();

	XSelectInput(client2, win, PointerMotionMask);
	XSync(client2, True);
	p = warppointer(client2, win, 1,1);
	(void) warppointer(client2, win, 10,10);
	(void) warppointer(client2, win, 1,1);
	(void) warppointer(client2, p->oroot, p->ox, p->oy);
	return (!getevent(client2, &ev));
}


/* Value list for use in test t005 */
static unsigned long	event_maskvallist[] = {
	ButtonPressMask,
	ButtonReleaseMask,
	EnterWindowMask,
	LeaveWindowMask,
	PointerMotionMask,
	PointerMotionHintMask,
	Button1MotionMask,
	Button2MotionMask,
	Button3MotionMask,
	Button4MotionMask,
	Button5MotionMask,
	ButtonMotionMask,
	KeymapStateMask,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	event_mask = PointerMotionMask;
	cursor = None;
	test_time = CurrentTime;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XChangeActivePointerGrab-1.(A)
 * When the pointer is actively grabbed by the client and the
 * specified time is not earlier than the last-pointer-grab
 * time nor later than the current X server time, then a call
 * to XChangeActivePointerGrab changes the specified dynamic
 * parameters.
 */
static void
t001()
{
XEvent	ev;
Window	win;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) warppointer(display, DRW(display), 0, 0);

	win = defwin(display);

	XGrabPointer(display, win, False, EnterWindowMask, GrabModeAsync,
		GrabModeAsync, None, None, CurrentTime);

	(void) warppointer(display, win, 0, 0);
	(void) warppointer(display, win, 2, 2);

	/*
	 * This is just a check to see that the grab is behaving as expected.
	 * delete and return if these fail.  We do the same checks
	 * after the parameters are changed.
	 */
	if (XCheckMaskEvent(display, EnterWindowMask, &ev))
		CHECK;
	else {
		delete("No enter event received when checking set up");
		return;
	}
	if (!XCheckMaskEvent(display, PointerMotionMask, &ev))
		CHECK;
	else {
		delete("A pointer motion event was received when checking set up");
		return;
	}

	(void) warppointer(display, DRW(display), 0, 0);

	startcall(display);
	if (isdeleted())
		return;
	XChangeActivePointerGrab(display, event_mask, cursor, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSync(display, True);	/* Flush any remaining event */
	(void) warppointer(display, win, 0, 0);
	(void) warppointer(display, win, 2, 2);
	if (isdeleted())
		return;

	if (!XCheckMaskEvent(display, EnterWindowMask, &ev))
		CHECK;
	else {
		report("Event mask was not changed to disallow enter events");
		FAIL;
	}
	if (XCheckMaskEvent(display, PointerMotionMask, &ev))
		CHECK;
	else {
		report("Event mask was not changed - no pointer motion event received");
		FAIL;
	}

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeActivePointerGrab-2.(A)
 * When the specified time is earlier than the
 * last-pointer-grab time or later than the current X server
 * time, then a call to XChangeActivePointerGrab does not
 * change the parameters.
 */
static void
t002()
{
XEvent	ev;
Window	win;
Time	t;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) warppointer(display, DRW(display), 0, 0);

	win = defwin(display);
	t = gettime(Dsp);

	XGrabPointer(display, win, False, EnterWindowMask, GrabModeAsync,
		GrabModeAsync, None, None, t);

	/* Earlier than last-pointer-grab time */
	test_time = t - 12;
	startcall(display);
	if (isdeleted())
		return;
	XChangeActivePointerGrab(display, event_mask, cursor, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSync(display, True);	/* Flush any event */
	(void) warppointer(display, win, 0, 0);
	(void) warppointer(display, win, 2, 2);
	if (isdeleted())
		return;

	if (XCheckMaskEvent(display, EnterWindowMask, &ev))
		CHECK;
	else {
		report("Event mask was changed for time earlier than last-pointer-grab time");
		FAIL;
	}
	if (!XCheckMaskEvent(display, PointerMotionMask, &ev))
		CHECK;
	else {
		report("Event mask was changed for time earlier than last-pointer-grab time");
		FAIL;
	}

	(void) warppointer(display, DRW(display), 0, 0);

	/* Later than X server time */
	t = gettime(Dsp);
	test_time = t + (config.speedfactor+1)*1000000;
	startcall(display);
	if (isdeleted())
		return;
	XChangeActivePointerGrab(display, event_mask, cursor, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSync(display, True);	/* Flush any event */
	(void) warppointer(display, win, 0, 0);
	(void) warppointer(display, win, 2, 2);
	if (isdeleted())
		return;

	if (XCheckMaskEvent(display, EnterWindowMask, &ev))
		CHECK;
	else {
		report("Event mask was changed for time later than X server time");
		FAIL;
	}
	if (!XCheckMaskEvent(display, PointerMotionMask, &ev))
		CHECK;
	else {
		report("Event mask was changed for time later than X server time");
		FAIL;
	}

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeActivePointerGrab-3.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A call to XChangeActivePointerGrab has no effect on the
 * passive parameters of the function XGrabButton.
 */
static void
t003()
{
Window	win;
Window	curwin;
XEvent	ev;
int	n;
Cursor	cur;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	win = defwin(display);
	curwin = defwin(display);
	cur = makecur2(display);
	XDefineCursor(display, curwin, cur);
	XSelectInput(display, win, 0L);
	XGrabButton(display, AnyButton, 0, win, False, 0, GrabModeAsync,
		GrabModeAsync, None, None);
	if (isdeleted()) {
		delete("Failed to set passive button grab.");
		return;
	} else
		CHECK;
	XSync(display, True);
	(void) warppointer(display, win, 1,1);
	(void) warppointer(display, win, 10,10);
	(void) warppointer(display, win, 1,1);
	if (n=getevent(display, &ev)) {
		delete("Unexpectedly received %d events, first was type %s.",
				n, eventname(ev.type));
		return;
	} else
		CHECK;
	event_mask = PointerMotionMask;
	cursor = cur;
	startcall(display);
	if (isdeleted())
		return;
	XChangeActivePointerGrab(display, event_mask, cursor, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XSync(display, True);
	(void) warppointer(display, win, 1,1);
	(void) warppointer(display, win, 10,10);
	(void) warppointer(display, win, 1,1);
	if (n=getevent(display, &ev)) {
		report("event_mask changed passive grab, received %d events, first was type %s.",
				n, eventname(ev.type));
		FAIL;
	} else
		CHECK;
	if (noext(1)) {
		report("Tested as far as possible without activating grab.");
		CHECKUNTESTED(3);
	} else {
		Window	oldf;
		int	oldr;

		XGetInputFocus(display, &oldf, &oldr);
		XSetInputFocus(display, win, RevertToPointerRoot, CurrentTime);
		buttonpress(display, Button1);
		if (!pgrabbed(win)) {
			report("Failed to activate grab.");
			relalldev();
			XSetInputFocus(display, oldf, oldr, CurrentTime);
			return;
		}
		XSync(display, True);
		(void) warppointer(display, win, 1,1);
		(void) warppointer(display, win, 10,10);
		(void) warppointer(display, win, 1,1);
		if (n=getevent(display, &ev)) {
			report("event_mask changed on grab activation, received %d events, first was type %s.",
					n, eventname(ev.type));
			FAIL;
		} else
			CHECK;
		if (spriteiswin(display, curwin)) {
			report("Cursor changed before grab activated.");
			FAIL;
		} else
			CHECK;
		event_mask = PointerMotionMask;
		cursor = cur;
		startcall(display);
		if (isdeleted())
			return;
		XChangeActivePointerGrab(display, event_mask, cursor, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		XSync(display, True);
		(void) warppointer(display, win, 1,1);
		(void) warppointer(display, win, 10,10);
		(void) warppointer(display, win, 1,1);
		if (!(n=getevent(display, &ev)) || ev.type != MotionNotify) {
			report("event_mask not changed properly for active grab.");
			if (n)
				report("First event of %d was %s instead of MotionNotify.",
					n, eventname(ev.type));
			FAIL;
		} else
			CHECK;
		if (!spriteiswin(display, curwin)) {
			report("Cursor not changed although grab was active.");
			FAIL;
		} else
			CHECK;
		
		relalldev();
		XSetInputFocus(display, oldf, oldr, CurrentTime);
		CHECKPASS(7);
	}

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECur.mc */
/*
 * Assertion XChangeActivePointerGrab-4.(A)
 * When a cursor argument does not name a valid Cursor, then a
 * BadCursor error occurs.
 */
/* End of included file error/ECur.mc */

/* Including from file error/ECur.mc */
static void
t004()
{
unsigned int shape;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	/* UNSUPPORTED is not allowed */
	shape = config.fontcursor_good;
	if (config.fontcursor_good == -1) {
		delete("A value of UNSUPPORTED is not allowed for XT_FONTCURSOR_GOOD");
		return;
	}

	A_CURSOR = XCreateFontCursor(A_DISPLAY, shape);
	XFreeCursor(A_DISPLAY, A_CURSOR);

	startcall(display);
	if (isdeleted())
		return;
	XChangeActivePointerGrab(display, event_mask, cursor, test_time);
	endcall(display);
	if (geterr() != BadCursor) {
		report("Got %s, Expecting BadCursor", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadCursor)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECur.mc */

/*
 * Assertion XChangeActivePointerGrab-5.(A)
 * When the value of event_mask is not a bitwise combination
 * of ButtonPressMask, ButtonReleaseMask, EnterWindowMask,
 * LeaveWindowMask, PointerMotionMask, PointerMotionHintMask,
 * Button1MotionMask, Button2MotionMask, Button3MotionMask,
 * Button4MotionMask, Button5MotionMask, ButtonMotionMask or
 * KeymapStateMask, then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t005()
{
#undef	VALUE_ARG
#define	VALUE_ARG event_mask
#undef	VALUE_LIST
#define	VALUE_LIST event_maskvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE unsigned
#undef	NOTMEMBER
#define	NOTMEMBER notmaskmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeActivePointerGrab(display, event_mask, cursor, test_time);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
