/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: chngpntrcn.m,v 1.5 92/06/11 17:11:58 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold)
 *   Display	*display;
 *   Bool	do_accel;
 *   Bool	do_threshold;
 *   int 	accel_numerator;
 *   int 	accel_denominator;
 *   int 	threshold;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XChangePointerControl	1
char    *TestName = "XChangePointerControl";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XChangePointerControl function
 */
static Display	*display;
static Bool	do_accel;
static Bool	do_threshold;
static int 	accel_numerator;
static int 	accel_denominator;
static int 	threshold;



static int 	oan, oad;
static int 	othresh;

/*
 * Save the original pointer values for restoring at the end of the test.
 */
static void
saveold()
{
	startup();
	if(Dsp)
		XGetPointerControl(Dsp, &oan, &oad, &othresh);
}

static void
restorepoint()
{
	if(Dsp)
		XChangePointerControl(Dsp, True, True, oan, oad, othresh);
	cleanup();
}



/* Value list for use in test t010 */
static int 	do_accelvallist[] = {
	True,
	False,
};



/* Value list for use in test t011 */
static int 	do_thresholdvallist[] = {
	True,
	False,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	do_accel = False;
	do_threshold = False;
	accel_numerator = 0;
	accel_denominator = 0;
	threshold = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XChangePointerControl-1.(A)
 * When do_accel is True, then the acceleration of the
 * pointer is set to the fraction accel_numerator over
 * accel_denominator.
 */
static void
t001()
{
int 	newnum, newden, newthresh;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	do_accel = True;
	accel_numerator = 12;
	accel_denominator = 11;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newnum == accel_numerator)
		CHECK;
	else {
		report("accel_numerator was %d, expecting %d", newnum, accel_numerator);
		FAIL;
	}
	if (newden == accel_denominator)
		CHECK;
	else {
		report("accel_denominator was %d, expecting %d", newden, accel_denominator);
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-2.(A)
 * When do_threshold is True, then the threshold is set to
 * the value in the threshold argument.
 */
static void
t002()
{
int 	newnum, newden, newthresh;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	do_threshold = True;
	threshold = 43;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newthresh == threshold)
		CHECK;
	else {
		report("threshold was %d, expecting %d", newthresh, threshold);
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-3.(A)
 * When threshold is -1, then the threshold is set to the
 * default.
 */
static void
t003()
{
int 	newnum, newden, newthresh;
int 	defnum, defden, defthresh;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	do_threshold = True;
	threshold = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &defnum, &defden, &defthresh);

	threshold = defthresh + 1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newthresh == threshold)
		CHECK;
	else {
		report("When setting a non-default threshold value,");
		delete("threshold was %d, expecting %d", newthresh, threshold);
		return;
	}

	threshold = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newthresh == defthresh)
		CHECK;
	else {
		report("When setting default threshold value,");
		report("threshold was %d, expecting %d", newthresh, defthresh);
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-4.(A)
 * When accel_numerator is -1, then the numerator of the
 * acceleration is set to the default.
 */
static void
t004()
{
int 	newnum, newden, newthresh;
int 	defnum, defden, defthresh;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	do_accel = True;
	accel_numerator = -1;
	accel_denominator = 11;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &defnum, &defden, &defthresh);

	accel_numerator = defnum + 1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newnum == accel_numerator)
		CHECK;
	else {
		report("When setting a non-default numerator value,");
		report("accel_numerator was %d, expecting %d", newnum, accel_numerator);
		FAIL;
	}

	accel_numerator = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newnum == defnum)
		CHECK;
	else {
		report("When setting default numerator value,");
		report("accel_numerator was %d, expecting %d", newnum, defnum);
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-5.(A)
 * When accel_denominator is -1, then the denominator of the
 * acceleration is set to the default.
 */
static void
t005()
{
int 	newnum, newden, newthresh;
int 	defnum, defden, defthresh;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	do_accel = True;
	accel_numerator = 12;
	accel_denominator = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &defnum, &defden, &defthresh);

	accel_denominator = defden + 1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newden == accel_denominator)
		CHECK;
	else {
		report("When setting a non-default denominator value,");
		report("accel_denominator was %d, expecting %d", newden, accel_denominator);
		FAIL;
	}

	accel_denominator = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newden == defden)
		CHECK;
	else {
		report("When setting default denominator value,");
		report("accel_denominator was %d, expecting %d", newden, defden);
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-6.(A)
 * When the value of the do_accel or do_threshold arguments
 * are False, then the corresponding parameter is not changed.
 */
static void
t006()
{
int 	oldnum, oldden, oldthresh;
int 	newnum, newden, newthresh;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	XGetPointerControl(display, &oldnum, &oldden, &oldthresh);

	accel_numerator = oldnum+4;
	accel_denominator = oldden+5;
	threshold = oldthresh+2;

	do_accel = False;
	do_threshold = False;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetPointerControl(display, &newnum, &newden, &newthresh);

	if (newnum == oldnum)
		CHECK;
	else {
		report("accel_numerator was changed when do_accel was False");
		FAIL;
	}
	if (newden == oldden)
		CHECK;
	else {
		report("accel_denominator was changed when do_accel was False");
		FAIL;
	}
	if (newthresh == oldthresh)
		CHECK;
	else {
		report("threshold was changed when do_threshold was False");
		FAIL;
	}

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-7.(A)
 * When threshold is a negative number other than -1, then a
 * BadValue error occurs.
 */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	do_threshold = True;
	threshold = -3;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-8.(A)
 * When accel_numerator or accel_denominator are negative
 * numbers other than -1, then a BadValue error occurs.
 */
static void
t008()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	do_accel = True;

	accel_denominator = 3;
	accel_numerator = -3;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		CHECK;

	accel_denominator = -3;
	accel_numerator = 2;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-9.(A)
 * When the accel_denominator argument is zero, then a
 * BadValue error occurs.
 */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	do_accel = True;
	accel_denominator = 0;
	accel_numerator = 1;

	startcall(display);
	if (isdeleted())
		return;
	XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangePointerControl-10.(A)
 * When the value of do_accel is other than True or False,
 * then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t010()
{
#undef	VALUE_ARG
#define	VALUE_ARG do_accel
#undef	VALUE_LIST
#define	VALUE_LIST do_accelvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/*
 * Assertion XChangePointerControl-11.(A)
 * When the value of do_threshold is other than True or False,
 * then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t011()
{
#undef	VALUE_ARG
#define	VALUE_ARG do_threshold
#undef	VALUE_LIST
#define	VALUE_LIST do_thresholdvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangePointerControl(display, do_accel, do_threshold, accel_numerator, accel_denominator, threshold);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = saveold;
void	(*tet_cleanup)() = restorepoint;
