/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: frcscrnsvr.m,v 1.5 92/06/11 17:12:44 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XForceScreenSaver(display, mode)
 *   Display	*display;
 *   int 	mode;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XForceScreenSaver	1
char    *TestName = "XForceScreenSaver";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XForceScreenSaver function
 */
static Display	*display;
static int 	mode;



/* Value list for use in test t003 */
static int 	modevallist[] = {
	ScreenSaverActive,
	ScreenSaverReset,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	mode = ScreenSaverReset;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XForceScreenSaver-1.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the specified mode is ScreenSaverActive and the screen
 * saver is currently deactivated, then a call to
 * XForceScreenSaver activates the screen saver even after the
 * screen saver has been disabled with a timeout of zero.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	mode = ScreenSaverActive;

	startcall(display);
	if (isdeleted())
		return;
	XForceScreenSaver(display, mode);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("There is no reliable test method - touch test for ScreenSaverActive");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XForceScreenSaver-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the specified mode is ScreenSaverReset and the screen
 * saver is currently enabled and activated, then a call to
 * XForceScreenSaver deactivates the screen saver and the
 * activation timer is reset to its initial state as though
 * device input had been received.
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	mode = ScreenSaverReset;

	startcall(display);
	if (isdeleted())
		return;
	XForceScreenSaver(display, mode);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("There is no reliable test method - touch test for ScreenSaverReset");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XForceScreenSaver-3.(A)
 * When the value of mode is other than ScreenSaverActive or
 * ScreenSaverReset, then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t003()
{
#undef	VALUE_ARG
#define	VALUE_ARG mode
#undef	VALUE_LIST
#define	VALUE_LIST modevallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XForceScreenSaver(display, mode);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
