/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: grbkybrd.m,v 1.7 92/06/11 17:13:17 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time)
 *   Display	*display;
 *   Window	grab_window;
 *   Bool	owner_events;
 *   int 	pointer_mode;
 *   int 	keyboard_mode;
 *   Time	test_time;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGrabKeyboard	1
char    *TestName = "XGrabKeyboard";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW grab_window
#define A_DRAWABLE grab_window


/*
 * Arguments to the XGrabKeyboard function
 */
static Display	*display;
static Window	grab_window;
static Bool	owner_events;
static int 	pointer_mode;
static int 	keyboard_mode;
static Time	test_time;


static int 	ValueReturn;


/*
 * Returns True if the keyboard is frozen.
 */
static int
iskfrozen()
{
XEvent	ev;
int 	res;
static int 	key;

	XSync(display, True); /* Flush previous events */
	key = getkeycode(display);

	/*
	 * Try to provoke a keypress on grab_window.
	 */
	(void) warppointer(display, grab_window, 1, 1);
	keypress(display, key);
	if (XCheckMaskEvent(display, (long)KeyPressMask, &ev))
		res = False;
	else
		res = True;

	return(res);
}



/*
 * Returns True if the pointer is frozen.
 */
static Bool
ispfrozen()
{
XEvent	ev;
Window	win;

	win = defwin(Dsp);
	(void) warppointer(Dsp, win, 0, 0);
	XSync(Dsp, True);	/* discard events */

	XSelectInput(Dsp, win, PointerMotionMask);

	(void) warppointer(Dsp, win, 1, 4);
	XSync(Dsp, False);

	if (XCheckWindowEvent(Dsp, win, PointerMotionMask, &ev))
		return(False);
	else
		return(True);
}



/* Value list for use in test t022 */
static int 	owner_eventsvallist[] = {
	True,
	False,
};



/* Value list for use in test t023 */
static int 	pointer_modevallist[] = {
	GrabModeSync,
	GrabModeAsync,
};



/* Value list for use in test t024 */
static int 	keyboard_modevallist[] = {
	GrabModeSync,
	GrabModeAsync,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	grab_window = defwin(display);
	owner_events = False;
	pointer_mode = GrabModeAsync;
	keyboard_mode = GrabModeAsync;
	test_time = CurrentTime;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGrabKeyboard-1.(A)
 * A successful call to XGrabKeyboard actively grabs control
 * of the keyboard and returns GrabSuccess.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ValueReturn == GrabSuccess)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-2.(A)
 * When the keyboard is grabbed, then FocusIn and FocusOut
 * events are generated as though the focus had changed from
 * the previous focus window to grab_window.
 */
static void
t002()
{
Window	win;
Window	ofocus;
XEvent	ev;
XFocusInEvent	figood;
XFocusOutEvent	fogood;
int 	orevert;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * Save current input focus to pose as little inconvenience as
	 * possible.
	 */
	XGetInputFocus(display, &ofocus, &orevert);

	win = defwin(display);
	XSetInputFocus(display, win, RevertToNone, CurrentTime);
	if (isdeleted()) {
		report("Could not set up focus");
		return;
	}

	XSelectInput(display, grab_window, FocusChangeMask);
	XSelectInput(display, win, FocusChangeMask);

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * Set up the expected good events.
	 */
	defsetevent(figood, display, FocusIn);
	figood.window = grab_window;
	figood.mode = NotifyGrab;
	figood.detail = NotifyNonlinear;

	defsetevent(fogood, display, FocusOut);
	fogood.window = win;
	fogood.mode = NotifyGrab;
	fogood.detail = NotifyNonlinear;

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Did not get expected FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;

	if (getevent(display, &ev) == 0 || ev.type != FocusIn) {
		report("Did not get expected FocusIn event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&figood, &ev))
		FAIL;
	else
		CHECK;

	/* Reset old focus */
	XSetInputFocus(display, ofocus, orevert, CurrentTime);

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-3.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When a successful call to XGrabKeyboard is made by a
 * client, then subsequent keyboard events are reported only to
 * that client.
 */
static void
t003()
{
Display	*client2;
XEvent	ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	if ((client2 = opendisplay()) == NULL) {
		tccabort("Could not open display (%s)", config.display);
		return;
	} else
		CHECK;

	XSelectInput(display, grab_window, KeyPressMask|KeyReleaseMask);
	XSelectInput(client2, grab_window, KeyPressMask|KeyReleaseMask);

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(client2, grab_window, 1, 1);
	keypress(display, getkeycode(display));

	if (XCheckMaskEvent(display, KeyPressMask, &ev))
		CHECK;
	else {
		report("Keyboard event not received on the grabbing client");
		FAIL;
	}
	if (XCheckMaskEvent(client2, KeyPressMask, &ev)) {
		report("Keyboard event was received on a non-grabbing client");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-4.(A)
 * A call to XGrabKeyboard overrides any active keyboard grab
 * by this client.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	pointer_mode = GrabModeAsync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen()) {
		delete("Could not setup grab");
		return;
	}

	pointer_mode = GrabModeSync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen())
		CHECK;
	else {
		report("A second grab did not override the first");
		FAIL;
	}

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-5.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When owner_events is False, then all generated keyboard
 * events are reported with respect to the grab_window.
 */
static void
t005()
{
Window	window2;
XEvent	ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	window2 = defwin(display);
	XSelectInput(display, grab_window, KeyPressMask|KeyReleaseMask);
	XSelectInput(display, window2, KeyPressMask|KeyReleaseMask);

	owner_events = False;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(display, window2, 1, 1);
	keypress(display, getkeycode(display));

	if (XCheckWindowEvent(display, grab_window, KeyPressMask, &ev))
		CHECK;
	else {
		report("Event was not reported on grab_window");
		FAIL;
	}
	if (XCheckWindowEvent(display, window2, KeyPressMask, &ev)) {
		report("Event was reported on the owner window");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-6.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When owner_events is True and a keyboard event is generated
 * that would normally be reported to the client, then it is
 * reported on the window it would normally be reported on.
 */
static void
t006()
{
Window	window2;
XEvent	ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	window2 = defwin(display);
	XSelectInput(display, grab_window, KeyPressMask|KeyReleaseMask);
	XSelectInput(display, window2, KeyPressMask|KeyReleaseMask);

	owner_events = True;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(display, window2, 1, 1);
	keypress(display, getkeycode(display));

	if (XCheckWindowEvent(display, window2, KeyPressMask, &ev))
		CHECK;
	else {
		report("Event was not reported on the owner window");
		FAIL;
	}
	if (XCheckWindowEvent(display, grab_window, KeyPressMask, &ev)) {
		report("Event was reported on the grab_window");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-7.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When owner_events is True, and a keyboard event is
 * generated that would not normally be reported to the client,
 * then it is reported on the grab_window.
 */
static void
t007()
{
Window	window2;
XEvent	ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	window2 = defwin(display);

	owner_events = True;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(display, window2, 1, 1);
	keypress(display, getkeycode(display));

	if (XCheckWindowEvent(display, grab_window, KeyPressMask, &ev))
		CHECK;
	else {
		report("Event was not reported on grab_window");
		FAIL;
	}
	if (XCheckWindowEvent(display, window2, KeyPressMask, &ev)) {
		report("Event was reported on the owner window");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-8.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the keyboard is grabbed, then KeyPress and KeyRelease
 * events are always reported, independent of any event
 * selection made by the client.
 */
static void
t008()
{
XEvent	ev;
int 	key;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(display, grab_window, 1, 1);
	key = getkeycode(display);
	keypress(display, key);

	if (XCheckMaskEvent(display, KeyPressMask, &ev))
		CHECK;
	else {
		report("KeyPress event was not reported during grab when not selected");
		FAIL;
	}
	keyrel(display, key);

	if (XCheckMaskEvent(display, KeyReleaseMask, &ev))
		CHECK;
	else {
		report("KeyRelease event was not reported during grab when not selected");
		FAIL;
	}

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-9.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When keyboard_mode is GrabModeAsync, then keyboard event
 * processing continues normally.
 */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	keyboard_mode = GrabModeAsync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (iskfrozen()) {
		report("Keyboard was frozen by GrabModeAsync");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-10.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When keyboard_mode is GrabModeAsync, and the keyboard is
 * currently frozen by this client, then processing of keyboard
 * events is resumed.
 */
static void
t010()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	XGrabPointer(display, grab_window, False, 0, GrabModeAsync, GrabModeSync,
		None, None, CurrentTime);

	keyboard_mode = GrabModeAsync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (iskfrozen()) {
		report("Keyboard was not released by GrabModeAsync");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-11.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When keyboard_mode is GrabModeSync, then the state of the
 * keyboard, as seen by client applications, appears to freeze
 * and no further keyboard events are generated until the
 * grabbing client issues a releasing XAllowEvents call or
 * until the keyboard grab is released.
 */
static void
t011()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	XSelectInput(display, grab_window, KeyPressMask);

	keyboard_mode = GrabModeSync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (iskfrozen())
		CHECK;
	else {
		report("Keyboard was not frozen with keyboard_mode GrabModeSync");
		FAIL;
	}

	XUngrabKeyboard(display, CurrentTime);
	XSync(display, False);
	if (iskfrozen()) {
		report("Keyboard was not thawed after the grab was released");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-12.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the keyboard is frozen, then the actual keyboard
 * changes are not lost while the keyboard is frozen and are
 * processed after the grab is released or the client calls
 * XAllowEvents.
 */
static void
t012()
{
int 	key;
int 	n;
int 	first;
XEvent	ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	key = getkeycode(display);
	XSelectInput(display, grab_window, KeyPressMask|KeyReleaseMask);

	keyboard_mode = GrabModeSync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(display, grab_window, 10, 10);
	keypress(display, key);
	keyrel(display, key);

	if (XCheckMaskEvent(display, KeyPressMask|KeyReleaseMask, &ev)) {
		delete("Events received while keyboard supposed to be frozen");
		return;
	} else
		CHECK;

	XUngrabKeyboard(display, test_time);
	if (isdeleted())
		return;

	n = getevent(display, &ev);
	if (n != 2) {
		report("Expecting two events to be released after grab");
		report("  got %d", n);
		FAIL;
	} else {
		first = ev.type;
		(void) getevent(display, &ev);
		/* (We have already checked that there is another event) */

		if (ev.type != KeyPress && first != KeyPress) {
			report("Did not get KeyPress event after releasing grab");
			FAIL;
		} else
			CHECK;
		if (ev.type != KeyRelease && first != KeyRelease) {
			report("Did not get KeyRelease event after releasing grab");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-13.(A)
 * When pointer_mode is GrabModeAsync, then pointer event
 * processing is unaffected by activation of the grab.
 */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	pointer_mode = GrabModeAsync;

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen() == False)
		CHECK;
	else {
		report("Pointer events were affected by GrabModeAsync");
		FAIL;
	}

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-14.(A)
 * When pointer_mode is GrabModeSync, then state of the
 * pointer as seen by client applications appears to freeze and
 * no further pointer events are generated until the grabbing
 * client issues a releasing XAllowEvents call or until the
 * keyboard grab is released.
 */
static void
t014()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	pointer_mode = GrabModeSync;

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen() == True)
		CHECK;
	else {
		report("Pointer events were not frozen by GrabModeSync");
		FAIL;
	}

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-15.(A)
 * When the pointer is frozen, then the actual pointer changes
 * are not lost and are processed after the grab is released or
 * the client calls XAllowEvents.
 */
static void
t015()
{
XEvent	ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) warppointer(display, grab_window, 0, 0);
	XSelectInput(display, grab_window, PointerMotionMask);

	pointer_mode = GrabModeSync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(display, grab_window, 1, 1);
	XSync(display, False);	/* warppointer has effectivly done this */
	if (!XCheckWindowEvent(display, grab_window, PointerMotionMask, &ev))
		CHECK;
	else {
		report("Events were received while pointer was frozen");
		FAIL;
	}

	XUngrabKeyboard(display, CurrentTime);
	XSync(display, False);

	if (XCheckWindowEvent(display, grab_window, PointerMotionMask, &ev))
		CHECK;
	else {
		report("Events were not saved while pointer was frozen");
		FAIL;
	}

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-16.(A)
 * When the event window for an active grab becomes not
 * viewable, then the grab is released automatically.
 */
static void
t016()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	pointer_mode = GrabModeSync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XUnmapWindow(display, grab_window);
	if (isdeleted())
		return;

	if (ispfrozen()) {
		report("Grab was not released when grab_window was unmapped");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-17.(A)
 * A successful call to XGrabKeyboard sets the
 * last-keyboard-grab time to the specified time, with
 * CurrentTime being replaced by the current X server time.
 */
static void
t017()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_time = gettime(display);
	pointer_mode = GrabModeSync;
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen())
		CHECK;
	else {
		delete("Could not freeze pointer");
		return;
	}

	XUngrabKeyboard(display, test_time-1);
	if (ispfrozen())
		CHECK;
	else {
		report("Last-keyboard-grab time not set correctly");
		FAIL;
	}
	XUngrabKeyboard(display, test_time);
	if (ispfrozen()) {
		report("Last-keyboard-grab time not set correctly");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-18.(A)
 * When the keyboard is actively grabbed by some other client,
 * then a call to XGrabKeyboard fails and returns
 * AlreadyGrabbed.
 */
static void
t018()
{
int 	ret;
Display	*client2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	client2 = opendisplay();

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	display = client2;
	startcall(display);
	if (isdeleted())
		return;
	ret = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ret == AlreadyGrabbed)
		CHECK;
	else {
		report("Return value was %s, expecting AlreadyGrabbed",
			grabreplyname(ret));
		FAIL;
	}

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-19.(A)
 * When the grab_window is not viewable, then a call to
 * XGrabKeyboard fails and returns GrabNotViewable.
 */
static void
t019()
{
int 	ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	XUnmapWindow(display, grab_window);

	startcall(display);
	if (isdeleted())
		return;
	ret = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ret == GrabNotViewable)
		CHECK;
	else {
		report("Return value was %s, expecting GrabNotViewable",
			grabreplyname(ret));
		FAIL;
	}

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-20.(A)
 * When the keyboard is frozen by an active grab of another
 * client, then a call to XGrabKeyboard fails and returns
 * GrabFrozen.
 */
static void
t020()
{
int 	ret;
Display	*client2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	client2 = opendisplay();

	display = Dsp;
	XGrabPointer(Dsp, grab_window, False, 0, GrabModeAsync, GrabModeSync,
		None, None, CurrentTime);
	if (isdeleted())
		return;

	display = client2;
	startcall(display);
	if (isdeleted())
		return;
	ret = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ret == GrabFrozen)
		CHECK;
	else {
		report("Return value was %s, expecting GrabFrozen",
			grabreplyname(ret));
		FAIL;
	}

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-21.(A)
 * When the specified time is earlier than the
 * last-keyboard-grab time or later than the current X server
 * time, then a call to XGrabKeyboard fails and returns
 * GrabInvalidTime.
 */
static void
t021()
{
int 	ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_time = gettime(display);

	/* Set the last-keyboard-grab time */
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XUngrabKeyboard(display, test_time);

	test_time -= 43;
	startcall(display);
	if (isdeleted())
		return;
	ret = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ret == GrabInvalidTime)
		CHECK;
	else {
		report("Trying time earlier than last-keyboard-grab time");
		report("Return value was %s, expecting GrabInvalidTime",
			grabreplyname(ret));
		FAIL;
	}
	XUngrabKeyboard(display, test_time);

	test_time = gettime(display);
	test_time += ((config.speedfactor+1) * 1000000);
	startcall(display);
	if (isdeleted())
		return;
	ret = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ret == GrabInvalidTime)
		CHECK;
	else {
		report("Trying time later than current X server time");
		report("Return value was %s, expecting GrabInvalidTime",
			grabreplyname(ret));
		FAIL;
	}

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGrabKeyboard-22.(A)
 * When the value of owner_events is other than True or False,
 * then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t022()
{
#undef	VALUE_ARG
#define	VALUE_ARG owner_events
#undef	VALUE_LIST
#define	VALUE_LIST owner_eventsvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
		endcall(display);
		if (ValueReturn != GrabSuccess) {
			report("Returned value was %d, expecting GrabSuccess", ValueReturn);
			FAIL;
		}
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/*
 * Assertion XGrabKeyboard-23.(A)
 * When the value of pointer_mode is other than GrabModeSync
 * or GrabModeAsync, then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t023()
{
#undef	VALUE_ARG
#define	VALUE_ARG pointer_mode
#undef	VALUE_LIST
#define	VALUE_LIST pointer_modevallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
		endcall(display);
		if (ValueReturn != GrabSuccess) {
			report("Returned value was %d, expecting GrabSuccess", ValueReturn);
			FAIL;
		}
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/*
 * Assertion XGrabKeyboard-24.(A)
 * When the value of keyboard_mode is other than GrabModeSync
 * or GrabModeAsync, then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t024()
{
#undef	VALUE_ARG
#define	VALUE_ARG keyboard_mode
#undef	VALUE_LIST
#define	VALUE_LIST keyboard_modevallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
		endcall(display);
		if (ValueReturn != GrabSuccess) {
			report("Returned value was %d, expecting GrabSuccess", ValueReturn);
			FAIL;
		}
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XGrabKeyboard-25.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t025()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGrabKeyboard(display, grab_window, owner_events, pointer_mode, keyboard_mode, test_time);
	endcall(display);
	if (ValueReturn != GrabSuccess) {
		report("Returned value was %d, expecting GrabSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	t025, 25,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = focusstartup;
void	(*tet_cleanup)() = focuscleanup;
