/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtkybrdmpp.m,v 1.10 92/06/11 17:13:59 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   KeySym *
 *   XGetKeyboardMapping(display, first_keycode, keycode_count, keysyms_per_keycode_return)
 *   Display	*display;
 *   KeyCode	first_keycode;
 *   int 	keycode_count;
 *   int 	*keysyms_per_keycode_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetKeyboardMapping	1
char    *TestName = "XGetKeyboardMapping";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGetKeyboardMapping function
 */
static Display	*display;
static KeyCode	first_keycode;
static int 	keycode_count;
static int 	*keysyms_per_keycode_return;



#define	MAXKPK	5	/* Maximum keysyms_per_keyocde we will use */
#define	MAXCODES 255	/* Max number of keycodes we will use */

static	KeyCode	First;
static	int 	ncodes;
static	int 	kpk;	/* Keysyms per keycode */
static	int 	oldkpk;	/* old Keysyms per keycode */
static	KeySym	*oldkeym;

/*
 * Can't really assume that there are any particular keysym names defined so
 * we use our own arbitrary values.
 */
#define	XT_KSYM1	0x5678
#define	XT_KSYM2	0x9228
#define	XT_KSYM3	0x4425
#define	XT_KSYM4	0x5326

static	KeySym	Keys[MAXKPK*MAXCODES];

/*
 * Set startup and cleanup functions to save and restore
 * the original keyboard map.
 */
static void
savekeymap()
{
int	tmp;
int 	last;
int 	i;

	startup();

	if (Dsp==(Display *)NULL)
		return;

	XDisplayKeycodes(Dsp, &tmp, &last);
	ncodes = last-tmp;
	First = (KeyCode)tmp;

	debug(2, "First keycode  :%d", First);
	debug(2, "Last keycode   :%d", last);
	debug(2, "Number of codes:%d", ncodes);

	oldkeym = XGetKeyboardMapping(Dsp, First, ncodes, &oldkpk);

	/*
	 * Initialise the keysym table.
	 */
	for (i = 0; i < MAXKPK*MAXCODES-4; i += 4) {
		Keys[i] = XT_KSYM1;
		Keys[i+1] = XT_KSYM2;
		Keys[i+2] = XT_KSYM3;
		Keys[i+3] = XT_KSYM4;
	}
}

static void
cleankeymap()
{
	if (Dsp) {
		XChangeKeyboardMapping(Dsp, First, oldkpk, oldkeym, ncodes);
		XFree((char*) oldkeym);
	}

	cleanup();
}



int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	first_keycode = First;
	keycode_count = ncodes;
	keysyms_per_keycode_return = &kpk;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetKeyboardMapping-1.(A)
 * A call to XGetKeyboardMapping returns an array, that can be
 * freed with XFree, of KeySyms associated with the specified
 * number, keycode_count, of KeyCodes starting with
 * first_keycode.
 */
static void
t001()
{
int 	i, j;
int 	syms_per_code;
KeySym	*newmap;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	first_keycode = First+3;
	syms_per_code = 3;
	keycode_count = 9;

	XChangeKeyboardMapping(display, first_keycode, syms_per_code, Keys, keycode_count);
	if (isdeleted())
		return;

	startcall(display);
	if (isdeleted())
		return;
	newmap = XGetKeyboardMapping(display, first_keycode, keycode_count, keysyms_per_keycode_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for (i = 0; i < keycode_count; i++) {
		for (j = 0; j < syms_per_code; j++) {
			if (Keys[i*syms_per_code+j] ==
				  newmap[i*keysyms_per_keycode_return[0]+j])
				CHECK;
			else {
				report("Keysym for keycode %d was 0x%x, expecting 0x%x",
					first_keycode+i,
					newmap[i*keysyms_per_keycode_return[0]+j],
					Keys[i*syms_per_code+j]
					);
				FAIL;
				break;	/* probably pointless to continue */
			}
		}
	}

	XFree((char*)newmap);

	CHECKPASS(syms_per_code*keycode_count);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetKeyboardMapping-2.(-)
 * On a call to XGetKeyboardMapping the returned KeySyms list
 * contains keycode_count*keysyms_per_keycode_return
 * elements.
 */
static void
t002()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XGetKeyboardMapping-3.(A)
 * On a call to XGetKeyboardMapping keysyms_per_keycode_return
 * is set to a value that is large enough to report all of the
 * KeySyms for any of the requested KeyCodes.
 */
static void
t003()
{
int 	syms_per_code = 6;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * I don't know a really good test for this.
	 */
	XChangeKeyboardMapping(display, first_keycode, syms_per_code, Keys, keycode_count);
	if (isdeleted())
		return;

	startcall(display);
	if (isdeleted())
		return;
	XGetKeyboardMapping(display, first_keycode, keycode_count, keysyms_per_keycode_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (keysyms_per_keycode_return[0] >= syms_per_code)
		CHECK;
	else {
		report("keysyms_per_keycode_return was unexpected");
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetKeyboardMapping-4.(-)
 * When an element for a particular KeyCode is unused, then a
 * KeySym value of NoSymbol is used in the returned array.
 */
static void
t004()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XGetKeyboardMapping-5.(A)
 * When the value specified in first_keycode is less than the
 * minimum keycode as returned by XDisplayKeycodes, then a
 * BadValue error occurs.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	first_keycode = First-1;

	startcall(display);
	if (isdeleted())
		return;
	XGetKeyboardMapping(display, first_keycode, keycode_count, keysyms_per_keycode_return);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetKeyboardMapping-6.(A)
 * When the expression first_keycode+keycode_count-1 is
 * greater than the maximum keycode as returned by
 * XDisplayKeycodes, then a BadValue error occurs.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	first_keycode = First+ncodes;

	startcall(display);
	if (isdeleted())
		return;
	XGetKeyboardMapping(display, first_keycode, keycode_count, keysyms_per_keycode_return);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = savekeymap;
void	(*tet_cleanup)() = cleankeymap;
