/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtpntrcntr.m,v 1.3 92/06/11 17:14:15 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XGetPointerControl(display, accel_numerator_return, accel_denominator_return, threshold_return)
 *   Display	*display;
 *   int 	*accel_numerator_return;
 *   int 	*accel_denominator_return;
 *   int 	*threshold_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetPointerControl	1
char    *TestName = "XGetPointerControl";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGetPointerControl function
 */
static Display	*display;
static int 	*accel_numerator_return;
static int 	*accel_denominator_return;
static int 	*threshold_return;



static int 	num;
static int 	denom;
static int 	thresh;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	accel_numerator_return = &num;
	accel_denominator_return = &denom;
	threshold_return = &thresh;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetPointerControl-1.(A)
 * A call to XGetPointerControl returns the current
 * acceleration multiplier and acceleration threshold to
 * accel_numerator_return, accel_denominator_return and
 * threshold_return.
 */
static void
t001()
{
int 	onum, odenom, othresh;
int 	val = 34;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/* First get original values */
	accel_numerator_return = &onum;
	accel_denominator_return = &odenom;
	threshold_return = &othresh;
	startcall(display);
	if (isdeleted())
		return;
	XGetPointerControl(display, accel_numerator_return, accel_denominator_return, threshold_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XChangePointerControl(display, True, True, val, val, val);
	if (isdeleted())
		return;

	accel_numerator_return = &num;
	accel_denominator_return = &denom;
	threshold_return = &thresh;
	startcall(display);
	if (isdeleted())
		return;
	XGetPointerControl(display, accel_numerator_return, accel_denominator_return, threshold_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (num == val)
		CHECK;
	else {
		report("accel_numerator_return was %d, expecting %d", num, val);
		FAIL;
	}
	if (denom == val)
		CHECK;
	else {
		report("accel_denominator_return was %d, expecting %d", denom, val);
		FAIL;
	}
	if (thresh == val)
		CHECK;
	else {
		report("threshold_return was %d, expecting %d", thresh, val);
		FAIL;
	}

	CHECKPASS(3);

	XChangePointerControl(display, True, True, onum, odenom, othresh);
	XSync(display, False);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
