/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: lstinstlld.m,v 1.4 92/06/11 17:15:10 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Colormap *
 *   XListInstalledColormaps(display, w, num_return)
 *   Display	*display;
 *   Window	w;
 *   int 	*num_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XListInstalledColormaps	1
char    *TestName = "XListInstalledColormaps";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XListInstalledColormaps function
 */
static Display	*display;
static Window	w;
static int 	*num_return;



/* Variable to hold the return value in */
static	int 	Num;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
	num_return = &Num;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
}

/*
 * Assertion XListInstalledColormaps-1.(A)
 * A call to XListInstalledColormaps returns a list, that can
 * be freed with XFree, of the currently installed colourmaps
 * for the screen of the specified window and the number of
 * such colourmaps in num_return.
 */
static void
t001()
{
Colormap	*cmp;
Screen	*screen;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	w = DRW(display);
	startcall(display);
	if (isdeleted())
		return;
	cmp = XListInstalledColormaps(display, w, num_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (cmp == NULL) {
		report("NULL was returned");
		FAIL;
		return;
	} else
		CHECK;

	screen = ScreenOfDisplay(display, DefaultScreen(display));
	if (*num_return < MinCmapsOfScreen(screen) ||
		*num_return > MaxCmapsOfScreen(screen)) {
		report("Number of installed colourmaps was outside range given by MinCmapsOfScreen and MaxCmapsOfScreen");
		FAIL;
	} else
		CHECK;

	XFree((char*)cmp);

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XListInstalledColormaps-2.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XListInstalledColormaps(display, w, num_return);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
