/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rprntwdw.m,v 1.5 92/06/11 17:15:57 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XReparentWindow(display, w, parent, x, y)
 *   Display	*display;
 *   Window	w;
 *   Window	parent;
 *   int 	x;
 *   int 	y;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XReparentWindow	1
char    *TestName = "XReparentWindow";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w
#define A_WINDOW2 parent
#define A_DRAWABLE2 parent


/*
 * Arguments to the XReparentWindow function
 */
static Display	*display;
static Window	w;
static Window	parent;
static int 	x;
static int 	y;



#define	RP_X	12
#define	RP_Y	15

char	*Treedesc[] = {
	"base borders",
	"A base (1,3) 50x30",
	  "A1 A (10,1) 4x5",
	  "A2 A (20,6) 4x5",
	"B base (4, 40) 20x20",
	  "B1 B (3,4) 10x10",

};

static	struct	buildtree	*Tree;
static	Window	WinA;
static	Window	WinB;
static	Window	WinBase;

static void
reparentinit()
{
Window	win;

	tpstartup();

	win = defwin(Dsp);
	Tree = buildtree(Dsp, win, Treedesc, NELEM(Treedesc));
	WinA = btntow(Tree, "A");
	WinB = btntow(Tree, "B");
	WinBase = btntow(Tree, "base");
}




#define	ON_WINDOW	0x1	/* event received on window itself */
#define	ON_PARENT	0x2	/* event received on parent window */
#define	ON_NEWPARENT 0x4	/* event received on new parent window */


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
	parent = 0;
	x = RP_X;
	y = RP_Y;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
	parent = ErrdefWindow;
}

/*
 * Assertion XReparentWindow-1.(A)
 * A call to XReparentWindow removes the specified window from
 * its current position in the hierarchy, and inserts it as the
 * child of the specified parent window at the position
 * specified by x and y.
 */
static void
t001()
{
Window	dummy;
Window	par;
Window	*children;
int 	nchild;
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	parent = WinA;
	w = WinB;

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * Check new parent.
	 */
	if (XQueryTree(display, WinB, &dummy, &par, &children, (unsigned*)&nchild) == False) {
		delete("XQueryTree on window failed");
		return;
	}

	if (par != parent) {
		report("New parent was incorrect");
		report(" Got 0x%x, expecting 0x%x", par, parent);
		FAIL;
	} else
		CHECK;

	if (nchild > 0 && children)
		XFree((char*) children);

	/*
	 * Check that window is not still a child of the old parent.
	 */
	if (XQueryTree(display, WinBase, &dummy, &par, &children, (unsigned*)&nchild) == False) {
		delete("XQueryTree on old parent failed");
		return;
	}
	while (nchild-- > 0) {
		if (children[nchild] == WinB)
			break;
	}

	if (nchild == -1)
		CHECK;
	else {
		report("window was still a child of the old parent after reparenting");
		FAIL;
	}

	if (nchild > 0 && children)
		XFree((char*) children);

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-2.(A)
 * When the specified window is mapped, then a call to
 * XReparentWindow automatically performs an UnmapWindow first,
 * and an UnmapNotify event is generated.
 */
static void
t002()
{
XUnmapEvent	good;
XUnmapEvent	*ump;
XEvent	ev;
int 	got = 0;
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	parent = WinA;
	w = WinB;

	XSelectInput(display, WinBase, SubstructureNotifyMask);
	XSelectInput(display, w, StructureNotifyMask);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	defsetevent(good, display, UnmapNotify);
	good.window = w;
	good.from_configure = False;

	got = 0;
	while (getevent(display, &ev)) {
		if (ev.type != UnmapNotify)
			continue;
		ump = (XUnmapEvent*)&ev;
		if (ump->event == ump->window) {
			/* On the window itself */
			if (got & ON_WINDOW) {
				report("Repeated unmap event on window");
				FAIL;
			} else
				CHECK;
			got |= ON_WINDOW;
			good.event = WinB;
		} else {
			if (got & ON_PARENT) {
				report("Repeated unmap event on parent window");
				FAIL;
			} else
				CHECK;
			got |= ON_PARENT;
			good.event = WinBase;
		}
		if (checkevent((XEvent*)&good, &ev))
			FAIL;
		else
			CHECK;
	}

	if (!(got & ON_PARENT)) {
		report("UnmapNotify event not received on parent window");
		FAIL;
	} else
		CHECK;
	if (!(got & ON_WINDOW)) {
		report("UnmapNotify event not received on window");
		FAIL;
	} else
		CHECK;
	
	CHECKPASS(6);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-3.(A)
 * The window is placed at the top of the stacking order with
 * respect to sibling windows.
 */
static void
t003()
{
int 	n;
int 	nsibs;
struct	buildtree	*btp;
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	parent = WinA;
	w = WinB;

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Count the number of siblings that window should have */
	nsibs = 0;
	for (btp = Tree+1; btp < Tree+NELEM(Treedesc); btp++) {
		if (btp->parent->wid == WinA)
			nsibs++;
	}

	/*
	 *This is the stacking order number that the window should have after
	 * reparenting
	 */
	n = stackorder(display, w);
	if (n != nsibs) {
		report("Reparented window had incorrect stacking order");
		report(" Got %d, expecting %d", n, nsibs);
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-4.(A)
 * A call to XReparentWindow generates a ReparentNotify event
 * on the window and on both the old and new parent windows.
 */
static void
t004()
{
XReparentEvent	good;
XReparentEvent	*rpp;
XEvent	ev;
int 	got = 0;
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	parent = WinA;
	w = WinB;

	XSelectInput(display, WinA, SubstructureNotifyMask);
	XSelectInput(display, WinBase, SubstructureNotifyMask);
	XSelectInput(display, w, StructureNotifyMask);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	defsetevent(good, display, ReparentNotify);
	good.event = 0;
	good.window = w;
	good.parent = WinA;
	good.x = x;
	good.y = y;
	good.override_redirect = False;

	got = 0;
	while (getevent(display, &ev)) {
		if (ev.type != ReparentNotify)
			continue;
		rpp = (XReparentEvent*)&ev;
		if (rpp->event == rpp->window) {
			/* On the window itself */
			if (got & ON_WINDOW) {
				report("Repeated unmap event on window");
				FAIL;
			} else
				CHECK;
			got |= ON_WINDOW;
			good.event = WinB;
		} else if (rpp->event == WinA) {
			if (got & ON_NEWPARENT) {
				report("Repeated unmap event on new parent window");
				FAIL;
			} else
				CHECK;
			got |= ON_NEWPARENT;
			good.event = WinA;
		} else if (rpp->event == WinBase) {
			if (got & ON_PARENT) {
				report("Repeated unmap event on old parent window");
				FAIL;
			} else
				CHECK;
			got |= ON_PARENT;
			good.event = WinBase;
		} else {
			report("Event received on window other than old or new parent");
			FAIL;
		}
		if (checkevent((XEvent*)&good, &ev))
			FAIL;
		else
			CHECK;
	}

	if (!(got & ON_NEWPARENT)) {
		report("UnmapNotify event not received on new parent window");
		FAIL;
	} else
		CHECK;
	if (!(got & ON_PARENT)) {
		report("UnmapNotify event not received on old parent window");
		FAIL;
	} else
		CHECK;
	if (!(got & ON_WINDOW)) {
		report("UnmapNotify event not received on window");
		FAIL;
	} else
		CHECK;
	
	CHECKPASS(9);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-5.(A)
 * When the specified window was originally mapped, then a
 * call to XReparentWindow automatically performs a MapWindow
 * request on it after the window is reparented, and a
 * MapRequest or MapNotify event is generated.
 */
static void
t005()
{
XMapEvent	good;
XMapEvent	*mp;
XMapRequestEvent	mreqgood;
XEvent	ev;
Display	*client2;
int 	n;
int 	got = 0;
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	parent = WinA;
	w = WinB;

	XSelectInput(display, parent, SubstructureNotifyMask);
	XSelectInput(display, w, StructureNotifyMask);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	defsetevent(good, display, MapNotify);
	good.window = w;
	good.override_redirect = False;

	got = 0;
	while (getevent(display, &ev)) {
		if (ev.type != MapNotify)
			continue;
		mp = (XMapEvent*)&ev;
		if (mp->event == mp->window) {
			/* On the window itself */
			if (got & ON_WINDOW) {
				report("Repeated map event on window");
				FAIL;
			} else
				CHECK;
			got |= ON_WINDOW;
			good.event = WinB;
		} else {
			if (got & ON_NEWPARENT) {
				report("Repeated map event on parent window");
				FAIL;
			} else
				CHECK;
			got |= ON_NEWPARENT;
			good.event = WinA;
		}
		if (checkevent((XEvent*)&good, &ev))
			FAIL;
		else
			CHECK;
	}

	if (!(got & ON_NEWPARENT)) {
		report("MapNotify event not received on parent window");
		FAIL;
	} else
		CHECK;
	if (!(got & ON_WINDOW)) {
		report("MapNotify event not received on window");
		FAIL;
	} else
		CHECK;

	if (isdeleted())
		return;

	client2 = opendisplay();
	XSelectInput(client2, parent, SubstructureRedirectMask);
	XSync(client2, True);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XSync(client2, False);

	defsetevent(mreqgood, client2, MapRequest);
	mreqgood.parent = parent;
	mreqgood.window = WinB;

	n =  getevent(client2, &ev);
	if (n != 1) {
		report("Expecting one MapRequest event, got %d events", n);
		FAIL;
	} else
		CHECK;
	if (ev.type != MapRequest) {
		report("Expecting one MapRequest event, got event type %s", eventname(ev.type));
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&mreqgood, &ev))
		FAIL;
	else
		CHECK;

	CHECKPASS(9);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-6.(A)
 * When a call to XReparentWindow uncovers part of any window
 * that was formerly obscured, then either Expose events are
 * generated or the contents are restored from backing store.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	parent = WinA;
	w = btntow(Tree, "B1");

	XSelectInput(display, WinB, ExposureMask);
	setforexpose(display, WinB);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (exposecheck(display, WinB))
		CHECK;
	else {
		report("Contents of window not restored correctly by Expose or backing store");
		FAIL;
	}

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-7.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XReparentWindow may generate Expose events for
 * regions uncovered by the initial UnmapWindow request that
 * are immediately obscured by the final MapWindow request.
 */
static void
t007()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XReparentWindow-8.(C)
 * If multiple screens are supported: When the new parent
 * window is not on the same screen as the old parent window,
 * then a BadMatch error occurs.
 */
static void
t008()
{
Window	w;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (config.alt_screen == -1) {
		unsupported("Multiple screens not supported");
		return;
	}

	w = defwin(display);
	parent = defdraw(display, VI_ALT_WIN);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-9.(A)
 * When the new parent window is the specified window, then a
 * BadMatch error occurs.
 */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	w = defwin(display);
	parent = w;

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-10.(A)
 * When the new parent window is an inferior of the specified
 * window, then a BadMatch error occurs.
 */
static void
t010()
{
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	w = WinA;
	parent = btntow(Tree, "A1");

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReparentWindow-11.(C)
 * If multiple window depths are supported: When the specified
 * window has a ParentRelative background, and the new parent
 * window does not have the same depth as the specified window,
 * then a BadMatch error occurs.
 */
static void
t011()
{
XVisualInfo	*vp;
unsigned int 	depth = 0;
int 	found;
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	found = 0;
	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		if (depth && depth != vp->depth) {
			found = 1;
			w = makewin(display, vp);
			break;
		} else {
			depth = vp->depth;
			parent = makewin(display, vp);
		}
	}

	if (!found) {
		unsupported("Only one depth window supported");
		return;
	}

	/*
	 * At the time of writing we do not have a machine on which
	 * the following code can be tested.
	 */
	XSetWindowBackgroundPixmap(display, w, ParentRelative);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XReparentWindow-12.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t012()
{
int 	pass = 0, fail = 0;

	reparentinit();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	seterrdef();
	A_WINDOW2 = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/*
 * Assertion XReparentWindow-13.(A)
 * When the new parent is InputOnly and the window is
 * InputOutput, then a BadMatch error occurs.
 */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	parent = iponlywin(display);
	w = defwin(display);

	startcall(display);
	if (isdeleted())
		return;
	XReparentWindow(display, w, parent, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;	/* already done */
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
