/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ungrbpntr.m,v 1.4 92/06/11 17:17:28 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XUngrabPointer(display, test_time)
 *   Display	*display;
 *   Time	test_time;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XUngrabPointer	1
char    *TestName = "XUngrabPointer";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XUngrabPointer function
 */
static Display	*display;
static Time	test_time;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	test_time = CurrentTime;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XUngrabPointer-1.(A)
 * When the client has actively grabbed the pointer with a
 * call to XGrabPointer, XGrabButton or from a normal button
 * press, then a call to XUngrabPointer releases the pointer
 * and any queued events.
 */
static void
t001()
{
XEvent	ev;
Window	win;
unsigned int 	mask = PointerMotionMask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	win = defwin(display);
	XSelectInput(display, win, PointerMotionMask);
	(void) warppointer(display, win, 0, 0);

	XGrabPointer(display, win, False, mask, GrabModeSync, GrabModeAsync,
		None, None, CurrentTime);

	(void) warppointer(display, win, 1, 1);
	if (XCheckMaskEvent(display, (unsigned long)mask, &ev)) {
		delete("Pointer event was received while frozen");
		return;
	} else
		CHECK;

	startcall(display);
	if (isdeleted())
		return;
	XUngrabPointer(display, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (XCheckMaskEvent(display, (unsigned long)mask, &ev) && ev.type == MotionNotify)
		CHECK;
	else {
		report("Pointer event was not saved while pointer was frozen");
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUngrabPointer-2.(A)
 * When the specified time is earlier than the
 * last-pointer-grab time or is later than the current X server
 * time, then a call to XUngrabPointer does not release the
 * pointer grab.
 */
static void
t002()
{
Window	win;
Window	grabwin;
Display	*client2;
XEvent	ev;
unsigned int 	mask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	mask = PointerMotionMask;

	client2 = opendisplay();

	win = defwin(display);
	(void) warppointer(display, win, 0, 0);
	XSelectInput(client2, win, (unsigned long)mask);

	grabwin = defwin(display);

	test_time = gettime(display);
	XGrabPointer(display, grabwin, False, mask, GrabModeSync, GrabModeAsync,
		None, None, test_time);

	XSync(client2, True);
	if (isdeleted())
		return;

	test_time -= 100;
	startcall(display);
	if (isdeleted())
		return;
	XUngrabPointer(display, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(display, win, 8, 8);
	if (XCheckMaskEvent(client2, (unsigned long)mask, &ev)) {
		report("Grab was released when time was earlier than last-pointer-grab time");
		FAIL;
	} else
		CHECK;

	XSync(client2, True);	/* Flush any remaining events */

	/*
	 * Get current time again and add several minutes to get a time in the
	 * future.
	 */
	test_time = gettime(display);
	test_time += ((config.speedfactor+1) * 1000000);

	startcall(display);
	if (isdeleted())
		return;
	XUngrabPointer(display, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) warppointer(display, win, 12, 1);
	if (XCheckMaskEvent(client2, (unsigned long)mask, &ev)) {
		report("Grab was released when time was earlier than last-pointer-grab time");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUngrabPointer-3.(A)
 * A call to XUngrabPointer generates EnterNotify and
 * LeaveNotify events.
 */
static void
t003()
{
Window	grabwin;
Window	win;
XEvent	ev;
XCrossingEvent	*cp;
XEnterWindowEvent	entergood;
XLeaveWindowEvent	leavegood;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	grabwin = defwin(display);
	win = defwin(display);

	(void) warppointer(display, win, 0, 0);

	XGrabPointer(display, grabwin, False, (unsigned int)PointerMotionMask,
		GrabModeSync, GrabModeAsync, None, None, CurrentTime);

	XSelectInput(display, win, EnterWindowMask|LeaveWindowMask);
	XSelectInput(display, grabwin, EnterWindowMask|LeaveWindowMask);

	startcall(display);
	if (isdeleted())
		return;
	XUngrabPointer(display, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	defsetevent(entergood, display, EnterNotify);
	entergood.root = DRW(display);
	entergood.subwindow = None;
	entergood.time = 0;
	entergood.mode = NotifyNormal;
	entergood.detail = NotifyNonlinear;
	entergood.same_screen = True;
	entergood.focus = False;
	entergood.state = 0;

	defsetevent(leavegood, display, LeaveNotify);
	leavegood.root = DRW(display);
	leavegood.subwindow = None;
	leavegood.time = 0;
	leavegood.mode = NotifyNormal;
	leavegood.detail = NotifyNonlinear;
	leavegood.same_screen = True;
	leavegood.focus = False;
	leavegood.state = 0;

	/*
	 * There is an Ungrab mode leave from the grab window followed by
	 * an enter to the original window.
	 */
	leavegood.window = grabwin;
	leavegood.mode = NotifyUngrab;
	entergood.window = win;
	entergood.mode = NotifyUngrab;

	if (getevent(display, &ev) == 0 || ev.type != LeaveNotify) {
		report("No ungrab-mode leave event from the grab window");
		FAIL;
	} else
		CHECK;

	cp = (XCrossingEvent*)&ev;
	/* Set the fields that we can't conveniently check */
	leavegood.time = cp->time;
	leavegood.x = cp->x;
	leavegood.y = cp->y;
	leavegood.x_root = cp->x_root;
	leavegood.y_root = cp->y_root;
	leavegood.focus = cp->focus;
	if (checkevent((XEvent*)&leavegood, &ev))
		FAIL;
	else
		CHECK;

	if (getevent(display, &ev) == 0 || ev.type != EnterNotify) {
		report("No ungrab-mode enter notify event to window pointer is in");
		FAIL;
	} else
		CHECK;

	/* The cursor was at 0,0 */
	entergood.x = 0;
	entergood.y = 0;

	cp = (XCrossingEvent*)&ev;
	/* Set the fields that we can't conveniently check */
	entergood.time = cp->time;
	entergood.x_root = cp->x_root;
	entergood.y_root = cp->y_root;
	entergood.focus = cp->focus;
	if (checkevent((XEvent*)&entergood, &ev))
		FAIL;
	else
		CHECK;
	
	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
