/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ungrbsrvr.m,v 1.3 92/06/11 17:17:37 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XUngrabServer(display)
 *   Display	*display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XUngrabServer	1
char    *TestName = "XUngrabServer";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XUngrabServer function
 */
static Display	*display;



#include	"Xatom.h"



static Display	*client2;

static Window	win;

static int 	waittime;


static void
pproc()
{
XEvent	ev;
int 	pass = 0, fail = 0;

	/* Allow time for change prop to occur */
	sleep(waittime);

	if (XCheckWindowEvent(display, win, PropertyChangeMask, &ev)) {
		delete("A request was processed for other than the grabbing client");
		FAIL;
	} else
		CHECK;

	/*
	 * Release the grab and wait a bit to allow the requests on the other
	 * connection to go ahead.
	 */
	startcall(display);
	if (isdeleted())
		return;
	XUngrabServer(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Allow time for change prop to occur */
	sleep(waittime);
	if (XCheckWindowEvent(display, win, PropertyChangeMask, &ev))
		CHECK;
	else {
		report("Requests were not processed after server grab was released.");
		FAIL;
	}

	CHECKPASS(2);
}

/*
 * Perform operation on client2 while display has the server grabbed.
 */
static void
cproc()
{
long 	val;
Atom	name;

	val = 5;

	settimeout(waittime*3);

	name = XInternAtom(client2, "name", False);
	XChangeProperty(client2, win, name, XA_INTEGER, 32, PropModeReplace,
		(unsigned char *)&val, 1);
	XFlush(client2);

	cleartimeout();

	exit(0);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XUngrabServer-1.(A)
 * A call to XUngrabServer resumes processing of requests and
 * close downs on other connections.
 */
static void
t001()
{
int 	ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	client2 = opendisplay();

	win = defwin(Dsp);
	XSelectInput(display, win, PropertyChangeMask);
	XSelectInput(client2, win, PropertyChangeMask);

	waittime = 2*config.speedfactor + 5;

	if (isdeleted())
		return;

	XGrabServer(display);
	XSync(display, False);

	ret = tet_fork(cproc, pproc, waittime, 1);
	/* Test now continues in pproc */

	if (ret == TIMEOUT_EXIT) {
		delete("Child process timed out");
	}
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
