/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: bttnprss.m,v 1.12 92/06/11 17:18:17 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_ButtonPress	1
char    *TestName = "ButtonPress";



#define EVENT ButtonPress
#define EVENTMASK ButtonPressMask
#define PTRX 4
#define PTRY 3

int 	tet_thistest;

/*
 * Assertion ButtonPress-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * When any pointer button is pressed, then a ButtonPress
 * event is generated.
 */
static void
t001()
{
Display			*display = Dsp;
Window			w;
XEvent			event_return;
XButtonPressedEvent	good;
PointerPlace		*ptr;
int 	pass = 0, fail = 0;

	tpstartup();

	if(noext(1))
		return;

/* Create window. */
	w = defwin(display);

/* Select for events. */
	XSelectInput(display, w, EVENTMASK);
	XSync(display, True);

/* Generate xname event. */
	XSync(display, True);
	ptr = warppointer(display, w, PTRX, PTRY);
	buttonpress(display, Button1);

/* Verify that event was delivered. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;

	good = event_return.xbutton;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.root = DRW(display);
	good.subwindow = None;
	good.x = PTRX;
	good.y = PTRY;
	good.x_root = ptr->nx;
	good.y_root = ptr->ny;
	good.state = 0L;
	good.button = Button1;
	good.same_screen = True;

	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;

	
	CHECKPASS(2);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is generated and no active pointer
 * grab is in progress and at least one ancestor of the source
 * window has a matching passive grab on the pressed pointer
 * button, then the X server activates a passive grab for the
 * first ancestor of the source window, searching from the root
 * window down.
 */
static void
t002()
{
Window			w;
Window			w1;
Window			w2;
XEvent			ev;
struct area		area;
char			*unexpstr = "Unexpected event on a child window.";
int 	pass = 0, fail = 0;

	tpstartup();

	if (noext(1))
		return;

        w = defwin(Dsp);
	setarea(&area, 10, 10, 30, 30);
	w1 = crechild(Dsp, w, &area);
	w2 = crechild(Dsp, w1, &area);
	(void) warppointer(Dsp, w2, 1,1);

	XSelectInput(Dsp, w, EnterWindowMask);
	XSelectInput(Dsp, w1, EnterWindowMask);
	XSelectInput(Dsp, w2, EnterWindowMask);

	XGrabButton(Dsp, Button1, AnyModifier, w, False,  0L, GrabModeAsync, GrabModeAsync, None, None);
	XGrabButton(Dsp, Button1, AnyModifier, w1, False, 0L, GrabModeAsync, GrabModeAsync, None, None);
	XGrabButton(Dsp, Button1, AnyModifier, w2, False, 0L, GrabModeAsync, GrabModeAsync, None, None);

	XSync(Dsp, True);
	buttonpress(Dsp, Button1);

	if(XCheckWindowEvent(Dsp, w1, EnterWindowMask, &ev) != False) {
		report(unexpstr);
		FAIL;
	} else
		CHECK;

	if(XCheckWindowEvent(Dsp, w2, EnterWindowMask, &ev) != False) {
		report(unexpstr);
		FAIL;
	} else
		CHECK;

	if(XCheckWindowEvent(Dsp, w, EnterWindowMask, &ev) == False) {
		report("No event was activated on the oldest ancestor window.");
		FAIL;
	} else {

		CHECK;
		if(ev.xcrossing.mode != NotifyGrab) {
			report("Passive grab was not activated on oldest ancestor window.");
			FAIL;
		} else
			CHECK;
	}

	buttonrel(Dsp, Button1);
	XUngrabButton(Dsp, Button1, AnyModifier, w);
	XUngrabButton(Dsp, Button1, AnyModifier, w1);
	XUngrabButton(Dsp, Button1, AnyModifier, w2);

	CHECKPASS(4);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is generated and no active pointer
 * grab is in progress and no ancestor of the source window has
 * a matching passive grab on the pressed pointer button, then
 * the X server automatically starts an active grab for the
 * client receiving the event and sets the last-pointer-grab
 * time to the current server time.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion ButtonPress-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is generated and no active pointer
 * grab is in progress and no ancestor of the source window has
 * a matching passive grab on the pressed pointer button and no
 * client has selected for button presses on the event window,
 * then the event is discarded.
 */
static void
t004()
{
Display			*display = Dsp;
Window			w;
int			count;
int 	pass = 0, fail = 0;

	tpstartup();

	if(noext(1))
		return;

/* Create window. */
	w = defwin(display);

	(void) warppointer(display, w, PTRX, PTRY);
	XSelectInput(display, w, ALLEVENTS & ~ButtonPressMask);
	XSync(display, True);
/* Generate xname event. */
	XSync(display, True);
	buttonpress(display, Button1);

/* Verify no events were delivered. */

	count = XPending(display);
	if (count != 0) {
		report("Got %d unexpected events.", count);
		FAIL;
	}
	else
		PASS;

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is generated, then all clients
 * having set ButtonPressMask event mask bits on the event
 * window are delivered a ButtonPress event.
 */
static void
t005()
{
Display			*display = Dsp;
Window			w;
XEvent			event_return;
XButtonPressedEvent	good;
PointerPlace		*ptr;
int 	pass = 0, fail = 0;

	tpstartup();


	if(config.extensions) {

		if(noext(1))
			return;
	/* Create window. */
		w = defwin(display);

	/* Select for events. */
		XSelectInput(display, w, EVENTMASK);
		XSync(display, True);
	
	/* Generate xname event. */
		XSync(display, True);
		ptr = warppointer(display, w, PTRX, PTRY);
		buttonpress(display, Button1);
	
	/* Verify that event was delivered. */
	/* Verify that event member fields are correctly set. */
		if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
			report("Expected %s event, got none", eventname(EVENT));
			FAIL;
		} else
			CHECK;
	
		good = event_return.xbutton;
		good.type = EVENT;
		good.send_event = False;
		good.display = display;
		good.window = w;
		good.root = DRW(display);
		good.subwindow = None;
		good.x = PTRX;
		good.y = PTRY;
		good.x_root = ptr->nx;
		good.y_root = ptr->ny;
		good.state = 0L;
		good.button = Button1;
		good.same_screen = True;
	
		if (checkevent((XEvent *) &good, &event_return)) {
			report("Unexpected values in delivered event");
			FAIL;
		}
		else
			CHECK;
		
		CHECKPASS(2);

	} else {

	/* Create window. */
		w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
	/* Set ButtonPressMask event mask bits on window. */
		XSelectInput(display, w, ButtonPressMask);
		XSync(display, True);
	/* Verify that no error occurred. */
		if (geterr() != Success) {
			delete("Error setting up for test.");
			return;
		}
		else
			CHECK;
		CHECKUNTESTED(1);
	}

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-6.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is generated, then clients not
 * having set ButtonPressMask event mask bits on the event
 * window are not delivered a ButtonPress event.
 */
static void
t006()
{
Display			*display = Dsp;
Display			*client2;
XEvent			ev;
Window			w;
int			count;
int 	pass = 0, fail = 0;

	tpstartup();

	if(noext(1))
		return;

	client2 = opendisplay();

/* Create window. */
	w = defwin(display);

	(void) warppointer(display, w, PTRX, PTRY);
	XSelectInput(display, w, ALLEVENTS & ~ButtonPressMask);
	XSelectInput(client2, w, ButtonPressMask);
	XSync(display, True);
	XSync(client2, True);
/* Generate xname event. */
	buttonpress(display, Button1);

	XSync(client2, False);


/* Verify a ButtonPress was generated. */
	if(XCheckWindowEvent(client2, w, ButtonPressMask, &ev) == False) {
		report("No ButtonPress event was generated.");
		FAIL;
	} else
		CHECK;

/* Verify no events were delivered. */
	count = XPending(display);
	if (count != 0) {
		report("Got %d unexpected events.", count);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-7.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is generated and no client has
 * selected ButtonPressMask on the source window, then the
 * event propagates, with propagation stopping at the root
 * window of the screen or at the first window with
 * ButtonPressMask in its do-not-propagate mask, from the
 * source window to the first ancestor window for which some
 * client has selected for ButtonPress events.
 */
static void
t007()
{
XEvent			ev;
Window			w;
Window			w1;
Window			w2;
XSetWindowAttributes	atts;
int 	pass = 0, fail = 0;

	tpstartup();

	if(noext(1))
		return;

	w = defwin(Dsp);
	w1 = crechild(Dsp, w, (struct area *) NULL);
	w2 = crechild(Dsp, w1, (struct area *) NULL);
	XSelectInput(Dsp, DRW(Dsp), ButtonPressMask);	
	(void) warppointer(Dsp, w2, 1,1);

	XSync(Dsp, True);
	buttonpress(Dsp, Button1);
	
	if (XCheckWindowEvent(Dsp, DRW(Dsp), ButtonPressMask, &ev) == False) {
		report("Expected event (%s) not received.", eventname(ButtonPress));
		FAIL;
	} else
		CHECK;

	XSelectInput(Dsp, DRW(Dsp), NoEventMask);	
	XSelectInput(Dsp, w, ButtonPressMask);	

	atts.do_not_propagate_mask = ButtonPressMask;
	XChangeWindowAttributes(Dsp, w1, CWDontPropagate, &atts);
	
	buttonrel(Dsp, Button1);	
	buttonpress(Dsp, Button1);

	if (XCheckWindowEvent(Dsp, w2, ButtonPressMask, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(ButtonPress));
		FAIL;
	} else
		CHECK;

	if (XCheckWindowEvent(Dsp, w1, ButtonPressMask, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(ButtonPress));
		FAIL;
	} else
		CHECK;

	if (XCheckWindowEvent(Dsp, w, ButtonPressMask, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(ButtonPress));
		FAIL;
	} else
		CHECK;

	XSelectInput(Dsp, w1, ButtonPressMask);	
	XChangeWindowAttributes(Dsp, w, CWDontPropagate, &atts);

	atts.do_not_propagate_mask = NoEventMask;
	XChangeWindowAttributes(Dsp, w1, CWDontPropagate, &atts);

	buttonrel(Dsp, Button1);	
	buttonpress(Dsp, Button1);

	if (XCheckWindowEvent(Dsp, w2, ButtonPressMask, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(ButtonPress));
		FAIL;
	} else
		CHECK;

	if (XCheckWindowEvent(Dsp, w, ButtonPressMask, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(ButtonPress));
		FAIL;
	} else
		CHECK;

	if (XCheckWindowEvent(Dsp, w1, ButtonPressMask, &ev) == False) {
		report("ButtonPress event was not delivered to selecting child window.");
		FAIL;
	} else
		CHECK;

CHECKPASS(7);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-8.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is delivered and the source window
 * is an inferior of the event window and the source window is
 * a child of the event window, then subwindow is set to the
 * source window.
 */
static void
t008()
{
XEvent			ev;
Window			w;
Window			w1;
int 	pass = 0, fail = 0;

	tpstartup();

	if(noext(1))
		return;

	w = defwin(Dsp);
	w1 = crechild(Dsp, w, (struct area *) NULL);
	XSelectInput(Dsp, w, ButtonPressMask);	
	(void) warppointer(Dsp, w1, 1,1);

	XSync(Dsp, True);
	buttonpress(Dsp, Button1);
	
	if (XCheckWindowEvent(Dsp, w, ButtonPressMask, &ev) == False) {
		report("Expected event (%s) not received.", eventname(ButtonPress));
		FAIL;
	} else {
		CHECK;
		if(ev.xbutton.subwindow != w1) {
			report("The subwindow component of the %s event was not set correctly.", eventname(ButtonPress));
			FAIL;
		} else 
			CHECK;
	}

	CHECKPASS(2);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-9.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is delivered and the source window
 * is an inferior of the event window and the source window is
 * not a child of the event window, then subwindow is set to
 * the child of the event window that is an ancestor of the
 * source window.
 */
static void
t009()
{
Window			w;
Window			w1;
Window			w2;
XEvent			ev;
struct area		area;
int 	pass = 0, fail = 0;

	tpstartup();

	if (noext(1))
		return;

        w = defwin(Dsp);
	setarea(&area, 10, 10, 30, 30);
	w1 = crechild(Dsp, w, &area);
	w2 = crechild(Dsp, w1, &area);
	(void) warppointer(Dsp, w2, 1,1);
	XSelectInput(Dsp, w, ButtonPressMask);

	XSync(Dsp, True);
	buttonpress(Dsp, Button1);
	
	if (XCheckWindowEvent(Dsp, w, ButtonPressMask, &ev) == False) {
		report("Expected event (%s) not received.", eventname(ButtonPress));
		FAIL;
	} else {
		CHECK;
		if(ev.xbutton.subwindow != w1) {
			report("The subwindow component of the %s event was not set correctly.", eventname(ButtonPress));
			FAIL;
		} else 
			CHECK;

	}

	CHECKPASS(2);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-10.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a ButtonPress event is delivered and the source window
 * is not an inferior of the event window, then subwindow is
 * set to None.
 */
static void
t010()
{
Window			w;
Window			w2;
XEvent			ev;
int 	pass = 0, fail = 0;

	tpstartup();

	if (noext(1))
		return;

        w = defwin(Dsp);

	(void) warppointer(Dsp, w, 1,1);
	XSelectInput(Dsp, w, ButtonPressMask);

	XSync(Dsp, True);
	buttonpress(Dsp, Button1);
	
	if (XCheckWindowEvent(Dsp, w, ButtonPressMask, &ev) == False) {
		report("Expected event (%s) not received.", eventname(ButtonPress));
		FAIL;
	} else {
		CHECK;
		if(ev.xbutton.subwindow != None) {
			report("The subwindow component of the %s event was not set correctly.", eventname(ButtonPress));
			FAIL;
		} else 
			CHECK;
	}

	w2 = defwin(Dsp);

	if( XGrabPointer(Dsp, w, False, ButtonPressMask, GrabModeAsync, GrabModeAsync, None, None, CurrentTime) != GrabSuccess) {
		delete("XGrabPointer() did not return GrabSuccess.");
		return;
	} else
		CHECK;

	(void) warppointer(Dsp, w2, 1,1);
	XSync(Dsp, True);
	buttonpress(Dsp, Button1);
	
	if (XCheckWindowEvent(Dsp, w, ButtonPressMask, &ev) == False) {
		report("Expected event (%s) not received.", eventname(ButtonPress));
		FAIL;
	} else {
		CHECK;
		if(ev.xbutton.subwindow != None) {
			report("The subwindow component of the %s event was not set correctly.", eventname(ButtonPress));
			FAIL;
		} else 
			CHECK;
	}

	XUngrabPointer(Dsp, CurrentTime);

	CHECKPASS(5);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-11.(D)
 *   Reason: There is no known portable test method for this assertion
 * If multiple screens are supported: When a ButtonPress event
 * is delivered and the event and root windows are not on the
 * same screen, then x and y are set to zero.
 */
static void
t011()
{
Window			w;
Window			w2;
XEvent			ev;
int 	pass = 0, fail = 0;

	tpstartup();

/* Report unsupported if multiple screens are not supported. */
	if (config.alt_screen == -1) {
		unsupported("Multiple screens not supported.");
		return;
	} else
		CHECK;

	if(noext(1))
		return;

        w = defwin(Dsp);
	
	(void) warppointer(Dsp, w, 1,1);
	XSelectInput(Dsp, w, ButtonPressMask);
	
	w2 = defdraw(Dsp, VI_ALT_WIN);
	
	if( XGrabPointer(Dsp, w, False, ButtonPressMask, GrabModeAsync, GrabModeAsync, None, None, CurrentTime) != GrabSuccess) {
		delete("XGrabPointer() did not return GrabSuccess.");
		return;
	} else
		CHECK;
	
	(void) warppointer(Dsp, w2, 1,1);
	XSync(Dsp, True);
	buttonpress(Dsp, Button1);
	
	if (XCheckWindowEvent(Dsp, w, ButtonPressMask, &ev) == False) {
		report("Expected event (%s) not received.", eventname(ButtonPress));
		FAIL;
	} else {
		CHECK;
		if(ev.xbutton.x != 0 || ev.xbutton.y != 0) {
			report("The x and y components of the %s event were not set correctly.", eventname(ButtonPress));
			FAIL;
		} else 
			CHECK;
	}
	
	XUngrabPointer(Dsp, CurrentTime);
	
	CHECKPASS(4);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ButtonPress-12.(D)
 *   Reason: There is no known portable test method for this assertion
 * If multiple screens are supported: When a ButtonPress event
 * is delivered and the event and root windows are not on the
 * same screen, then same_screen is set to False.
 */
static void
t012()
{
Window			w;
Window			w2;
XEvent			ev;
int 	pass = 0, fail = 0;

	tpstartup();

	if (config.alt_screen == -1) {
		unsupported("Multiple screens not supported.");
		return;
	} else
		CHECK;

	if(noext(1))
		return;

        w = defwin(Dsp);
	
	(void) warppointer(Dsp, w, 1,1);
	XSelectInput(Dsp, w, ButtonPressMask);
	
	w2 = defdraw(Dsp, VI_ALT_WIN);
	
	if( XGrabPointer(Dsp, w, False, ButtonPressMask, GrabModeAsync, GrabModeAsync, None, None, CurrentTime) != GrabSuccess) {
		delete("XGrabPointer() did not return GrabSuccess.");
		return;
	} else
		CHECK;
	
	(void) warppointer(Dsp, w2, 1,1);
	XSync(Dsp, True);
	buttonpress(Dsp, Button1);
	
	if (XCheckWindowEvent(Dsp, w, ButtonPressMask, &ev) == False) {
		report("Expected event (%s) not received.", eventname(ButtonPress));
		FAIL;
	} else {
		CHECK;
		if(ev.xbutton.same_screen != False ) {
			report("The same_screen component of the %s event was not set correctly.", eventname(ButtonPress));
			FAIL;
		} else 
			CHECK;
	}
	
	XUngrabPointer(Dsp, CurrentTime);
	
	CHECKPASS(4);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
