/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: dsplymtnbf.m,v 1.8 92/06/11 17:20:06 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   unsigned long
 *   XDisplayMotionBufferSize(display)
 *   Display *display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XDisplayMotionBufferSize	1
char    *TestName = "XDisplayMotionBufferSize";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XDisplayMotionBufferSize function
 */
static Display *display;


#include <stdio.h>

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XDisplayMotionBufferSize-1.(C)
 * If the implementation supports a more complete history of
 * pointer motion than is reported by event notification: a
 * call to XDisplayMotionBufferSize returns the approximate
 * maximum number of elements in the motion buffer.
 */
static void
t001()
{
unsigned long bufsize;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* If motion buffer not supported, call UNSUPPORTED. */
	if (config.displaymotionbuffersize == 0) {
		report("Motion buffer not supported.");
		UNSUPPORTED;
		return;
	}
	else
		CHECK;
/* Call XDisplayMotionBufferSize. */
	startcall(display);
	if (isdeleted())
		return;
	bufsize = XDisplayMotionBufferSize(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify value returned by XDisplayMotionBufferSize. */
	if (bufsize != config.displaymotionbuffersize) {
		report("Returned %d, expected %d", bufsize, config.displaymotionbuffersize);
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDisplayMotionBufferSize-2.(C)
 * If the implementation does not support a more complete
 * history of pointer motion than is reported by event
 * notification: a call to XDisplayMotionBufferSize returns
 * zero.
 */
static void
t002()
{
unsigned long bufsize;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* If motion buffer supported, call UNSUPPORTED. */
	if (config.displaymotionbuffersize != 0) {
		report("Motion buffer supported.");
		UNSUPPORTED;
		return;
	}
	else
		CHECK;
/* Call XDisplayMotionBufferSize. */
	startcall(display);
	if (isdeleted())
		return;
	bufsize = XDisplayMotionBufferSize(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify value returned by XDisplayMotionBufferSize. */
	if (bufsize != 0) {
		report("Returned %d, expected %d", bufsize, 0);
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
