/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: evntsqd.m,v 1.11 92/06/11 17:20:46 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XEventsQueued(display, mode)
 *   Display *display;
 *   int mode;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XEventsQueued	1
char    *TestName = "XEventsQueued";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XEventsQueued function
 */
static Display *display;
static int mode;


static int 	ValueReturn;


/*
 * Can not use "xcall" because it empties the event queue.
 */
#define	_xcall_(return_value)	\
		_startcall(display);\
		return_value = XEventsQueued(display, mode);\
		_endcall(display)

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	mode = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XEventsQueued-1.(A)
 * When the number of events already in the event queue is
 * non-zero, then a call to XEventsQueued returns the number of
 * events in the event queue.
 */
static void
t001()
{
int	eventsput;
int	event_count;
XEvent	event;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = MapNotify;
	eventsput = 0;
	XPutBackEvent(display, &event), eventsput++;
	XPutBackEvent(display, &event), eventsput++;
	XPutBackEvent(display, &event), eventsput++;
/* Call XEventsQueued with mode QueuedAlready. */
	mode = QueuedAlready;
	_xcall_(event_count);
/* Verify that XEventsQueued returned the correct number of events. */
	if (event_count != eventsput) {
		report("Returned %d, expected %d", event_count, eventsput);
		FAIL;
	}
	else
		CHECK;
/* Call XEventsQueued with mode QueuedAfterFlush. */
	mode = QueuedAfterFlush;
	_xcall_(event_count);
/* Verify that XEventsQueued returned the correct number of events. */
	if (event_count != eventsput) {
		report("Returned %d, expected %d", event_count, eventsput);
		FAIL;
	}
	else
		CHECK;
/* Call XEventsQueued with mode QueuedAfterReading. */
	mode = QueuedAfterReading;
	_xcall_(event_count);
/* Verify that XEventsQueued returned the correct number of events. */
	if (event_count != eventsput) {
		report("Returned %d, expected %d", event_count, eventsput);
		FAIL;
	}
	else
		CHECK;
	/* empty event queue */
	XSync(display, True);

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEventsQueued-2.(A)
 * When the number of events already in the event queue is
 * non-zero, then a call to XEventsQueued does not block.
 */
static void
t002()
{
int	eventsput;
XEvent	event;
Block_Info info;
int	block_status;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = MapNotify;
	eventsput = 0;
	XPutBackEvent(display, &event), eventsput++;
	XPutBackEvent(display, &event), eventsput++;
	XPutBackEvent(display, &event), eventsput++;
/* Call XEventsQueued with mode QueuedAlready */
/* and verify that blocking did not occur. */
	XEventsQueued_Type(info, QueuedAlready);
	block_status = block(display, (XEvent *) NULL, &info);
	if (block_status == -1)
		return;
	else
		CHECK;
	if (block_status == 1) {
		report("Blocking occurred when it should not have.");
		FAIL;
	}
	else
		CHECK;
/* Verify that XEventsQueued returned the expected number of events. */
	if (info.int_return != eventsput) {
		delete("Unexpected number of events in event queue: %d", info.int_return);
		return;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = MapNotify;
	eventsput = 0;
	XPutBackEvent(display, &event), eventsput++;
	XPutBackEvent(display, &event), eventsput++;
	XPutBackEvent(display, &event), eventsput++;
/* Call XEventsQueued with mode QueuedAfterFlush */
/* and verify that blocking did not occur. */
	XEventsQueued_Type(info, QueuedAfterFlush);
	block_status = block(display, (XEvent *) NULL, &info);
	if (block_status == -1)
		return;
	else
		CHECK;
	if (block_status == 1) {
		report("Blocking occurred when it should not have.");
		FAIL;
	}
	else
		CHECK;
/* Verify that XEventsQueued returned the expected number of events. */
	if (info.int_return != eventsput) {
		delete("Unexpected number of events in event queue: %d", info.int_return);
		return;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = MapNotify;
	eventsput = 0;
	XPutBackEvent(display, &event), eventsput++;
	XPutBackEvent(display, &event), eventsput++;
	XPutBackEvent(display, &event), eventsput++;
/* Call XEventsQueued with mode QueuedAfterReading */
/* and verify that blocking did not occur. */
	XEventsQueued_Type(info, QueuedAfterReading);
	block_status = block(display, (XEvent *) NULL, &info);
	if (block_status == -1)
		return;
	else
		CHECK;
	if (block_status == 1) {
		report("Blocking occurred when it should not have.");
		FAIL;
	}
	else
		CHECK;
/* Verify that XEventsQueued returned the expected number of events. */
	if (info.int_return != eventsput) {
		delete("Unexpected number of events in event queue: %d", info.int_return);
		return;
	}
	else
		CHECK;
	CHECKPASS(9);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEventsQueued-3.(A)
 * When the number of events already in the event queue is
 * non-zero, then a call to XEventsQueued does not flush the
 * output buffer.
 */
static void
t003()
{
XEvent	event;
int	eventsput;
int	event_count;
Pixmap	pm;
Display *client2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create client2. */
	client2 = opendisplay();
	if (client2 == (Display *) NULL) {
		delete("Can not open display");
		return;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Create pixmap. */
	/* avoid using makepixm() */
	pm = XCreatePixmap(display, DRW(display), 10, 10, 1);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = MapNotify;
	eventsput = 0;
	XPutBackEvent(display, &event), eventsput++;
/* Call XEventsQueued with mode QueuedAlready. */
	mode = QueuedAlready;
	_xcall_(event_count);
/* Empty the buffer. */
	(void)XTestDiscard(display);
/* Ensure the server has dealt with anything flushed to it: do XSync() */
	XSync(display, False);
/* Verify that the output buffer was not flushed by effect on server. */
	_startcall(client2);
	XFreePixmap(client2, pm);
	XSync(client2, True);
	_endcall(client2);
	if (geterr() == Success) {
		/* pixmap was free'd */
		report("The output buffer was flushed.");
		FAIL;
	}
	else	/* no need to free as not created */
		CHECK;

	if (event_count != eventsput) {
		report("Incorrect number of events returned.");
		FAIL;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Create pixmap. */
	/* avoid using makepixm() */
	pm = XCreatePixmap(display, DRW(display), 10, 10, 1);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = MapNotify;
	eventsput = 0;
	XPutBackEvent(display, &event), eventsput++;
/* Call XEventsQueued with mode QueuedAfterFlush. */
	mode = QueuedAfterFlush;
	_xcall_(event_count);
/* Empty the buffer. */
	(void)XTestDiscard(display);
/* Ensure the server has dealt with anything flushed to it: do XSync() */
	XSync(display, False);
/* Verify that the output buffer was not flushed by effect on server. */
	_startcall(client2);
	XFreePixmap(client2, pm);
	XSync(client2, True);
	_endcall(client2);
	if (geterr() == Success) {
		/* pixmap was free'd */
		report("The output buffer was flushed.");
		FAIL;
	}
	else	/* no need to free as not created */
		CHECK;

	if (event_count != eventsput) {
		report("Incorrect number of events returned.");
		FAIL;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Create pixmap. */
	/* avoid using makepixm() */
	pm = XCreatePixmap(display, DRW(display), 10, 10, 1);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = MapNotify;
	eventsput = 0;
	XPutBackEvent(display, &event), eventsput++;
/* Call XEventsQueued with mode QueuedAfterReading. */
	mode = QueuedAfterReading;
	_xcall_(event_count);
/* Empty the buffer. */
	(void)XTestDiscard(display);
/* Ensure the server has dealt with anything flushed to it: do XSync() */
	XSync(display, False);
/* Verify that the output buffer was not flushed by effect on server. */
	_startcall(client2);
	XFreePixmap(client2, pm);
	XSync(client2, True);
	_endcall(client2);
	if (geterr() == Success) {
		/* pixmap was free'd */
		report("The output buffer was flushed.");
		FAIL;
	}
	else	/* no need to free as not created. */
		CHECK;

	if (event_count != eventsput) {
		report("Incorrect number of events returned.");
		FAIL;
	}
	else
		CHECK;
/* Discard all left-over events in the event queue. */
	_startcall(display);
	XSync(display, True);
	_endcall(display);

	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEventsQueued-4.(A)
 * When there are no events in the event queue, then a call to
 * XEventsQueued with mode set to QueuedAlready returns zero.
 */
static void
t004()
{
int	event_count;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XEventsQueued with mode QueuedAlready. */
	mode = QueuedAlready;
	_xcall_(event_count);
/* Verify that XEventsQueued returned the correct number of events. */
	if (event_count != 0) {
		report("Returned %d, expected none", event_count);
		FAIL;
	}
	else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEventsQueued-5.(A)
 * When there are no events in the event queue, then a call to
 * XEventsQueued with mode set to QueuedAfterFlush flushes the
 * output buffer.
 */
static void
t005()
{
int	event_count;
Pixmap	pm;
Display *client2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create client2. */
	client2 = opendisplay();
	if (client2 == (Display *) NULL) {
		delete("Can not open display");
		return;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Create pixmap. */
	/* avoid using makepixm() */
	pm = XCreatePixmap(display, DRW(display), 10, 10, 1);
/* Call XEventsQueued with mode QueuedAfterFlush. */
	mode = QueuedAfterFlush;
	_xcall_(event_count);
/* Empty the buffer. */
	(void)XTestDiscard(display);
/* Ensure the server has dealt with anything flushed to it: do XSync() */
	XSync(display, False);
/* Verify that the output buffer was flushed by effect on server. */
	_startcall(client2);
	XFreePixmap(client2, pm);
	XSync(client2, True);
	_endcall(client2);
	if (geterr() != Success) {
		report("The output buffer was not flushed.");
		XFreePixmap(display, pm);
		FAIL;
	}
	else
		CHECK;
/* Verify that XEventsQueued returned the correct number of events. */
	if (event_count != 0) {
		report("Returned %d, expected none", event_count);
		FAIL;
	}
	else
		CHECK;
/* Verify that errors did not occur after call to XEventsQueued. */
	if (geterr() != Success) {
		report("The output buffer was flushed.");
		FAIL;
	}
	else
		CHECK;

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEventsQueued-6.(A)
 * When there are no events in the event queue, then a call to
 * XEventsQueued with mode set to QueuedAfterFlush or
 * QueuedAfterReading attempts to read more events out of the
 * client's connection without blocking and returns the number
 * read.
 */
static void
t006()
{
Block_Info info;
int	block_status;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XEventsQueued with mode QueuedAfterFlush */
/* and verify that blocking did not occur. */
	XEventsQueued_Type(info, QueuedAfterFlush);
	block_status = block(display, (XEvent *) NULL, &info);
	if (block_status == -1)
		return;
	else
		CHECK;
	if (block_status == 1) {
		report("Blocking occurred with QueuedAfterFlush when it should not have.");
		FAIL;
	}
	else
		CHECK;
	if (info.int_return != 0) {
		delete("Unexpected number of events in event queue: %d", info.int_return);
		return;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XEventsQueued with mode QueuedAfterReading */
/* and verify that blocking did not occur. */
	XEventsQueued_Type(info, QueuedAfterReading);
	block_status = block(display, (XEvent *) NULL, &info);
	if (block_status == -1)
		return;
	else
		CHECK;
	if (block_status == 1) {
		report("Blocking occurred with QueuedAfterReading when it should not have.");
		FAIL;
	}
	else
		CHECK;
	if (info.int_return != 0) {
		delete("Unexpected number of events in event queue: %d", info.int_return);
		return;
	}
	else
		CHECK;
	CHECKPASS(6);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
