/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: exps.m,v 1.8 92/06/11 17:20:54 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_Expose	1
char    *TestName = "Expose";



#define	EVENT		Expose
#define	MASK		ExposureMask

int 	tet_thistest;

/*
 * Assertion Expose-1.(A)
 * The server does not generate Expose events on windows whose
 * class is specified as InputOnly.
 */
static void
t001()
{
Display	*display = Dsp;
Window	w;
XEvent	event_return;
int	events;
int 	pass = 0, fail = 0;

	tpstartup();

#ifdef	OTHERMASK
#undef	OTHERMASK
#endif
#define	OTHERMASK	StructureNotifyMask
#ifdef	OTHEREVENT
#undef	OTHEREVENT
#endif
#define	OTHEREVENT	MapNotify

/* Create InputOnly window. */
	w = iponlywin(display);
/* Select for Expose and MapNotify events on window. */
	XSelectInput(display, w, MASK|OTHERMASK);
/* Map window. */
	XSync(display, True);
	XMapWindow(display, w);
	XSync(display, False);
/* Verify that a MapNotify event was received. */
	if (!XCheckTypedWindowEvent(display, w, OTHEREVENT, &event_return)) {
		delete("Expected %s event, got none", eventname(OTHEREVENT));
		return;
	}
	else
		CHECK;
/* Verify that no Expose events were received. */
	if (XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		delete("Got %s event, expected none", eventname(EVENT));
		return;
	}
	else
		CHECK;
	events = XPending(display);
	if (events > 0) {
		delete("Received %d more events than expected", events);
		return;
	}
	else
		CHECK;

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion Expose-2.(-)
 * When no valid contents are available for regions of a
 * window and the regions are visible, then an Expose event is
 * generated.
 */
static void
t002()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion Expose-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When no valid contents are available for regions of a
 * window and the regions are viewable but not visible and the
 * server is maintaining backing store on the window, then an
 * Expose event is generated.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion Expose-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When no valid contents are available for regions of a
 * window and the window is not viewable but the server is
 * honoring the window's backing-store attribute of Always or
 * WhenMapped, then an Expose event is generated.
 */
static void
t004()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion Expose-5.(A)
 * When an Expose events are generated, then they are
 * contiguously delivered by the server.
 */
static void
t005()
{
Display	*display = Dsp;
Winh	*eventw;
Winh	*child;
Winhe	*winhe;
int	expected;
int	lastcount;
int 	pass = 0, fail = 0;

	tpstartup();

/* Build and create window hierarchy. */
	if (winh(display, 2, WINH_MAP)) {
		report("Could not build window hierarchy");
		return;
	}
	else
		CHECK;
/* Choose event window. */
	eventw = guardian->firstchild;
/* Select for Expose events on event window. */
	if (winh_selectinput(display, eventw, MASK))
		return;
	else
		CHECK;
/* Generate Expose events on event window. */
	XSync(display, True);
	for (child = eventw->firstchild; child != (Winh *) NULL; child = child->nextsibling) {
		static	int	first = 1;

		if (first) {
			first = 0;
			CHECK;
		}
		XUnmapWindow(display, child->window);
	}
	XSync(display, False);
/* Verify that events are contiguously delivered. */
	if (winh_harvest(display, (Winh *) NULL)) {
		report("Could not harvest events");
		return;
	}
	else
		CHECK;
	if (winh_qdel == (Winhe *) NULL) {
		delete("No events delivered.");
		return;
	}
	else
		CHECK;
	expected = -1;
	for (winhe = winh_qdel; winhe != (Winhe *) NULL; winhe = winhe->next) {
		if (expected == -1)
			CHECK;
		if (winhe->event->type != EVENT) {
			report("Received %s event while only expecting %s types",
				eventname(winhe->event->type), eventname(EVENT));
			delete("Unexpected event received.");
			return;
		}
		lastcount = winhe->event->xexpose.count;
		if (expected == -1) {
			expected = lastcount;
		}
		else {
			if (lastcount > expected) {
				report("Count (%d) greater than expected (%d)",
					lastcount, expected);
				FAIL;
				break;
			}
			else if (lastcount < expected)
				expected--;
		}
	}
	if (lastcount != 0) {
		report("Last %s had count set to %d, not zero",
			eventname(EVENT), lastcount);
		FAIL;
	}
	else
		CHECK;

	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion Expose-6.(A)
 * When an Expose event is generated, then all clients having
 * set ExposureMask event mask bits on the event window are
 * delivered an Expose event.
 */
static void
t006()
{
Display	*display = Dsp;
Display	*client2, *client3;
Winh	*eventw;
int	events;
Window	w;
XEvent	event_return;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
/* Build and create window hierarchy. */
	if (winh(display, 2, WINH_MAP)) {
		report("Could not build window hierarchy");
		return;
	}
	else
		CHECK;
/* Choose event window. */
	eventw = guardian->firstchild;
	w = eventw->window;
/* Select for Expose events on event window. */
	XSelectInput(display, w, MASK);
/* Select for Expose events on event window with client2. */
	XSelectInput(client2, w, MASK);
/* Select for no events on event window with client3. */
	XSelectInput(client3, w, NoEventMask);
/* Generate Expose events on event window. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XUnmapWindow(display, eventw->firstchild->window);
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that events are delivered to selecting clients. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		delete("Expected %s event, got none", eventname(EVENT));
		return;
	}
	else
		CHECK;
	if (!XCheckTypedWindowEvent(client2, w, EVENT, &event_return)) {
		delete("Expected %s event, got none with client2", eventname(EVENT));
		return;
	}
	else
		CHECK;
/* Verify that no Expose events are received for client3. */
	if (XCheckTypedWindowEvent(client3, w, EVENT, &event_return)) {
		delete("Got %s event, expected none", eventname(EVENT));
		return;
	}
	else
		CHECK;

	CHECKPASS(6);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion Expose-7.(-)
 * When an Expose event is generated, then clients not having
 * set ExposureMask event mask bits on the event window are not
 * delivered an Expose event.
 */
static void
t007()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
