/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gterrrdtbs.m,v 1.7 92/06/11 17:21:46 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XGetErrorDatabaseText(display, name, message, default_string, buffer_return, length)
 *   Display *display;
 *   char *name;
 *   char *message;
 *   char *default_string;
 *   char *buffer_return;
 *   int length;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetErrorDatabaseText	1
char    *TestName = "XGetErrorDatabaseText";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGetErrorDatabaseText function
 */
static Display *display;
static char *name;
static char *message;
static char *default_string;
static char *buffer_return;
static int length;


#include "Xproto.h"

static char buffer[512];

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	name = 0;
	message = 0;
	default_string = "default_string";
	buffer_return = buffer;
	length = sizeof(buffer);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetErrorDatabaseText-1.(A)
 * A call to XGetErrorDatabaseText returns a string in
 * buffer_return.
 */
static void
t001()
{
int	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Set each character in buffer_return to non-ASCII NUL. */
	for (i=0; i < sizeof(buffer); i++) {
		if (!i)
			CHECK;
		buffer[i] = 'A';
	}
	buffer_return = buffer;
/* Set name to "foo". */
	name = "foo";
/* Set message to "bar". */
	message = "bar";
/* Call XGetErrorText. */
	startcall(display);
	if (isdeleted())
		return;
	XGetErrorDatabaseText(display, name, message, default_string, buffer_return, length);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that there exists at least one ASCII NUL */
/* character in buffer_return. */
	for (i=0; i < sizeof(buffer); i++) {
		if (buffer[i] == '\0') {
			CHECK;
			break;
		}
	}
	if (i == sizeof(buffer)) {
		report("String not returned.");
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetErrorDatabaseText-2.(A)
 * A call to XGetErrorDatabaseText with name set to the string
 * XProtoError and message set to the string representation of
 * a protocol error number returns a string terminated with
 * ASCII NUL in buffer_return.
 */
static void
t002()
{
int	i;
char	msgbuf[512];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Set each character in buffer_return to non-ASCII NUL. */
	for (i=0; i < sizeof(buffer); i++) {
		if (!i)
			CHECK;
		buffer[i] = 'A';
	}
	buffer_return = buffer;
/* Set name to "XProtoError". */
	name = "XProtoError";
/* Set message to the string representation of BadWindow. */
	(void) sprintf(msgbuf, "%d", BadWindow);
	message = msgbuf;
/* Call XGetErrorText. */
	startcall(display);
	if (isdeleted())
		return;
	XGetErrorDatabaseText(display, name, message, default_string, buffer_return, length);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that there exists at least one ASCII NUL */
/* character in buffer_return. */
	for (i=0; i < sizeof(buffer); i++) {
		if (buffer[i] == '\0') {
			CHECK;
			break;
		}
	}
	if (i == sizeof(buffer)) {
		report("Returned string not terminated with ASCII NUL.");
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetErrorDatabaseText-3.(A)
 * A call to XGetErrorDatabaseText with name set to the string
 * XRequest and message set to the string representation of a
 * major protocol number returns a string terminated with ASCII
 * NUL in buffer_return.
 */
static void
t003()
{
int	i;
char	msgbuf[512];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Set each character in buffer_return to non-ASCII NUL. */
	for (i=0; i < sizeof(buffer); i++) {
		if (!i)
			CHECK;
		buffer[i] = 'A';
	}
	buffer_return = buffer;
/* Set name to "XRequest". */
	name = "XRequest";
/* Set message to the string representation of X_DestroyWindow. */
	(void) sprintf(msgbuf, "%d", X_DestroyWindow);
	message = msgbuf;
/* Call XGetErrorText. */
	startcall(display);
	if (isdeleted())
		return;
	XGetErrorDatabaseText(display, name, message, default_string, buffer_return, length);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that there exists at least one ASCII NUL */
/* character in buffer_return. */
	for (i=0; i < sizeof(buffer); i++) {
		if (buffer[i] == '\0') {
			CHECK;
			break;
		}
	}
	if (i == sizeof(buffer)) {
		report("Returned string not terminated with ASCII NUL.");
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetErrorDatabaseText-4.(D)
 *   Reason: There is no known portable test method for this assertion
 * If the implementation supports an error database: The error
 * message database used is/usr/lib/X11/XErrorDB.
 */
static void
t004()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XGetErrorDatabaseText-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a call to XGetErrorDatabaseText is made and no string
 * is found in the error database, then default_string is
 * returned in buffer_return.
 */
static void
t005()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
