/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtmtnevnts.m,v 1.10 92/06/11 17:22:02 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XTimeCoord *
 *   XGetMotionEvents(display, w, start, stop, nevents_return)
 *   Display *display;
 *   Window w;
 *   Time start;
 *   Time stop;
 *   int *nevents_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetMotionEvents	1
char    *TestName = "XGetMotionEvents";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XGetMotionEvents function
 */
static Display *display;
static Window w;
static Time start;
static Time stop;
static int *nevents_return;


static	int	_nevents_return;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(display);
	start = CurrentTime;
	stop = CurrentTime;
	nevents_return = &_nevents_return;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetMotionEvents-1.(D)
 *   Reason: There is no known portable test method for this assertion
 * If the implementation supports a more complete history of
 * the pointer motion than is reported by event notification: a
 * call to XGetMotionEvents returns all events in the motion
 * history buffer that fall between the start and stop times,
 * inclusive, which have coordinates that lie within the
 * specified window (including its borders) at its present
 * placement and sets nevents_return to the number of events
 * returned.
 */
static void
t001()
{
/* If a pointer motion buffer is not supported, return. */
	if (!config.displaymotionbuffersize)
		unsupported("Pointer motion buffer is not supported.");
	else
		untested("There is no known portable test method for this assertion");
}

/*
 * Assertion XGetMotionEvents-2.(C)
 * If the implementation does not support a more complete
 * history of pointer motion than is reported by event
 * notification: a call to XGetMotionEvents returns no events.
 */
static void
t002()
{
XTimeCoord *tc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* If a pointer motion buffer is supported, return. */
	if (config.displaymotionbuffersize != 0) {
		report("Pointer motion buffer is supported.");
		UNSUPPORTED;
		return;
	}
	else
		CHECK;
	start = 0;
	stop = CurrentTime;
	*nevents_return = 1;
/* Call XGetMotionEvents. */
	startcall(display);
	if (isdeleted())
		return;
	tc = XGetMotionEvents(display, w, start, stop, nevents_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that no events were returned. */
	if (tc != (XTimeCoord *) NULL) {
		report("Returned 0x%x, expected NULL", tc);
		FAIL;
		XFree((char*)tc);
	}
	else
		CHECK;
	if (*nevents_return != 0) {
		report("Returned %d, expected 0", *nevents_return);
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetMotionEvents-3.(A)
 * When start is later than stop, then a call to
 * XGetMotionEvents returns no events.
 */
static void
t003()
{
XTimeCoord *tc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Set stop to current time. */
	stop = gettime(display);
/* Call XGetMotionEvents with start greater than stop. */
	start = stop + 1;
	startcall(display);
	if (isdeleted())
		return;
	tc = XGetMotionEvents(display, w, start, stop, nevents_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that no events were returned. */
	if (tc != (XTimeCoord *) NULL) {
		report("Returned 0x%x, expected NULL", tc);
		FAIL;
		XFree((char*)tc);
	}
	else
		CHECK;
	if (*nevents_return != 0) {
		report("Returned %d, expected 0", *nevents_return);
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetMotionEvents-4.(A)
 * When start is in the future, then a call to
 * XGetMotionEvents returns no events.
 */
static void
t004()
{
XTimeCoord *tc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Set stop to current time. */
	stop = CurrentTime;
/* Set start to a future time. */
	start = gettime(display) + 10000;
/* Call XGetMotionEvents. */
	startcall(display);
	if (isdeleted())
		return;
	tc = XGetMotionEvents(display, w, start, stop, nevents_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that no events were returned. */
	if (tc != (XTimeCoord *) NULL) {
		report("Returned 0x%x, expected NULL", tc);
		FAIL;
		XFree((char*)tc);
	}
	else
		CHECK;
	if (*nevents_return != 0) {
		report("Returned %d, expected 0", *nevents_return);
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetMotionEvents-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XGetMotionEvents with stop in the future, is
 * equivalent to specifying a value of CurrentTime for stop.
 */
static void
t005()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XGetMotionEvents-6.(D)
 *   Reason: There is no known portable test method for this assertion
 * On a call to XGetMotionEvents the x and y members of the
 * events returned are set to the coordinates of the pointer
 * relative to the origin of the specified window and the time
 * member is set to the time the pointer reached this
 * coordinate.
 */
static void
t006()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EWin.mc */
/*
 * Assertion XGetMotionEvents-7.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XGetMotionEvents(display, w, start, stop, nevents_return);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
