/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: mppngntfy.m,v 1.8 92/06/11 17:23:01 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_MappingNotify	1
char    *TestName = "MappingNotify";



#define	EVENT		MappingNotify

int 	tet_thistest;

/*
 * Assertion MappingNotify-1.(A)
 * When a MappingNotify event is generated, a MappingNotify
 * event is delivered to all clients.
 */
static void
t001()
{
Display	*display = Dsp;
Display	*client2;
XEvent	event_return;
int	nmap;
int	count;
unsigned char *map;
unsigned char buf[2];
int 	pass = 0, fail = 0;

	tpstartup();

/* Create client2. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
/* Generate MappingNotify event. */
	nmap = XGetPointerMapping(display, buf, sizeof(buf));
	if (nmap < 1) {
		report("XGetPointerMapping returned %d, expected greater than 0", nmap);
		delete("Can't determine size of pointer mapping buffer");
		return;
	}
	else
		CHECK;
	map = (unsigned char *) malloc((unsigned int) (nmap * sizeof(*map)));
	if (map == (unsigned char *) NULL) {
		report("Couldn't allocate %d bytes", nmap * sizeof(*map));
		delete("Memory allocation error");
		return;
	}
	else
		CHECK;
	count = XGetPointerMapping(display, map, nmap);
	if (count < nmap) {
		report("XGetPointerMapping returned %d, expected %d",
			count, nmap);
		delete("XGetPointerMapping inconsistencies");
		free(map);
		return;
	}
	else
		CHECK;
	XSync(display, True);
	XSync(client2, True);
	count = XSetPointerMapping(display, map, nmap);
	if (count != Success) {
		report("XSetPointerMapping returned %d, expected %d",
			count, Success);
		delete("XSetPointerMapping failed");
		free(map);
		return;
	}
	else
		CHECK;
	XSync(display, False);
	XSync(client2, False);
/* Verify that MappingNotify event was delivered. */
	if (!XCheckTypedEvent(display, EVENT, &event_return)) {
		report("No events generated, expected %s event",
			eventname(EVENT));
		FAIL;
	}
	else {
		XMappingEvent good;

/* Verify members in delivered event structure. */
		good = event_return.xmapping;
		good.type = EVENT;
		good.request = MappingPointer;
		if (checkevent((XEvent *) &good, &event_return)) {
			report("Unexpected values in delivered event");
			FAIL;
		}
		else
			CHECK;
	}
/* Verify that MappingNotify event was delivered to client2. */
	if (!XCheckTypedEvent(client2, EVENT, &event_return)) {
		report("No events generated, expected %s event with client2",
			eventname(EVENT));
		FAIL;
	}
	else {
		XMappingEvent good;

/* Verify members in delivered event structure. */
		good = event_return.xmapping;
		good.type = EVENT;
		good.request = MappingPointer;
		if (checkevent((XEvent *) &good, &event_return)) {
			report("Unexpected values in delivered event with client2");
			FAIL;
		}
		else
			CHECK;
	}
	free(map);

	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
