/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: prprtyntfy.m,v 1.8 92/06/11 17:24:14 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_PropertyNotify	1
char    *TestName = "PropertyNotify";



#include	"Xatom.h"
#define	EVENT		PropertyNotify
#define	MASK		PropertyChangeMask

int 	tet_thistest;

/*
 * Assertion PropertyNotify-1.(A)
 * When a PropertyNotify event is generated, then all clients
 * having set PropertyChangeMask event mask bits on the event
 * window are delivered a PropertyNotify event.
 */
static void
t001()
{
Display	*display = Dsp;
Display	*client2;
Display	*client3;
Window	w;
int	count;
XEvent	event_return;
XPropertyEvent good;
Atom	atom;
int	data = 42;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
/* Create window. */
	w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Select for PropertyNotify events using PropertyChangeMask. */
	XSelectInput(display, w, MASK);
/* Select for PropertyNotify events using PropertyChangeMask with client2. */
	XSelectInput(client2, w, MASK);
/* Select for no events with client3. */
	XSelectInput(client3, w, NoEventMask);
/* Create atom for property. */
	atom = XInternAtom(display, XT_TIMESTAMP, False);
	if (atom == None) {
		delete("XInternAtom of '%s' failed.", XT_TIMESTAMP);
		return;
	}
	else
		CHECK;
/* Generate PropertyNotify event. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XChangeProperty(display, w, atom, XA_INTEGER, 32, PropModeReplace, (unsigned char *)&data, 1);
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that a PropertyNotify event was delivered. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xproperty;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.atom = atom;
	good.state = PropertyNewValue;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;
/* Verify that a PropertyNotify event was delivered to client2. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(client2, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xproperty;
	good.type = EVENT;
	good.send_event = False;
	good.display = client2;
	good.window = w;
	good.atom = atom;
	good.state = PropertyNewValue;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event for client2");
		FAIL;
	}
	else
		CHECK;
/* Verify that no events were delivered to client3. */
	count = XPending(client3);
	if (count != 0) {
		report("Got %d events, expected %d for client3", count, 0);
		FAIL;
		return;
	}
	else
		CHECK;
	
	CHECKPASS(8);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion PropertyNotify-2.(-)
 * When a PropertyNotify event is generated, then clients not
 * having set PropertyChangeMask event mask bits on the event
 * window are not delivered a PropertyNotify event.
 */
static void
t002()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
