/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rprntntfy.m,v 1.9 92/06/11 17:24:31 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_ReparentNotify	1
char    *TestName = "ReparentNotify";



#define	EVENT		ReparentNotify
#define	MASK		StructureNotifyMask
#define	MASKP		SubstructureNotifyMask

int 	tet_thistest;

/*
 * Assertion ReparentNotify-1.(A)
 * When a ReparentNotify event is generated, then all clients
 * having set StructureNotifyMask event mask bits on the
 * reparented window are delivered a ReparentNotify event.
 */
static void
t001()
{
Display	*display = Dsp;
Display	*client2;
Display	*client3;
Window	parent, w;
int	count;
XEvent	event_return;
XReparentEvent good;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
	/*
	 * The order in which these windows are created is critical.
	 * If the order were reversed, a BadWindow error would result
	 * when the resource registration routines destroyed these windows.
	 */
/* Create parent window. */
	parent = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Create window. */
	w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Select for ReparentNotify events using StructureNotifyMask. */
	XSelectInput(display, w, MASK);
/* Select for ReparentNotify events using StructureNotifyMask with client2. */
	XSelectInput(client2, w, MASK);
/* Select for no events with client3. */
	XSelectInput(client3, w, NoEventMask);
/* Generate ReparentNotify event. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XReparentWindow(display, w, parent, 1, 1);
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that a ReparentNotify event was delivered. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xreparent;
	good.type = EVENT;
	good.event = w;
	good.window = w;
	good.parent = parent;
	good.x = 1;
	good.y = 1;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;
/* Verify that a ReparentNotify event was delivered to client2. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(client2, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xreparent;
	good.type = EVENT;
	good.event = w;
	good.window = w;
	good.parent = parent;
	good.x = 1;
	good.y = 1;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event for client2");
		FAIL;
	}
	else
		CHECK;
/* Verify that no events were delivered to client3. */
	count = XPending(client3);
	if (count != 0) {
		report("Got %d events, expected %d for client3", count, 0);
		FAIL;
		return;
	}
	else
		CHECK;
	
	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ReparentNotify-2.(A)
 * When a ReparentNotify event is generated, then all clients
 * having set SubstructureNotifyMask event mask bits on the
 * parent of the reparented window are delivered a
 * ReparentNotify event.
 */
static void
t002()
{
Display	*display = Dsp;
Display	*client2;
Display	*client3;
Window	oldparent, parent, w;
int	count;
XEvent	event_return;
XReparentEvent good;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
	oldparent = DRW(display);
	/*
	 * The order in which these windows are created is critical.
	 * If the order were reversed, a BadWindow error would result
	 * when the resource registration routines destroyed these windows.
	 */
/* Create parent window. */
	parent = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Create window. */
	w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Select for ReparentNotify events on new parent using SubstructureNotifyMask. */
	XSelectInput(display, parent, MASKP);
/* Select for ReparentNotify events on old parent using SubstructureNotifyMask. */
	XSelectInput(display, oldparent, MASKP);
/* Select for ReparentNotify events on new parent using SubstructureNotifyMask with client2. */
	XSelectInput(client2, parent, MASKP);
/* Select for ReparentNotify events on old parent using SubstructureNotifyMask with client2. */
	XSelectInput(client2, oldparent, MASKP);
/* Select for no events on new parent with client3. */
	XSelectInput(client3, parent, NoEventMask);
/* Select for no events on old parent with client3. */
	XSelectInput(client3, oldparent, NoEventMask);
/* Generate ReparentNotify event. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XReparentWindow(display, w, parent, 1, 1);
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that a ReparentNotify event was delivered. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(display, parent, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xreparent;
	good.type = EVENT;
	good.event = parent;
	good.window = w;
	good.parent = parent;
	good.x = 1;
	good.y = 1;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;
/* Verify that a ReparentNotify event was delivered for oldparent. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(display, oldparent, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xreparent;
	good.type = EVENT;
	good.event = oldparent;
	good.window = w;
	good.parent = parent;
	good.x = 1;
	good.y = 1;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;
/* Verify that a ReparentNotify event was delivered to client2. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(client2, parent, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xreparent;
	good.type = EVENT;
	good.event = parent;
	good.window = w;
	good.parent = parent;
	good.x = 1;
	good.y = 1;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event for client2");
		FAIL;
	}
	else
		CHECK;
/* Verify that a ReparentNotify event was delivered to client2 for old parent. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(client2, oldparent, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xreparent;
	good.type = EVENT;
	good.event = oldparent;
	good.window = w;
	good.parent = parent;
	good.x = 1;
	good.y = 1;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event for client2");
		FAIL;
	}
	else
		CHECK;
/* Verify that no events were delivered to client3. */
	count = XPending(client3);
	if (count != 0) {
		report("Got %d events, expected %d for client3", count, 0);
		FAIL;
		return;
	}
	else
		CHECK;
	
	CHECKPASS(11);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion ReparentNotify-3.(-)
 * When a ReparentNotify event is generated, then all clients
 * having set SubstructureNotifyMask event mask bits on either
 * the old or new parent of the reparented window are delivered
 * a ReparentNotify event.
 */
static void
t003()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion ReparentNotify-4.(-)
 * When a ReparentNotify event is generated, then clients not
 * having set StructureNotifyMask event mask bits on the
 * reparented window and also not having set
 * SubstructureNotifyMask event mask bits on either the old or
 * new parent of the reparented window are not delivered a
 * ReparentNotify event.
 */
static void
t004()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
