/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: vsbltyntfy.m,v 1.9 92/06/11 17:26:22 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_VisibilityNotify	1
char    *TestName = "VisibilityNotify";



#define	EVENT		VisibilityNotify
#define	MASK		VisibilityChangeMask

int 	tet_thistest;

/*
 * Assertion VisibilityNotify-1.(A)
 * The server does not generate VisibilityNotify events on
 * windows whose class is specified as InputOnly.
 */
static void
t001()
{
Display	*display = Dsp;
Window	w;
int	count;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create class InputOnly window. */
	w = iponlywin(display);
/* Select for VisibilityNotify events. */
	XSelectInput(display, w, MASK);
/* Attempt to generate VisibilityNotify event on this window. */
	XSync(display, True);
	XMapWindow(display, w);
	XSync(display, False);
/* Verify that no events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d", count, 0);
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion VisibilityNotify-2.(A)
 * When a VisibilityNotify event is generated by a hierarchy
 * change, then the VisibilityNotify event is delivered after
 * any hierarchy event.
 */
static void
t002()
{
Display	*display;
Winh	*eventw, *sibling, *rootw;
XEvent	event;
Winhg	winhg;
int	status;
int 	pass = 0, fail = 0;

	tpstartup();

#ifdef	OTHERMASK
#undef	OTHERMASK
#endif
#define	OTHERMASK	SubstructureNotifyMask
#ifdef	OTHEREVENT
#undef	OTHEREVENT
#endif
#define	OTHEREVENT	MapNotify

/* Create client. */
	/*
	 * Can not use Dsp because we are selecting on root window.
	 * We could instead de-select on root window prior to returning,
	 * but this is actually easier.
	 */
	display = opendisplay();
	if (display == (Display *) NULL) {
		delete("Could not open display.");
		return;
	}
	else
		CHECK;
/* Create window large enough to be partially obscurred by each top-level */
/* window in a "standard" window hierarchy. */
	winhg.area.x = 1;
	winhg.area.y = 1;
	winhg.area.width = DisplayWidth(display, DefaultScreen(display));
	winhg.area.height = DisplayHeight(display, DefaultScreen(display));
	/*
	 * if too large then it will extend off edge of screen and
	 * be (permanently) partially obscurred
	 */
	winhg.area.width -= 4;
	winhg.area.height -= 4;
	winhg.border_width = 1;
	eventw = winh_adopt(display, (Winh *) NULL, 0L, (XSetWindowAttributes *) NULL, &winhg, WINH_MAP);
	if (eventw == (Winh *) NULL) {
		report("Could not create first window");
		return;
	}
	else
		CHECK;
/* Build and create window hierarchy. */
	if (winh(display, 1, WINH_NOMASK)) {
		report("Could not build window hierarchy");
		return;
	}
	else
		CHECK;
/* Select for VisibilityNotify events on eventw. */
	if (winh_selectinput(display, eventw, MASK))
		return;
	else
		CHECK;
/* Select for MapNotify events on root window. */
	rootw = guardian;
	if (winh_selectinput(display, rootw, OTHERMASK))
		return;
	else
		CHECK;
	sibling = eventw->nextsibling;
/* Generate VisibilityNotify and MapNotify events. */
	XSync(display, True);
	XMapWindow(display, sibling->window);
	XSync(display, False);
/* Initialize for expected events. */
	event.xany.type = OTHEREVENT;
	event.xany.window = rootw->window;
	if (winh_plant(rootw, &event, OTHERMASK, WINH_NOMASK))
		return;
	else
		CHECK;
	event.xany.type = EVENT;
	event.xany.window = eventw->window;
	if (winh_plant(eventw, &event, MASK, WINH_NOMASK))
		return;
	else
		CHECK;
/* Harvest events from event queue. */
	if (winh_harvest(display, (Winh *) NULL)) {
		report("Could not harvest events");
		return;
	}
	else
		CHECK;
/* Verify that expected events were delivered. */
	status = winh_weed((Winh *) NULL, -1, WINH_WEED_IDENTITY);
	if (status < 0)
		return;
	else if (status > 0) {
		report("Event delivery was not as expected");
		FAIL;
	}
	else
		CHECK;
/* Verify that all VisibilityNotify events are delivered after all */
/* MapNotify events. */
	status = winh_ordercheck(OTHEREVENT, EVENT);
	if (status == -1)
		return;
	else if (status)
		FAIL;
	else
		CHECK;

	CHECKPASS(10);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion VisibilityNotify-3.(A)
 * When a VisibilityNotify event is generated, then the
 * VisibilityNotify event is delivered before any Expose events
 * on that window.
 */
static void
t003()
{
Display	*display = Dsp;
Winh	*eventw;
XEvent	event;
Winhg	winhg;
Window	w;
int	status;
int 	pass = 0, fail = 0;

	tpstartup();

#ifdef	OTHERMASK
#undef	OTHERMASK
#endif
#define	OTHERMASK	ExposureMask
#ifdef	OTHEREVENT
#undef	OTHEREVENT
#endif
#define	OTHEREVENT	Expose

/* Create window large enough to be partially obscurred by each top-level */
/* window in a "standard" window hierarchy. */
	winhg.area.x = 1;
	winhg.area.y = 1;
	winhg.area.width = DisplayWidth(display, DefaultScreen(display));
	winhg.area.height = DisplayHeight(display, DefaultScreen(display));
	/*
	 * if too large then it will extend off edge of screen and
	 * be (permanently) partially obscurred
	 */
	winhg.area.width -= 4;
	winhg.area.height -= 4;
	winhg.border_width = 1;
	eventw = winh_adopt(display, (Winh *) NULL, 0L, (XSetWindowAttributes *) NULL, &winhg, WINH_MAP);
	if (eventw == (Winh *) NULL) {
		report("Could not create first window");
		return;
	}
	else
		CHECK;
/* Build and create window hierarchy. */
	if (winh(display, 0, WINH_NOMASK)) {
		report("Could not build window hierarchy");
		return;
	}
	else
		CHECK;
/* Create and map a window. */
	w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, True);
/* Select for VisibilityNotify and Expose events on eventw. */
	if (winh_selectinput(display, eventw, MASK|OTHERMASK))
		return;
	else
		CHECK;
/* Generate VisibilityNotify and MapNotify events. */
	XSync(display, True);
	XUnmapWindow(display, w);
	XSync(display, False);
/* Initialize for expected events. */
	event.xany.type = EVENT;
	event.xany.window = eventw->window;
	if (winh_plant(eventw, &event, MASK, WINH_NOMASK))
		return;
	else
		CHECK;
/* Harvest events from event queue. */
	if (winh_harvest(display, (Winh *) NULL)) {
		report("Could not harvest events");
		return;
	}
	else
		CHECK;
/* Ignore Expose events. */
	if (winh_ignore_event((Winh *) NULL, OTHEREVENT, WINH_NOMASK)) {
		report("Could not mark %s events to be ignored",
			eventname(OTHEREVENT));
		return;
	}
	else
		CHECK;
/* Verify that expected events were delivered. */
	status = winh_weed((Winh *) NULL, -1, WINH_WEED_IDENTITY);
	if (status < 0)
		return;
	else if (status > 0) {
		report("Event delivery was not as expected");
		FAIL;
	}
	else
		CHECK;
/* Verify that all VisibilityNotify events are delivered before all */
/* Expose events. */
	status = winh_ordercheck(EVENT, OTHEREVENT);
	if (status == -1)
		return;
	else if (status)
		FAIL;
	else
		CHECK;

	CHECKPASS(8);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion VisibilityNotify-4.(A)
 * When a VisibilityNotify event is generated, then all
 * clients having set VisibilityChangeMask event mask bits on
 * the event window are delivered a VisibilityNotify event.
 */
static void
t004()
{
Display	*display = Dsp;
Display	*client2;
Display	*client3;
Window	w;
int	count;
XEvent	event_return;
XVisibilityEvent good;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
/* Create window. */
	w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Select for VisibilityNotify events using VisibilityChangeMask. */
	XSelectInput(display, w, MASK);
/* Select for VisibilityNotify events using VisibilityChangeMask with client2. */
	XSelectInput(client2, w, MASK);
/* Select for no events with client3. */
	XSelectInput(client3, w, NoEventMask);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XMapWindow(display, w);
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.window = w;
	good.state = VisibilityUnobscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;
/* Verify that a VisibilityNotify event was delivered to client2. */
	if (!XCheckTypedWindowEvent(client2, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.window = w;
	good.state = VisibilityUnobscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event for client2");
		FAIL;
	}
	else
		CHECK;
/* Verify that no events were delivered to client3. */
	count = XPending(client3);
	if (count != 0) {
		report("Got %d events, expected %d for client3", count, 0);
		FAIL;
		return;
	}
	else
		CHECK;
	
	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion VisibilityNotify-5.(-)
 * When a VisibilityNotify event is generated, then clients
 * not having set VisibilityChangeMask event mask bits on the
 * event window are not delivered a VisibilityNotify event.
 */
static void
t005()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion VisibilityNotify-6.(-)
 * When a VisibilityNotify event is delivered, then window is
 * set to the window whose visibility state changes.
 */
static void
t006()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion VisibilityNotify-7.(A)
 * When a VisibilityNotify event is delivered and window
 * changes state from partially obscured, fully obscured, or
 * not viewable to viewable and completely unobscured, then
 * state is set to VisibilityUnobscured.
 */
static void
t007()
{
Display	*display = Dsp;
Window	w, w2;
struct area	area;
XEvent	event_return;
XVisibilityEvent	good;
int	count;
int 	pass = 0, fail = 0;

	tpstartup();

	/*
	 * test ordering:
	 *
	 *	not viewable		-> viewable and completely unobscured
	 *	fully obscured		-> viewable and completely unobscured
	 *	partially obscured	-> viewable and completely unobscured
	 */
/* Create window. */
	area.x = 1;
	area.y = 1;
	area.width = W_STDWIDTH;
	area.height = W_STDHEIGHT;
	w = mkwin(display, (XVisualInfo *) NULL, &area, False);
/* Select for VisibilityNotify events on window. */
	XSelectInput(display, w, MASK);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XMapWindow(display, w);
	XSync(display, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none (not viewable test)", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.state = VisibilityUnobscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event (not viewable test)");
		FAIL;
	}
	else
		CHECK;
/* Verify that no other events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d (not viewable test)", count+1, 1);
		FAIL;
	}
	else
		CHECK;
/* Fully obscure window. */
	w2 = mkwin(display, (XVisualInfo *) NULL, &area, True);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XUnmapWindow(display, w2);
	XSync(display, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none (fully obscured test)", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.state = VisibilityUnobscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event (fully obscured test)");
		FAIL;
	}
	else
		CHECK;
/* Verify that no other events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d (fully obscured test)", count+1, 1);
		FAIL;
	}
	else
		CHECK;
/* Partially obscure window. */
	area.x = 1;
	area.y = 1;
	area.width = W_STDWIDTH/2;
	area.height = W_STDHEIGHT/2;
	w2 = mkwin(display, (XVisualInfo *) NULL, &area, True);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XUnmapWindow(display, w2);
	XSync(display, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none (partially obscured test)", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.state = VisibilityUnobscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event (partially obscured test)");
		FAIL;
	}
	else
		CHECK;
/* Verify that no other events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d (partially obscured test)", count+1, 1);
		FAIL;
	}
	else
		CHECK;
	
	CHECKPASS(9);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion VisibilityNotify-8.(A)
 * When a VisibilityNotify event is delivered and window
 * changes state from viewable and completely unobscured or not
 * viewable to viewable and partially obscured, then state is
 * set to VisibilityPartiallyObscured.
 */
static void
t008()
{
Display	*display = Dsp;
Window	w, wpo;
struct area	area;
XEvent	event_return;
XVisibilityEvent	good;
int	count;
int 	pass = 0, fail = 0;

	tpstartup();

	/*
	 * test ordering:
	 *
	 *	not viewable
	 *		-> viewable and partially obscured
	 *	viewable & completely unobscured
	 *		-> viewable & partially unobscured
	 */
/* Create window. */
	area.x = 1;
	area.y = 1;
	area.width = W_STDWIDTH;
	area.height = W_STDHEIGHT;
	w = mkwin(display, (XVisualInfo *) NULL, &area, False);
/* Create partially obscuring window. */
	area.x += W_STDWIDTH/2;
	area.y += W_STDHEIGHT/2;
	wpo = mkwin(display, (XVisualInfo *) NULL, &area, True);
/* Select for VisibilityNotify events on window. */
	XSelectInput(display, w, MASK);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XMapWindow(display, w);
	XSync(display, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none (not viewable test)", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.state = VisibilityPartiallyObscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event (not viewable test)");
		FAIL;
	}
	else
		CHECK;
/* Verify that no other events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d (not viewable test)", count+1, 1);
		FAIL;
	}
	else
		CHECK;
/* Unobscure window. */
	XUnmapWindow(display, wpo);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XMapWindow(display, wpo);
	XSync(display, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none (viewable & completely unobscured test)", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.state = VisibilityPartiallyObscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event (viewable & completely unobscured test)");
		FAIL;
	}
	else
		CHECK;
/* Verify that no other events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d (viewable & completely unobscured test)", count+1, 1);
		FAIL;
	}
	else
		CHECK;
	
	CHECKPASS(6);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion VisibilityNotify-9.(A)
 * When a VisibilityNotify event is delivered and window
 * changes state from viewable and completely unobscured,
 * viewable and partially obscured, or not viewable to viewable
 * and fully obscured, then state is set to
 * VisibilityFullyObscured.
 */
static void
t009()
{
Display	*display = Dsp;
Window	w, wfo, wpo;
struct area	area;
XEvent	event_return;
XVisibilityEvent	good;
int	count;
int 	pass = 0, fail = 0;

	tpstartup();

	/*
	 * test ordering:
	 *
	 *	not viewable ->
	 *		-> viewable and fully obscured
	 *	viewable and completely unobscured
	 *		-> viewable and fully obscured
	 *	viewable and partially obscured
	 *		-> viewable and fully obscured
	 */
/* Create window. */
	area.x = 1;
	area.y = 1;
	area.width = W_STDWIDTH;
	area.height = W_STDHEIGHT;
	w = mkwin(display, (XVisualInfo *) NULL, &area, False);
/* Create completely obscuring window. */
	wfo = mkwin(display, (XVisualInfo *) NULL, &area, True);
/* Create partially obscuring window. */
	area.x += W_STDWIDTH/2;
	area.y += W_STDHEIGHT/2;
	wpo = mkwin(display, (XVisualInfo *) NULL, &area, False);
/* Select for VisibilityNotify events on window. */
	XSelectInput(display, w, MASK);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XMapWindow(display, w);
	XSync(display, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none (not viewable test)", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.state = VisibilityFullyObscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event (not viewable test)");
		FAIL;
	}
	else
		CHECK;
/* Verify that no other events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d (not viewable test)", count+1, 1);
		FAIL;
	}
	else
		CHECK;
/* Unobscure window. */
	XUnmapWindow(display, wfo);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XMapWindow(display, wfo);
	XSync(display, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none (viewable and completely unobscured test)", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.state = VisibilityFullyObscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event (viewable and completely unobscured test)");
		FAIL;
	}
	else
		CHECK;
/* Verify that no other events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d (viewable and completely unobscured test)", count+1, 1);
		FAIL;
	}
	else
		CHECK;
/* Partially obscure window. */
	XUnmapWindow(display, wfo);
	XMapWindow(display, wpo);
/* Generate VisibilityNotify event. */
	XSync(display, True);
	XMapWindow(display, wfo);
	XSync(display, False);
/* Verify that a VisibilityNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none (viewable and partially obscured test)", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xvisibility;
	good.type = EVENT;
	good.send_event = False;
	good.display = display;
	good.window = w;
	good.state = VisibilityFullyObscured;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event (viewable and partially obscured test)");
		FAIL;
	}
	else
		CHECK;
/* Verify that no other events were delivered. */
	count = XPending(display);
	if (count > 0) {
		report("Received %d events, expected %d (viewable and partially obscured test)", count+1, 1);
		FAIL;
	}
	else
		CHECK;
	
	CHECKPASS(9);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
