/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: allcszhnts.m,v 1.6 92/06/11 17:27:14 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XSizeHints *
 *   XAllocSizeHints()
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XAllocSizeHints	1
char    *TestName = "XAllocSizeHints";



int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XAllocSizeHints-1.(A)
 * A call to XAllocSizeHints allocates and returns a pointer
 * to a XSizeHints structure, which can be freed with XFree,
 * in which each component is set to zero.
 */
static void
t001()
{
XSizeHints	*shints;
int 	pass = 0, fail = 0;

	tpstartup();

	shints = XAllocSizeHints();

	if (shints == (XSizeHints *)NULL) {
		delete("XAllocSizeHints returned NULL.");
		return;
	} else
		CHECK;

	if(shints->flags != 0L) {
		report("The flags component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->x != 0) {
		report("The x component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->y != 0) {
		report("The y component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->width != 0) {
		report("The width component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->height != 0) {
		report("The height component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->min_width != 0) {
		report("The min_width component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->min_height != 0) {
		report("The min_height component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->max_width != 0) {
		report("The max_width component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->max_height != 0) {
		report("The max_height component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->width_inc != 0) {
		report("The width_inc component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->height_inc != 0) {
		report("The height_inc component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if((shints->min_aspect.x != 0) || (shints->min_aspect.y != 0)){
		report("The min_aspect components of the XSizeHints structure were not zero.");
		FAIL;
	} else
		CHECK;

	if((shints->max_aspect.x != 0) || (shints->max_aspect.y != 0)){
		report("The max_aspect components of the XSizeHints structure were not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->base_width != 0) {
		report("The base_width component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->base_height != 0) {
		report("The base_height component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;

	if(shints->win_gravity != 0) {
		report("The win_gravity component of the XSizeHints structure was not zero.");
		FAIL;
	} else
		CHECK;


	XFree((char *) shints);
	CHECKPASS(17);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocSizeHints-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When insufficient memory is available, then a call to
 * XAllocSizeHints returns NULL.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
