/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtwmclrmpw.m,v 1.8 92/06/11 17:28:43 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XGetWMColormapWindows(display, w, colormap_windows_return, count_return)
 *   Display	*display;
 *   Window	w;
 *   Window	**colormap_windows_return;
 *   int	*count_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetWMColormapWindows	1
char    *TestName = "XGetWMColormapWindows";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XGetWMColormapWindows function
 */
static Display	*display;
static Window	w;
static Window	**colormap_windows_return;
static int	*count_return;


static int 	StatusReturn;

#include	"Xatom.h"
Window	*window;
int	count;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(display);
	colormap_windows_return = &window;
	count_return = &count;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetWMColormapWindows-1.(A)
 * When the WM_COLORMAP_WINDOWS property is set on the window
 * w, the property is of type WINDOW, and is of format 32,
 * then a call to XGetWMColormapWindows returns the list of
 * window identifiers stored in the property, which can be
 * freed with XFree, in the colormap_windows_return argument
 * and the number of windows in the count_return argument and
 * returns non-zero.
 */
static void
t001()
{
Status		status;
Window		rwin;
XVisualInfo	*vp;
int		nwins = 7;
Window		pwins[7];
Window		*rwins = (Window *) NULL;
Window		*wp;
int		rcnt = 0;
int		i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(i=0; i<nwins; i++)
		pwins[i] = (Window) i;

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);
	
	XSetWMColormapWindows(display, w, pwins, nwins);

	colormap_windows_return = &rwins;
	count_return = &rcnt;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetWMColormapWindows(display, w, colormap_windows_return, count_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == False) {
		report("%s() returned False.", TestName);
		FAIL;
	} else
		CHECK;

	if(rcnt != nwins) {
		report("The number of elements set for the WM_COLORMAP_WINDOWS property was %d instead of %d", rcnt, nwins);
		FAIL;
	} else
		CHECK;

	if(rwins == (Window *) NULL) {
		report("The WM_COLORMAP_WINDOWS property had no associated value.");
		FAIL;
	} else {

		CHECK;
		for(i = 0, wp = rwins; i < nwins; i++, wp++)
			if( *wp != pwins[i]) {
				report("Element %d of the WM_COLORMAP_WINDOWS value was %lu instead of %lu", i+1, (long) *wp, (long) pwins[i]);
				FAIL;
			} else
				CHECK;

		XFree((char*)rwins);
	}

	CHECKPASS(nwins + 3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetWMColormapWindows-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the atom name (lqWM_COLORMAP_WINDOWS(rq cannot be
 * interned, then a call to XGetWMColormapWindows does not set
 * the colormap_windows_return or count_return arguments and
 * returns zero.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XGetWMColormapWindows-3.(A)
 * When the WM_COLORMAP_WINDOWS property is not set on the
 * window w, or is not of type WINDOW, or is not of format 32,
 * then a call to XGetWMColormapWindows does not set the
 * colormap_windows_return or count_return arguments and
 * returns zero.
 */
static void
t003()
{
Status		status;
Atom		xa_wm_colormap_windows;
Atom		*rat;
int		rcnt;
XVisualInfo	*vp;
Window		wdw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);

	colormap_windows_return = &rat;
	count_return = &rcnt;
	w = makewin(display, vp);
	
	if(XSetWMColormapWindows(display, w, &wdw, 1) == False) {
		delete("XSetWMColormapWindows() returned False.");
		return;
	} else
		CHECK;

	if( (xa_wm_colormap_windows = XInternAtom(display, "WM_COLORMAP_WINDOWS", True)) == None) {
		delete("The \"WM_COLORMAP_WINDOWS\" string was not interned.");
		return;
	} else
		CHECK;

	w = makewin(display, vp);

/* property unset */

	rat = (Atom *) -1;
	rcnt = -1;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetWMColormapWindows(display, w, colormap_windows_return, count_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when the WM_COLORMAP_WINDOWS property was not set.", TestName);
		FAIL;
	} else
		CHECK;

	if( rat != (Atom *) -1) {
		report("Atom list pointer variable was updated when the WM_COLORMAP_WINDOWS property was not set.");
		FAIL;
	} else
		CHECK;

	if( rcnt != -1) {
		report("Atom count variable was updated when the WM_COLORMAP_WINDOWS property was not set.");
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);

/* format 8 */
 	XChangeProperty(display, w, xa_wm_colormap_windows, XA_WINDOW, 8, PropModeReplace, (unsigned char *) &wdw, 1);

	rat = (Atom *) -1;
	rcnt = -1;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetWMColormapWindows(display, w, colormap_windows_return, count_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when the WM_COLORMAP_WINDOWS property had format 8.", TestName);
		FAIL;
	} else
		CHECK;

	if( rat != (Atom *) -1) {
		report("Atom list pointer variable was updated when the WM_COLORMAP_WINDOWS property had format 8.");
		FAIL;
	} else
		CHECK;

	if( rcnt != -1) {
		report("Atom count variable was updated when the WM_COLORMAP_WINDOWS property had format 8.");
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);

/* type  ATOM */
 	XChangeProperty(display, w, xa_wm_colormap_windows, XA_ATOM, 32, PropModeReplace, (unsigned char *) &wdw, 1);

	rat = (Atom *) -1;
	rcnt = -1;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetWMColormapWindows(display, w, colormap_windows_return, count_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when the WM_COLORMAP_WINDOWS property had type ATOM.", TestName);
		FAIL;
	} else
		CHECK;

	if( rat != (Atom *) -1) {
		report("Atom list pointer variable was updated when the WM_COLORMAP_WINDOWS property had type ATOM.");
		FAIL;
	} else
		CHECK;

	if( rcnt != -1) {
		report("Atom count variable was updated when the WM_COLORMAP_WINDOWS property had type ATOM.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(11);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XGetWMColormapWindows-4.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XGetWMColormapWindows(display, w, colormap_windows_return, count_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
