/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtwmhnts.m,v 1.7 92/06/11 17:28:51 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XWMHints *
 *   XGetWMHints(display, w)
 *   Display	*display;
 *   Window	w;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetWMHints	1
char    *TestName = "XGetWMHints";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XGetWMHints function
 */
static Display	*display;
static Window	w;


#include	"Xatom.h"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetWMHints-1.(A)
 * When the WM_HINTS property has been set for the window w,
 * and is format 32, length (>= 9 elements, and of type
 * WM_HINTS, then a call to XGetWMHints returns a pointer to a
 * XWMHints structure which contains the window manager hints
 * for the window and can be freed with XFree.
 */
static void
t001()
{
Window		win;
XVisualInfo	*vp;
XWMHints	hints;
XWMHints	*hints_ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);

	hints.flags = AllHints;
	hints.input = True;
	hints.initial_state = IconicState;
	hints.icon_pixmap =  1L;
	hints.icon_window = 1L;
	hints.icon_x = 13;
	hints.icon_y = 7;
	hints.icon_mask = 1L;
	hints.window_group = 1L;

	XSetWMHints(display, win, &hints);

	w = win;
	startcall(display);
	if (isdeleted())
		return;
	hints_ret = XGetWMHints(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(hints_ret == (XWMHints *) 0) {
		report("XGetWMHints() returned NULL.");
		FAIL;
		return; /* failure invalidates rest of test, and no freeing */
	} else
		CHECK;

	if(hints_ret->flags != AllHints) {
		report("The flags component was %lu instead of AllHints.", hints_ret->flags);
		FAIL;
	} else
		CHECK;

	if(hints_ret->input != True) {
		report("The hints_ret component of the XWMHints structure was %d instead of True.", (Bool) hints_ret->input);
		FAIL;
	} else
		CHECK;

	if(hints_ret->initial_state != IconicState) {
		report("The initial_state component of the XWMHints structure was %d instead of IconicState.",
			hints_ret->initial_state);
		FAIL;
	} else
		CHECK;

	if(hints_ret->icon_pixmap !=  1L) {
		report("The icon_pixmap component of the XWMHints structure was %lu instead of 1.", hints_ret->icon_pixmap);
		FAIL;
	} else
		CHECK;

	if(hints_ret->icon_window != 1L) {
		report("The icon_window component of the XWMHints structure was %lu instead of 1.", hints_ret->icon_window);
		FAIL;
	} else
		CHECK;

	if(hints_ret->icon_x != 13) {
		report("The icon_x component of the XWMHints structure was %d instead of 13.", hints_ret->icon_x);
		FAIL;
	} else
		CHECK;

	if(hints_ret->icon_y != 7) {
		report("The icon_y component of the XWMHints structure was %d instead of 7.", hints_ret->icon_y);
		FAIL;
	} else
		CHECK;

	if(hints_ret->icon_mask != 1L) {
		report("The icon_mask component of the XWMHints structure was %lu instead of 1.", hints_ret->icon_mask);
		FAIL;
	} else
		CHECK;

	if(hints_ret->window_group != 1L) {
		report("The window_group component of the XWMHints structure was %lu instead of 1.", hints_ret->window_group);
		FAIL;
	} else
		CHECK;

	XFree((char*)hints_ret);

	CHECKPASS(10);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetWMHints-2.(A)
 * When the WM_HINTS property has not been set for the window
 * w, or is format other than 32, length < 9 elements, or of
 * type other than WM_HINTS, then a call to XGetWMHints
 * returns NULL.
 */
static void
t002()
{
XVisualInfo	*vp;
XWMHints	hints;
XWMHints	*hints_ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);

	w = makewin(display, vp);
/* Property unset */

	startcall(display);
	if (isdeleted())
		return;
	hints_ret = XGetWMHints(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(hints_ret != (XWMHints *) NULL){	
		report("XGetWMHints() did not return NULL with the WM_HINTS property unset.");
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);
/* format 16 */
	XChangeProperty(display, w, XA_WM_HINTS, XA_WM_HINTS, 16, PropModeReplace, (unsigned char *) &hints, 9);

	startcall(display);
	if (isdeleted())
		return;
	hints_ret = XGetWMHints(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(hints_ret != (XWMHints *) NULL){	
		report("XGetWMHints() did not return NULL with the WM_HINTS property format set to 16.");
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);
/* type ATOM */
	XChangeProperty(display, w, XA_WM_HINTS, XA_ATOM, 32, PropModeReplace, (unsigned char *) &hints, 9);

	startcall(display);
	if (isdeleted())
		return;
	hints_ret = XGetWMHints(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(hints_ret != (XWMHints *) NULL){	
		report("XGetWMHints() did not return NULL with the WM_HINTS type set to ATOM.");
		FAIL;
	} else
		CHECK;


	w = makewin(display, vp);
/* size 1 */
	XChangeProperty(display, w, XA_WM_HINTS, XA_WM_HINTS, 32, PropModeReplace, (unsigned char *) &hints, 1);

	startcall(display);
	if (isdeleted())
		return;
	hints_ret = XGetWMHints(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(hints_ret != (XWMHints *) NULL){	
		report("XGetWMHints() did not return NULL with the WM_HINTS size set to 1.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XGetWMHints-3.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XGetWMHints(display, w);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
