/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: strgbclrmp.m,v 1.6 92/06/11 17:30:22 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetRGBColormaps(display, w, std_colormp, count, property)
 *   Display			*display;
 *   Window			w;
 *   XStandardColormap	*std_colormp;
 *   int			count;
 *   Atom			property;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetRGBColormaps	1
char    *TestName = "XSetRGBColormaps";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w
#define A_ATOM property


/*
 * Arguments to the XSetRGBColormaps function
 */
static Display			*display;
static Window			w;
static XStandardColormap	*std_colormp;
static int			count;
static Atom			property;


#include		"Xatom.h"
static XStandardColormap	sclrmp;
static XStandardColormap	scmp1 = { (Colormap)  1,  2L,  3L,  4L,  5L,  6L,  7L,  8L, (VisualID) 9L, (XID) 10L };

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	std_colormp = &sclrmp;
	count = 1;
	property = XA_RGB_DEFAULT_MAP;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetRGBColormaps-1.(A)
 * A call to XSetRGBColormaps sets the property, specified by
 * the property argument, for the window w to be of type
 * RGB_COLOR_MAP, format 32, and to have value set to the
 * count XStandardColormap structures specified by the
 * std_colormap argument.
 */
static void
t001()
{
Atom			actual_type;
int			actual_format;
unsigned long		leftover;
unsigned long		nitems;
XVisualInfo		*vp;
XStandardColormap	*rcmap = (XStandardColormap *) NULL;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);

	std_colormp = &scmp1;
	startcall(display);
	if (isdeleted())
		return;
	XSetRGBColormaps(display, w, std_colormp, count, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( XGetWindowProperty(display, w, property, 0L, 40L, False, XA_RGB_COLOR_MAP, &actual_type,
				&actual_format, &nitems, &leftover, (unsigned char **) &rcmap) != Success ) {
		delete("XGetWindowProperty() did not return Success.");
		return;
	} else
		CHECK;
	
	if(leftover != 0) {
		report("The leftover elements numbered %lu instead of 0", leftover);
		FAIL;
	} else
		CHECK;

	if(actual_format != 32) {
		report("The format of the RGB_DEFAULT_MAP property was %lu instead of 32", actual_format);
		FAIL;
	} else
		CHECK;

	if(actual_type != XA_RGB_COLOR_MAP) {
		report("The type of the RGB_DEFAULT_MAP property was %lu instead of RGB_DEFAULT_MAP (%lu)", actual_type,
				 (long) XA_RGB_DEFAULT_MAP);
		FAIL;
	} else
		CHECK;

	if( rcmap == (XStandardColormap *) NULL) {
		report("No value was obtained for the RGB_DEFAULT_MAP property.");
		FAIL;
	} else {

		CHECK;
		
		if(nitems != 10) {
			report("The RGB_DEFAULT_MAP property comprised %d elements instead of %d", nitems, 10);
			FAIL;
		} else
			if(actual_format == 32) {
				CHECK;
  				if(rcmap->colormap != scmp1.colormap) {
					report("The colormap component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->red_max != scmp1.red_max)  {
					report("The red_max component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->red_mult != scmp1.red_mult) {
					report("The red_mult component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->green_max != scmp1.green_max) {
					report("The green_max component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->green_mult != scmp1.green_mult) {
					report("The green_mult component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->blue_max != scmp1.blue_max) {
					report("The blue_max component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->blue_mult != scmp1.blue_mult) {
					report("The blue_mult component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->base_pixel != scmp1.base_pixel) {
					report("The base_pixel component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->visualid != scmp1.visualid) {
					report("The visualid component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
				if(rcmap->killid != scmp1.killid ) {
					report("The killid component of the XStandardColormap structure %d was incorrect.");
					FAIL;
				} else
					CHECK;
			
			}
			XFree((char *) rcmap);
		}

	CHECKPASS(16);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetRGBColormaps-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EAto.mc */
/*
 * Assertion XSetRGBColormaps-3.(A)
 * When an atom argument does not name a valid Atom, then a
 * BadAtom error occurs.
 */
/* End of included file error/EAto.mc */

/* Including from file error/EAto.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_ATOM = ~0L;
	startcall(display);
	if (isdeleted())
		return;
	XSetRGBColormaps(display, w, std_colormp, count, property);
	endcall(display);
	if (geterr() != BadAtom) {
		report("Got %s, Expecting BadAtom", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadAtom)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAto.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XSetRGBColormaps-4.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetRGBColormaps(display, w, std_colormp, count, property);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
