/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: sttrnsntfr.m,v 1.6 92/06/11 17:30:45 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XSetTransientForHint(display, w, prop_window)
 *   Display	*display;
 *   Window	w;
 *   Window	prop_window;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetTransientForHint	1
char    *TestName = "XSetTransientForHint";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w
#define A_WINDOW2 prop_window
#define A_DRAWABLE2 prop_window


/*
 * Arguments to the XSetTransientForHint function
 */
static Display	*display;
static Window	w;
static Window	prop_window;


#include	"Xatom.h"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	prop_window = DRW(Dsp);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetTransientForHint-1.(A)
 * A call to XSetTransientForHint sets the WM_TRANSIENT_FOR
 * property for the window w to be of type WINDOW, format 32
 * and to have value set to the window specified by the
 * prop_window argument.
 */
static void
t001()
{
Window		win, pwin;
XVisualInfo	*vp;
Window		*retwin = (Window *) NULL;
unsigned long	leftover, nitems, len;
int		actual_format;
Atom		actual_type;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);
	pwin = makewin(display, vp);	

	w = win;
	prop_window = pwin;
	startcall(display);
	if (isdeleted())
		return;
	XSetTransientForHint(display, w, prop_window);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (XGetWindowProperty(display, win, XA_WM_TRANSIENT_FOR, 0L, 1L, False,
	    AnyPropertyType, &actual_type, &actual_format, &nitems, &leftover, (unsigned char **) &retwin) != Success) {
		delete("XGetWindowProperty() did not return Success.");
		return;
	} else
		CHECK;

	if(leftover != 0) {
		report("The leftover elements numbered %lu instead of 0", leftover);
		FAIL;
	} else
		CHECK;

	if(actual_format != 32) {
		report("The format of the WM_TRANSIENT_FOR property was %lu instead of 32", actual_format);
		FAIL;
	} else
		CHECK;

	if(actual_type != XA_WINDOW) {
		report("The type of the WM_TRANSIENT_FOR property was %lu instead of WINDOW (%lu)", actual_type, (long) XA_WINDOW);
		FAIL;
	} else
		CHECK;
	
	if(nitems != 1) {
		report("The WM_TRANSIENT_FOR property comprised %ul elements instead of 1", nitems);
		FAIL;
	} else
		CHECK;

	if( retwin == (Window *) NULL) {
		report("No value was set for the WM_TRANSIENT_FOR property.");
		FAIL;

	} else {

		if(*retwin != pwin) {
			report("The WM_TRANSIENT_FOR property had value %lx instead of %lx", (long) *retwin, (long) pwin);
			FAIL;
		} else
			CHECK;

		XFree((char*)retwin);
	}

	CHECKPASS(6);



	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetTransientForHint-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/*
 * Assertion XSetTransientForHint-3.(A)
 * When the window specified by the w argument does not name a
 * valid window, then a BadWindow error occurs.
 */
static void
t003()
{
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	prop_window = makewin(display, vp);

	w = badwin(display);
	
        startcall(display);
        if (isdeleted())
        	return;
        XSetTransientForHint(display, w, prop_window);
        endcall(display);
        if (geterr() != BadWindow) {
        	report("Got %s, Expecting BadWindow", errorname(geterr()));
        	FAIL;
        }

        if (geterr() == BadWindow)
                PASS;
        else
                FAIL;



	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
