/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: wthdrwwdw.m,v 1.12 92/06/11 17:32:38 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XWithdrawWindow(display, w, screen_number)
 *   Display		*display;
 *   Window		w;
 *   int		screen_number;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XWithdrawWindow	1
char    *TestName = "XWithdrawWindow";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XWithdrawWindow function
 */
static Display		*display;
static Window		w;
static int		screen_number;


static int 	StatusReturn;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	screen_number = DefaultScreen(Dsp);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XWithdrawWindow-1.(A)
 * A call to XWithdrawWindow unmaps the window specified by
 * the w argument and sends, to the root window of the screen
 * specified by the screen_number argument, a synthetic
 * UnmapNotify event with a window of w, an event equal to the
 * receiving root window and a from_configure element of False
 * using an event mask of SubstructureRedirectMask|
 * SubstructureNotifyMask and returns non-zero.
 */
static void
t001()
{
int			nevents;
XVisualInfo		*vp;
XWindowAttributes	atts;
XEvent			ev;
XEvent			rev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	resetvinf(VI_WIN);
	nextvinf(&vp);

	ev.type = UnmapNotify;
	ev.xany.display = display;
	ev.xunmap.type = UnmapNotify;
	ev.xunmap.send_event = False;
	ev.xunmap.display = display;
	ev.xunmap.event = DRW(display);
	ev.xunmap.from_configure = False;

	CATCH_ERROR(display);
	XSelectInput(display, DRW(display), SubstructureNotifyMask);
	RESTORE_ERROR(display);

	if(GET_ERROR(display) != Success) {
		delete("XSelectInput() failed with an event mask of 0x%lx.", SubstructureNotifyMask);
		return;
	} else
		CHECK;

	w = makewin(display, vp);
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XWithdrawWindow(display, w, screen_number);
	endcall(display);
	if (StatusReturn == 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	
	if(XGetWindowAttributes(display, w, &atts) == False){
		delete("XGetWindowAttributes Failed on window id %lx.", (long) w);
		return;
	} else
		CHECK;
	
	if(atts.map_state != IsUnmapped) {
		report("Window ID %lx was not unmapped", (long) w);
		FAIL;
	} else
		CHECK;
	
	ev.xunmap.window = w;
	ev.xunmap.send_event = False;
	rev.type = -1;
	if( (nevents = getevent(display, &rev)) == 0 ) {
		report("No event was generated.");
		FAIL;
	} else {
		CHECK;
		if( checkevent(&ev, &rev) != 0 )
			FAIL;
		else
			CHECK;
	}
	
	ev.xunmap.send_event = True;
	rev.type = -1;

	if( (nevents = getevent(display, &rev)) == 0 ) {
		report("No event was generated.");
		FAIL;
	} else {
		CHECK;
		if( checkevent(&ev, &rev) != 0 )
			FAIL;
		else
			CHECK;
	}

	CATCH_ERROR(display);
	XSelectInput(display, DRW(display), SubstructureRedirectMask);
	RESTORE_ERROR(display);

	if(GET_ERROR(display) != Success) {
		delete("XSelectInput() failed with an event mask of 0x%lx.",SubstructureRedirectMask);
		return;
	} else
		CHECK;

	w = makewin(display, vp);
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XWithdrawWindow(display, w, screen_number);
	endcall(display);
	if (StatusReturn == 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	
	if(XGetWindowAttributes(display, w, &atts) == False){
		delete("XGetWindowAttributes Failed on window id %lx.", (long) w);
		return;
	} else
		CHECK;
	
	if(atts.map_state != IsUnmapped) {
		report("Window ID %lx was not unmapped", (long) w);
		FAIL;
	} else
		CHECK;
	
	ev.xunmap.window = w;
	ev.xunmap.send_event = False;
	rev.type = -1;
	if( (nevents = getevent(display, &rev)) == 0 ) {
		report("No event was generated.");
		FAIL;
	} else {
		CHECK;
		if( checkevent(&ev, &rev) != 0 )
			FAIL;
		else
			CHECK;
	}
	
	CHECKPASS(12);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWithdrawWindow-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the UnmapNotify event is not successfully sent, then
 * XWithdrawWindow returns zero.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EWin.mc */
/*
 * Assertion XWithdrawWindow-3.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XWithdrawWindow(display, w, screen_number);
	endcall(display);
	if (StatusReturn == 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
