/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: crtbtmpfrm.m,v 1.9 92/06/11 17:33:17 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Pixmap
 *   XCreateBitmapFromData(display, d, data, width, height)
 *   Display *display;
 *   Drawable d;
 *   char *data;
 *   unsigned int width;
 *   unsigned int height;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCreateBitmapFromData	1
char    *TestName = "XCreateBitmapFromData";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE d


/*
 * Arguments to the XCreateBitmapFromData function
 */
static Display *display;
static Drawable d;
static char *data;
static unsigned int width;
static unsigned int height;


/* data for a "checkerboard" 10x20 bitmap. */
static unsigned char cbfd_one[]= {
	0x1f, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0x1f, 0x00,
	0x1f, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0xe0, 0x03, 0xe0, 0x03,
	0xe0, 0x03, 0xe0, 0x03, 0xe0, 0x03, 0xe0, 0x03, 0xe0, 0x03, 0xe0, 0x03,
	0xe0, 0x03, 0xe0, 0x03};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	d = defwin(display);
	data = (char*)cbfd_one;
	width = 10;
	height = 20;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XCreateBitmapFromData-1.(A)
 * A call to XCreateBitmapFromData returns a bitmap containing
 * the data of width width and height height.
 */
static void
t001()
{
Pixmap ret;
Window wt_r;
int it_r;
unsigned int width_ret, height_ret;
unsigned int uit_r;
unsigned int depth_ret;
struct area ar;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call xname to create the bitmap from the data. */
	startcall(display);
	if (isdeleted())
		return;
	ret=XCreateBitmapFromData(display, d, data, width, height);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Use XGetGeometry to check pixmap. */
	if(!XGetGeometry(display, ret, &wt_r, &it_r, &it_r,
		&width_ret, &height_ret, &uit_r, &depth_ret)) {
		delete("XGetGeometry failed.");
		return;
	} else
		CHECK;

#ifdef TESTING
	depth_ret = 2;
	width_ret++;
	height_ret++;
#endif

	if(width_ret!=width || height_ret!=height) {
		FAIL;
		report("%s created a pixmap of unexpected geometry.",
			TestName);
		report("Expected width, height: %d,%d", width, height);
		report("Returned width, height: %d,%d", width_ret, height_ret);
	} else
		CHECK;

	if(depth_ret != 1) {
		FAIL;
		report("%s created a pixmap of unexpected depth.",
			TestName);
		report("Expected depth: 1");
		report("Returned depth: %u", depth_ret);
	} else
		CHECK;

/* Verify the pixmap was as expected. */
	setarea(&ar,0,0,5,10);
	if(!checkarea(display, ret, &ar, W_FG, 0, CHECK_IN)) {
		FAIL;
		report("%s did not return the expected pixmap.", TestName);
		trace("Top right quarter was incorrect");
	} else
		CHECK;

	setarea(&ar,5,0,5,10);
	if(!checkarea(display, ret, &ar, W_BG, 0, CHECK_IN)) {
		FAIL;
		report("%s did not return the expected pixmap.", TestName);
		trace("Top left quarter was incorrect");
	} else
		CHECK;

	setarea(&ar,0,10,5,10);
	if(!checkarea(display, ret, &ar, W_BG, 0, CHECK_IN)) {
		FAIL;
		report("%s did not return the expected pixmap.", TestName);
		trace("Bottom right quarter was incorrect");
	} else
		CHECK;

	setarea(&ar,5,10,5,10);
	if(!checkarea(display, ret, &ar, W_FG, 0, CHECK_IN)) {
		FAIL;
		report("%s did not return the expected pixmap.", TestName);
		trace("Bottom Left quarter was incorrect");
	} else
		CHECK;

	CHECKPASS(7);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateBitmapFromData-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When sufficient temporary storage cannot be allocated, then
 * a call to XCreateBitmapFromData returns None.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EAll.mc */
/*
 * Assertion XCreateBitmapFromData-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/*
 * Assertion XCreateBitmapFromData-4.(A)
 * When a drawable argument does not name a valid drawable,
 * then one or more BadDrawable errors, one or more BadGC
 * errors or both types of error occur.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XCreateBitmapFromData(display, d, data, width, height);
	endcall(display);

	if (geterr() == BadDrawable || geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
