/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: crtpxmpfrm.m,v 1.11 92/06/11 17:33:32 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Pixmap
 *   XCreatePixmapFromBitmapData(display, d, data, width, height, fg, bg, depth)
 *   Display *display;
 *   Drawable d;
 *   char *data;
 *   unsigned int width;
 *   unsigned int height;
 *   unsigned long fg;
 *   unsigned long bg;
 *   unsigned int depth;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCreatePixmapFromBitmapData	1
char    *TestName = "XCreatePixmapFromBitmapData";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE d


/*
 * Arguments to the XCreatePixmapFromBitmapData function
 */
static Display *display;
static Drawable d;
static char *data;
static unsigned int width;
static unsigned int height;
static unsigned long fg;
static unsigned long bg;
static unsigned int depth;


/* Data for a "checkerboard" pixmap. */
static unsigned char cpfbd_one[] = {
	0xff, 0x00, 0xff, 0x00, 0xff, 0x00, 0xff, 0x00,
	0x00, 0xff, 0x00, 0xff, 0x00, 0xff, 0x00, 0xff};
	

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	d = 0;
	data = (char*)cpfbd_one;
	width = 16;
	height = 8;
	fg = W_BG;
	bg = W_FG;
	depth = 1;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	d = ErrdefDrawable;
}

/*
 * Assertion XCreatePixmapFromBitmapData-1.(A)
 * A call to XCreatePixmapFromBitmapData returns a pixmap
 * containing the bitmap-format XPutImage of the data of depth
 * depth, width width, and height height, and with
 * foreground and background pixel values of fg and bg.
 */
static void
t001()
{
XVisualInfo *vp;
Pixmap ret;
Window wr_r;
int it_r;
unsigned int width_ret, height_ret;
unsigned int uit_r;
unsigned int depth_ret;
struct area ar;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a drawable for the pixmap. */
	resetvinf(VI_WIN); nextvinf(&vp);
	d = makewin(display, vp);
	depth = vp->depth;

/* Call xname to create the pixmap from the data. */
	startcall(display);
	if (isdeleted())
		return;
	ret=XCreatePixmapFromBitmapData(display, d, data, width, height, fg, bg, depth);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Call XGetGeometry to check the pixmap. */
	if(!XGetGeometry(display, ret, &wr_r, &it_r, &it_r,
		&width_ret, &height_ret, &uit_r, &depth_ret)) {
		delete("XGetGeometry failed.");
		return;
	} else
		CHECK;

	if(width_ret!=width || height_ret!=height) {
		FAIL;
		report("%s created a pixmap of unexpected geometry",
			TestName);
		report("Expected width, height: %d,%d", width, height);
		report("Returned width, height: %d,%d", width_ret, height_ret);
	} else
		CHECK;

	if(depth_ret != depth) {
		FAIL;
		report("%s created a pixmap of unexpected depth.",
			TestName);
		report("Expected depth: %u", depth);
		report("Returned depth: %u", depth_ret);
	} else
		CHECK;

/* Verify the pixmap was as expected. */
	setarea(&ar,0,0,8,4);
	if (!checkarea(display, ret, &ar, fg, 0, CHECK_IN)) {
		FAIL;
		report("%s did not return the expected pixmap.", TestName);
		trace("Top left quarter was incorrect");
	} else
		CHECK;

	setarea(&ar,8,0,8,4);
	if (!checkarea(display, ret, &ar, bg, 0, CHECK_IN)) {
		FAIL;
		report("%s did not return the expected pixmap.", TestName);
		trace("Top right quarter was incorrect");
	} else
		CHECK;

	setarea(&ar,0,4,8,4);
	if (!checkarea(display, ret, &ar, bg, 0, CHECK_IN)) {
		FAIL;
		report("%s did not return the expected pixmap.", TestName);
		trace("Bottom left quarter was incorrect");
	} else
		CHECK;

	setarea(&ar,8,4,8,4);
	if (!checkarea(display, ret, &ar, fg, 0, CHECK_IN)) {
		FAIL;
		report("%s did not return the expected pixmap.", TestName);
		trace("Bottom right quarter was incorrect");
	} else
		CHECK;

	CHECKPASS(7);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapFromBitmapData-2.(A)
 * When the depth is not supported by the screen of the
 * specified drawable d, then on a call to
 * XCreatePixmapFromBitmapData a BadValue error occurs.
 */
static void
t002()
{
XVisualInfo *vp;
int baddepth;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


/* Calculate a bad depth. */
	baddepth=1;
	resetvinf(VI_PIX);
	for(baddepth=1; nextvinf(&vp); )
		if(baddepth < vp->depth)
			baddepth=vp->depth;
	baddepth *= 2;

/* Call xname with a bad depth */
	depth=baddepth;
	d = defwin(display);
	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmapFromBitmapData(display, d, data, width, height, fg, bg, depth);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

/* Verify that a bad value error occurred. */
	if (geterr() != BadValue) {
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapFromBitmapData-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When sufficient temporary storage cannot be allocated, then
 * a call to XCreatePixmapFromBitmapData returns NULL.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EAll.mc */
/*
 * Assertion XCreatePixmapFromBitmapData-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/*
 * Assertion XCreatePixmapFromBitmapData-5.(A)
 * When a drawable argument does not name a valid drawable,
 * then one or more BadDrawable errors, one or more BadGC
 * errors or both types of error occur.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmapFromBitmapData(display, d, data, width, height, fg, bg, depth);
	endcall(display);

	if (geterr() == BadDrawable || geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
