/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: dltcntxt.m,v 1.5 92/06/11 17:33:48 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XDeleteContext(display, w, context)
 *   Display *display;
 *   Window w;
 *   XContext context;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XDeleteContext	1
char    *TestName = "XDeleteContext";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XDeleteContext function
 */
static Display *display;
static Window w;
static XContext context;


static int 	ValueReturn;

static char *xdc_data = "a context!";


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = defwin(display);
	context = XUniqueContext();
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XDeleteContext-1.(A)
 * A call to XDeleteContext deletes the table entry for
 * previously saved context data for display display, window w
 * and context type context, and returns zero.
 */
static void
t001()
{
int a;
int ret;
caddr_t b;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a test window. */
/* Save some context information using XSaveContext. */
	a = XSaveContext(display, w, context, (caddr_t)xdc_data);
	if (a != 0) {
		delete("Could not save test context.");
		report("Returned error: %s", contexterrorname(a));
		return;
	} else
		CHECK;

/* Delete the context information using xname. */
	startcall(display);
	if (isdeleted())
		return;
	ret = XDeleteContext(display, w, context);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ret != 0) {
		FAIL;
		report("%s returned an error when expected to succeed.",
			TestName);
		report("Returned error: %s", contexterrorname(ret));
	} else
		CHECK;


/* Verify that the context was deleted using XFindContext. */
	a = XFindContext(display, w, context, &b);
	if (a != XCNOENT) {
		FAIL;
		report("%s did not delete the context.", TestName);
		report("Return code: %s", contexterrorname(a));
		if (b == (caddr_t)xdc_data) {
			report("Returned data was the context supplied.");
		} else {
			report("Returned data was not the context supplied.");
			report("Supplied context: %0x", (unsigned int)xdc_data);
			report("Returned context: %0x", (unsigned int)b);
		}
	} else
		CHECK;
		
	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDeleteContext-2.(A)
 * When there is no previously saved context data for display
 * display, window w and context type context, then a call to
 * XDeleteContext returns XCNOENT.
 */
static void
t002()
{
int ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a test window with no context. */

/* Call xname to delete the non-existent context. */
	startcall(display);
	if (isdeleted())
		return;
	ret = XDeleteContext(display, w, context);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XCNOENT was returned. */
	if (ret != XCNOENT) {
		FAIL;
		report("%s did not return XCNOENT when",
			TestName);
		report("deleting a non-existent context.");
		report("Return code: %s", contexterrorname(ret));
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
