/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: emptyrgn.m,v 1.7 92/06/11 17:34:10 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Bool
 *   XEmptyRegion(r)
 *   Region	r;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XEmptyRegion	1
char    *TestName = "XEmptyRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XEmptyRegion function
 */
static Region	r;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	r = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XEmptyRegion-1.(A)
 * When the region r is empty, then a call to XEmptyRegion
 * returns True.
 */
static void
t001()
{
Region			reg;
Bool			res;
Region			R1;
Region			R2;
static XRectangle	rect1 = {0,0, 7,13 };
static XRectangle	rect2 = {8, 14, 12,12};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	reg = makeregion();
	R1 = makeregion();
	R2 = makeregion();

	if(isdeleted()) return;

	r = reg;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XEmptyRegion(r);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != True) {
		report("%s() did not return True", TestName);
		report("for a region obtain by a call to XCreateRegion().");
		FAIL;
	} else
		CHECK;

	XUnionRectWithRegion(&rect1, R1, R1);
	XUnionRectWithRegion(&rect2, R2, R2);
	XIntersectRegion(R1, R2, R1);

	r = R1;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XEmptyRegion(r);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != True) {
		report("%s() did not return True", TestName);
		report("for a region obtained from the union of two disjoint regions.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEmptyRegion-2.(A)
 * When the region r is not empty, then a call to XEmptyRegion
 * returns False.
 */
static void
t002()
{
Bool			res;
Region			R1;
static	XRectangle	rect1 = {0,0, 1,1 };
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = makeregion();	
	if(isdeleted()) return;

	XUnionRectWithRegion(&rect1, R1, R1);
	r = R1;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XEmptyRegion(r);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != False) {
		report("%s() did not return False", TestName);
		report("for a region obtain by a call to XUnionRectWithRegion().");
		FAIL;
	} else
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
