/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ftchbffr.m,v 1.7 92/06/11 17:34:34 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char *
 *   XFetchBuffer(display, nbytes_return, buffer)
 *   Display	*display;
 *   int	*nbytes_return;
 *   int	buffer;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFetchBuffer	1
char    *TestName = "XFetchBuffer";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XFetchBuffer function
 */
static Display	*display;
static int	*nbytes_return;
static int	buffer;


#include	"Xatom.h"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	nbytes_return = 0;
	buffer = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XFetchBuffer-1.(A)
 * When the cut buffer specified by the buffer argument
 * contains data, then a call to XFetchBuffer returns in the
 * nbytes_return argument the number of bytes stored in the
 * buffer and returns a pointer to storage allocated for the
 * data, which can be freed with XFree.
 */
static void
t001()
{
char	*bp;
char	*rbp;
int	nr;
int	len;
int 	i;
int 	j;
struct	bstrct {
	char	*data;
	int	len;
}	bfrs[8], *bptr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(i=0, bptr=bfrs; i<8; i++, bptr++) {

		len = 1+i*123;
		bptr->len = len;

		if((bptr->data = (char*)malloc(len)) == (char *) NULL) {
			delete("malloc() returned NULL.");
			return;
		} else
			CHECK;

		for(j=len, bp=bptr->data; j>0; *bp++ = (j) %  (256 - i), j--)
			;

		XStoreBuffer(display, bptr->data, len, i);
	}

	for(i=0, bptr=bfrs; i<8; i++, bptr++) {
		
		buffer = i;
		nbytes_return = &nr;
		startcall(display);
		if (isdeleted())
			return;
		rbp = XFetchBuffer(display, nbytes_return, buffer);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(rbp == (char *) NULL) {
			report("%s() did not set buffer %d to contain any data.", TestName, i);
			FAIL;
		} else {
			CHECK;

			if(bptr->len != nr) {
				report("%s() set buffer %d to contain %d bytes instead of %d.", TestName, i, nr, bptr->len);
				FAIL;
			} else {
				CHECK;

				if(memcmp(rbp, bptr->data, nr) != 0) {
					report("%s() set buffer %d to contain the wrong data.", TestName, i);
					FAIL;
				} else
					CHECK;
			}

			free(bptr->data);
			XFree(rbp);
		}
	}


	CHECKPASS(8 + 8 * (3));

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFetchBuffer-2.(A)
 * When the cut buffer specified by the buffer argument does
 * not contain any data, then a call to XFetchBuffer sets the
 * nbytes_return argument to zero, and returns NULL.
 */
static void
t002()
{
char	*rbp;
char	*tstr = "XTest Multi buffer string";
int	len = 1 + strlen(tstr);
int	nr;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(i=0; i < 8; i++)
		XStoreBuffer(display, tstr, len, i);

	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER0);
	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER1);
	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER2);
	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER3);
	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER4);
	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER5);
	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER6);
	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER7);

	for(i=0; i < 8; i++) {
		
		buffer = i;
		nbytes_return = &nr;
		startcall(display);
		if (isdeleted())
			return;
		rbp = XFetchBuffer(display, nbytes_return, buffer);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(rbp != (char *) NULL) {
			report("%s() did not return NULL for buffer %d.", TestName, i);
			FAIL;
		} else
			CHECK;

		if(nr != 0) {
			report("%s() set nbytes_return to %d instead of 0 for buffer %d.", TestName, nr, i);
			FAIL;
		} else
			CHECK;

	}

	CHECKPASS(8 * 2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
