/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: intrsctrgn.m,v 1.6 92/06/11 17:36:04 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XIntersectRegion(sra, srb, dr_return)
 *   Region	sra;
 *   Region	srb;
 *   Region	dr_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XIntersectRegion	1
char    *TestName = "XIntersectRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XIntersectRegion function
 */
static Region	sra;
static Region	srb;
static Region	dr_return;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	sra = 0;
	srb = 0;
	dr_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XIntersectRegion-1.(A)
 * A call to XIntersectRegion computes the intersection of the
 * regions sra and srb and replaces the value of dr_return with
 * the result.
 */
static void
t001()
{
static	XPoint	noninter1[] = { {-25,-5}, {-15,-10}, {-10,-25}, {-20,-35}, {-30,-30}, {-35,-20}, {-30,-10} };
static	XPoint	inter1[] = { {25,5}, {15,10}, {10,25}, {20,35}, {30,30}, {35,20}, {30,10} };
static	XPoint	inter2[] = { {25,20}, {37,22}, {40,30}, {35,37}, {30,40}, {25,38}, {18,32}, {20,25} };
Region		R1;
Region		R2;
Region		Ri;
Region		Rt;
Region		Ru;
Region		Rx;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = makeregion();
	R2 = makeregion();
	Ri = makeregion();
	Rt = makeregion();
	Ru = makeregion();
	Rx = makeregion();

	if(isdeleted()) return;

	R1 = XPolygonRegion( inter1, NELEM(inter1), WindingRule );
	R2 = XPolygonRegion( inter2, NELEM(inter2), WindingRule );
	XUnionRegion(R1, R2, Ru);

	sra = R1;
	srb = R2;
	dr_return = Ri;	
	startcall(Dsp);
	if (isdeleted())
		return;
	XIntersectRegion(sra, srb, dr_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( XEmptyRegion(Ri) == True ) {
		report("%s() returned the empty region for two intersecting regions.", TestName);
		FAIL;
	} else
		CHECK;

	XXorRegion(R1, R2, Rx);
	XUnionRegion(Rx, Ri, Rt);
	
	if(XEqualRegion(Ru, Rt) != True) {
		report("%s() returned a region not completely contained in a source region.", TestName);
		FAIL;
	} else
		CHECK;

	R1 = makeregion();
	R2 = makeregion();
	Ri = makeregion();

	sra = R1;
	srb = R2;
	dr_return = Ri;	
	startcall(Dsp);
	if (isdeleted())
		return;
	XIntersectRegion(sra, srb, dr_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( XEmptyRegion(Ri) != True ) {
		report("%s() returned a non-empty region for two non-intersecting intersecting regions.", TestName);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
