/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: kysymtkycd.m,v 1.4 92/06/11 17:37:06 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   KeyCode
 *   XKeysymToKeycode(display, keysym)
 *   Display	*display;
 *   KeySym	keysym;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XKeysymToKeycode	1
char    *TestName = "XKeysymToKeycode";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XKeysymToKeycode function
 */
static Display	*display;
static KeySym	keysym;


#define XK_LATIN1
#include	"keysymdef.h"
#undef XK_LATIN1

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	keysym = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XKeysymToKeycode-1.(A)
 * A call to XKeysymToKeycode returns the KeyCode defined for
 * the KeySym specified by the keysym argument.
 */
static void
t001()
{
KeyCode	res_lc, res_uc;
KeySym	ks;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	keysym = XK_A;
	startcall(display);
	if (isdeleted())
		return;
	res_uc = XKeysymToKeycode(display, keysym);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	keysym = XK_a;
	startcall(display);
	if (isdeleted())
		return;
	res_lc = XKeysymToKeycode(display, keysym);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( res_lc != res_uc ) {
		report("%s() mapped KeySyms XK_A and XK_a to KeyCodes %lu and %lu instead of to the same KeyCode.", TestName, (long) res_uc, (long) res_lc);
		FAIL;
	} else
		CHECK;

	ks = XKeycodeToKeysym(display, res_lc, 0);

	if(ks != XK_a) {
		report("%s() returned KeyCode %lu which did not map to KeySym XK_a.", TestName, (long) res_lc);
		FAIL;
	} else
		CHECK;

	ks = XKeycodeToKeysym(display, res_uc, 1);

	if(ks != XK_A) {
		report("%s() returned KeyCode %lu which did not map to KeySym XK_A.", TestName, (long) res_uc);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XKeysymToKeycode-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the keysym argument is not defined for any KeyCode,
 * then a call to XKeysymToKeycode returns zero.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
