/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: lkpkysym.m,v 1.5 92/06/11 17:37:26 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   KeySym
 *   XLookupKeysym(key_event, index)
 *   XKeyEvent	*key_event;
 *   int		index;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XLookupKeysym	1
char    *TestName = "XLookupKeysym";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XLookupKeysym function
 */
static XKeyEvent	*key_event;
static int		index;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	key_event = 0;
	index = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XLookupKeysym-1.(A)
 * A call to XLookupKeysym returns the KeySym element
 * specified by index of the KeyCode vector specified by the
 * keycode member of the key_event argument.
 */
static void
t001()
{
int		minkc, maxkc;
int		syms_per_kc;
KeyCode		kc;
KeySym		ks, res;
KeySym		*keysyms;
XKeyEvent	ke;
int		i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	XDisplayKeycodes(Dsp, &minkc, &maxkc);
	keysyms = XGetKeyboardMapping(Dsp, minkc, 1, &syms_per_kc);
	XFree((char *) keysyms);
	key_event = &ke;
	ke.display = Dsp;

	for(i=minkc; i<=maxkc; i++)
		for(index = 0; index < syms_per_kc; index++) {
			ks = XKeycodeToKeysym(Dsp, i, index);
			ke.keycode = i;
			startcall(Dsp);
			if (isdeleted())
				return;
			res = XLookupKeysym(key_event, index);
			endcall(Dsp);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			if(res != ks){
				report("%s() returned KeySym %lu instead of %lu for KeyCode %lu with index %d.", TestName, (long) res, (long) ks, (long) i, index);
				FAIL;
			} else
				CHECK;
		}


	CHECKPASS( (1+maxkc-minkc) * syms_per_kc);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XLookupKeysym-2.(A)
 * When no KeySym is defined for the keycode of the event
 * key_event, then a call to XLookupKeysym returns NoSymbol.
 */
static void
t002()
{
int		minkc, maxkc;
int		syms_per_kc;
KeyCode		kc;
KeySym		res;
KeySym		*keysyms;
XKeyEvent	ke;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	XDisplayKeycodes(Dsp, &minkc, &maxkc);
	keysyms = XGetKeyboardMapping(Dsp, minkc, 1, &syms_per_kc);
	XFree( (char *) keysyms);
	kc = maxkc + 1;

	if(kc == minkc) {
		delete("Cannot establish an invalid KeyCode.");
		return;
	} else
		CHECK;

	key_event = &ke;
	ke.display = Dsp;
	ke.keycode = kc;

	for(index=0; index < syms_per_kc; index++) {

		startcall(Dsp);
		if (isdeleted())
			return;
		res = XLookupKeysym(key_event, index);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(res != NoSymbol) {
			report("%s() returned %lu instead of NoSymbol (%lu) for invalid KeyCode %lu with index %d.", TestName, (long) res, (long) NoSymbol, (long) kc, index);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(1 + syms_per_kc);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
