/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: prmllc.m,v 1.5 92/06/11 17:38:14 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char *
 *   Xpermalloc(size)
 *   unsigned int size;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_Xpermalloc	1
char    *TestName = "Xpermalloc";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the Xpermalloc function
 */
static unsigned int size;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	size = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion Xpermalloc-1.(A)
 * A call to Xpermalloc returns a pointer to a block of memory
 * size bytes large.
 */
static void
t001()
{
int i, badaccess;
char *ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call xname to allocate a block of memory. */
	size = 2048;
	startcall(Dsp);
	if (isdeleted())
		return;
	ret = Xpermalloc(size);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( ret == (char *)NULL ) {
		delete("%s returned a NULL pointer.", TestName);
		report("Expecting a pointer to a block of %u bytes", size);
	} else
		CHECK;

/* Verify that all addresses within the block may be accessed. */
	for(i=0; i<size; i++) {
		*(ret+i)='s';	/* A SIGSEGV indicates t r o u b l e */
		CHECK;
	}

	badaccess=0;
	for(i=0; i<size; i++) {
		if( *(ret+i) != 's' ) {
			badaccess++;
		} else
			CHECK;
	}

	if( badaccess!=0 ) {
		FAIL;
		report("%s did not return a writable block of memory.",
			TestName);
		report("%d addresses within the allocated block could not be written to.", badaccess);
	} else
		CHECK;

	CHECKPASS(2+size*2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion Xpermalloc-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When sufficient temporary storage cannot be allocated, then
 * a call to Xpermalloc returns NULL.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
