/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: prsclr.m,v 1.7 92/06/11 17:38:22 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XParseColor(display, colormap, spec, exact_def_return)
 *   Display		*display;
 *   Colormap	colormap;
 *   char		*spec;
 *   XColor		*exact_def_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XParseColor	1
char    *TestName = "XParseColor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XParseColor function
 */
static Display		*display;
static Colormap	colormap;
static char		*spec;
static XColor		*exact_def_return;


static int 	StatusReturn;


#include	<ctype.h>

XColor		color_ret;

Status
checkcolor( red, green, blue, shift, ret_desired)
unsigned int	red, green, blue;
unsigned int	shift;
XColor		*ret_desired;
{
	ret_desired->red = (unsigned short) red<<shift;
	ret_desired->green = (unsigned short) green<<shift;
	ret_desired->blue = (unsigned short) blue<<shift;
	return((ret_desired->red == color_ret.red) && (ret_desired->green == color_ret.green) && (ret_desired->blue == color_ret.blue));
}

rgb_report(desired)
XColor	*desired;
{
	report("%s() returned red 0x%x green 0x%x blue 0x%x instead of red 0x%x green 0x%x blue 0x%x.",  
		TestName,
		(int) color_ret.red, (int) color_ret.green, (int) color_ret.blue,
		(int) desired->red, (int) desired->green, (int) desired->blue);
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(Dsp, 0);
	spec = config.good_colorname;
	exact_def_return = &color_ret;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XParseColor-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the first character of the spec argument is not #,
 * then a call to XParseColor returns in the exact_def_return
 * argument the rgb values for the colour named by the spec
 * argument on the screen associated with the colormap argument
 * and returns non-zero.
 */
static void
t001()
{
Status result;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	result = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(result == 0) {
		report("%s() returned zero.", TestName);
		FAIL;
	} else
		CHECK;

	CHECKUNTESTED(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XParseColor-2.(A)
 * When the first character of the spec argument is a #, and
 * the remainder of the string comprises 3 hexadecimal digits,
 * then a call to XParseColor sets the exact_def_return
 * argument to have red component equal to the value of the
 * first digit << 12, to have green component equal to the
 * value of the second digit << 12, to have blue component
 * equal to the value of the third digit << 12 and returns
 * non-zero.
 */
static void
t002()
{
Status	ret;
XColor	dcol;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	spec = "#18f";
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret == 0 ) {
		report("%s() with spec \"%s\" returned 0.", TestName, spec);
		FAIL;
	} else
		CHECK;

	if( checkcolor(0x1, 0x8, 0xf, 12, &dcol) == 0) {
		rgb_report(&dcol);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XParseColor-3.(A)
 * When the first character of the spec argument is a #, and
 * the remainder of the string comprises 6 hexadecimal digits,
 * then a call to XParseColor sets the exact_def_return
 * argument to have red component equal to the value of the
 * first two digits << 8, to have green component equal to the
 * value of the second two digits << 8, to have blue component
 * equal to the value of the third two digits << 8 and returns
 * non-zero.
 */
static void
t003()
{
Status	ret;
XColor	dcol;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	spec = "#f1f8ff";
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret == 0 ) {
		report("%s() with spec \"%s\" returned 0.", TestName, spec);
		FAIL;
	} else
		CHECK;

	if( checkcolor(0xf1, 0xf8, 0xff, 8, &dcol) == 0) {
		rgb_report(&dcol);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XParseColor-4.(A)
 * When the first character of the spec argument is a #, and
 * the remainder of the string comprises 9 hexadecimal digits,
 * then a call to XParseColor sets the exact_def_return
 * argument to have red component equal to the value of the
 * first three digits << 4, to have green component equal to
 * the value of the second three digits << 4, to have blue
 * component equal to the value of the third three digits << 4
 * and returns non-zero.
 */
static void
t004()
{
Status	ret;
XColor	dcol;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	spec = "#af1bf8cff";
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret == 0 ) {
		report("%s() with spec \"%s\" returned 0.", TestName, spec);
		FAIL;
	} else
		CHECK;

	if( checkcolor(0xaf1, 0xbf8, 0xcff, 4, &dcol) == 0) {
		rgb_report(&dcol);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XParseColor-5.(A)
 * When the first character of the spec argument is a #, and
 * the remainder of the string comprises 12 hexadecimal digits,
 * then a call to XParseColor sets the exact_def_return
 * argument to have red component equal to the value of the
 * first four digits, to have green component equal to the
 * value of the second four digits, to have blue component
 * equal to the value of the third four digits and returns
 * non-zero.
 */
static void
t005()
{
Status	ret;
XColor	dcol;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	spec = "#faf01bf81cff";
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret == 0 ) {
		report("%s() with spec \"%s\" returned 0.", TestName, spec);
		FAIL;
	} else
		CHECK;

	if( checkcolor(0xfaf0, 0x1bf8, 0x1cff, 0, &dcol) == 0) {
		rgb_report(&dcol);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XParseColor-6.(A)
 * Upper and lower case characters in the spec argument refer
 * to the same colour.
 */
static void
t006()
{
char		*ptr;
char		*str;
Status		result;
XColor		dcol;
unsigned short	red, green, blue;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	result = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(result == 0) {
		report("%s() with spec \"%s\" returned zero.", TestName, config.good_colorname);
		FAIL;
	} else
		CHECK;

	red = color_ret.red;
	green = color_ret.green;
	blue = color_ret.blue;

	str = xt_strdup(config.good_colorname);

	if(str == (char *) NULL) {
		delete("xt_strdup() returned NULL.");
		return;
	} else
		CHECK;

	for(ptr = str; *ptr; ptr++)
		if(isupper(*ptr)) 
			*ptr = tolower(*ptr);
		else
			*ptr = toupper(*ptr);
	
	spec = str;
	startcall(display);
	if (isdeleted())
		return;
	result = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(result == 0) {
		report("%s() with spec \"%s\" returned zero.", TestName, str);
		FAIL;
	} else
		CHECK;

	if((red != color_ret.red) || (green != color_ret.green) || (blue != color_ret.blue)) {
		report("%s() did not map color names %s and %s to the same rgb values.", TestName, config.good_colorname, str);
		FAIL;		
	} else
		CHECK;

	spec = "#feAAc1";
	startcall(display);
	if (isdeleted())
		return;
	result = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( result == 0 ) {
		report("%s() with spec \"%s\" returned 0.", TestName, spec);
		FAIL;
	} else
		CHECK;

	if( checkcolor(0xfe, 0xaa, 0xc1, 8, &dcol) == 0) {
		rgb_report(&dcol);
		FAIL;
	} else
		CHECK;

	spec = "#FEAaC1";
	startcall(display);
	if (isdeleted())
		return;
	result = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( result == 0 ) {
		report("%s() with spec \"%s\" returned 0.", TestName, spec);
		FAIL;
	} else
		CHECK;

	if( checkcolor(0xfe, 0xaa, 0xc1, 8, &dcol) == 0) {
		rgb_report(&dcol);
		FAIL;
	} else
		CHECK;

	CHECKPASS(8);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XParseColor-7.(A)
 * A call to XParseColor sets the flags component of the
 * exact_def_return argument to DoRed|DoGreen|DoBlue.
 */
static void
t007()
{
Status	ret;
char	flags;
XColor	dcol;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	spec = "#000";
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret == 0 ) {
		report("%s() with spec \"%s\" returned 0.", TestName, spec);
		FAIL;
	} else
		CHECK;

	if(color_ret.flags != (flags = DoRed | DoGreen | DoBlue)) {
		report("%s() set the flags component of the retured XColor structure to %d instead of DoRed|DoGreen|DoBlue (%d).",
			TestName, (int) color_ret.flags, (int) flags);

		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XParseColor-8.(A)
 * When the first character of the spec argument is a # and
 * the remainder of the string does not comprise 3, 6, 9 or 12
 * hexadecimal digits, then a call to XParseColor returns zero.
 */
static void
t008()
{
Status ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	spec = "#f0";
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret != 0 ) {
		report("%s() with spec \"%s\" did not return zero.", TestName, spec);
		FAIL;
	} else
		CHECK;

	spec = "#1010";
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret != 0 ) {
		report("%s() with spec \"%s\" did not return zero.", TestName, spec);
		FAIL;
	} else
		CHECK;

	spec = "#ffeeffeeffe"; /* 11 digits */
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret != 0 ) {
		report("%s() with spec \"%s\" did not return zero.", TestName, spec);
		FAIL;
	} else
		CHECK;

	spec = "#0011223344556"; /* 13 digits */
	startcall(display);
	if (isdeleted())
		return;
	ret = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if( ret != 0 ) {
		report("%s() with spec \"%s\" did not return zero.", TestName, spec);
		FAIL;
	} else
		CHECK;

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XParseColor-9.(A)
 * When the first character of the spec argument is not a #
 * and the colour named by the spec argument is not in the
 * colour database, then a call to XParseColor returns zero.
 */
static void
t009()
{
char		*ptr;
char		*str;
Status		result;
XColor		dcol;
unsigned short	red, green, blue;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	spec = config.bad_colorname;
	startcall(display);
	if (isdeleted())
		return;
	result = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(result != 0) {
		report("%s() with spec \"%s\" did not return zero.", TestName, config.bad_colorname);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XParseColor-10.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t010()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XParseColor(display, colormap, spec, exact_def_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
