/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rbndkysym.m,v 1.15 92/06/11 17:38:44 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XRebindKeysym(display, keysym, list, mod_count, string, bytes_string)
 *   Display		*display;
 *   KeySym		keysym;
 *   KeySym		*list;
 *   int		mod_count;
 *   unsigned char	*string;
 *   int		bytes_string;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XRebindKeysym	1
char    *TestName = "XRebindKeysym";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XRebindKeysym function
 */
static Display		*display;
static KeySym		keysym;
static KeySym		*list;
static int		mod_count;
static unsigned char	*string;
static int		bytes_string;



/* Arbitary keysyms for use as modifiers */
#define	MODKS1	0x12
#define	MODKS2	0x45

#define XK_LATIN1
#include    "keysymdef.h"
#undef XK_LATIN1

#define	MAXRES	256	/* Maximum length of bound string */


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	keysym = 0;
	list = 0;
	mod_count = 0;
	string = 0;
	bytes_string = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XRebindKeysym-1.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A call to XRebindKeysym rebinds the meaning of the keysym
 * argument for use in XLookupString to the bytes_string length
 * string and to use the mod_count modifiers specified by the
 * list argument.
 */
static void
t001()
{
XKeyEvent	ev;
char	*astring = "Bound String";
char	*modstring = "String to bind to modified key";
static char	rstring[MAXRES] = "UninitializedBoundString."; 
int		bb;
KeySym	ksr;
KeySym	modlist[5];
KeySym	keylist[6];
XModifierKeymap	*origmap;
XModifierKeymap	*modmap;
KeySym	*origkeymap;
int 	minkc, maxkc;
int 	kpk;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * Save old keyboard map.  Map the keysyms that we want to use
	 * to the first available keycodes.
	 */
	XDisplayKeycodes(display, &minkc, &maxkc);
	origkeymap = XGetKeyboardMapping(display, minkc, maxkc-minkc+1, &kpk);
	keylist[0] = MODKS1;
	keylist[1] = MODKS1;
	keylist[2] = MODKS2;
	keylist[3] = MODKS2;
	keylist[4] = XK_A;
	keylist[5] = XK_A;
	XChangeKeyboardMapping(display, minkc, 2, keylist, 3);

	origmap = XGetModifierMapping(display);

	/*
	 * Set up a modifier mapping to use the modifier keysyms that we are
	 * going to use.
	 */
	modmap = XNewModifiermap(0);
	modmap = XInsertModifiermapEntry(modmap, minkc, ShiftMapIndex);
	modmap = XInsertModifiermapEntry(modmap, minkc+1, Mod1MapIndex);
	if( XSetModifierMapping(display, modmap) != MappingSuccess) {
		report("XSetModifierMapping did not return MappingSuccess");
		UNTESTED;
		/*
		 * Attempt to reset the state of the keyboard.
		 */
		XSetModifierMapping(display, origmap);
		XChangeKeyboardMapping(display, minkc, kpk, origkeymap,
			maxkc-minkc+1);
		return;
	} else
		CHECK;

	/*
	 * Need to refresh the display structure mappings, we just open a new
	 * connection and use that.
	 */
	display = opendisplay();

	/*
	 * Set up a rebinding for A (unshifted)
	 */
	keysym = XK_A;
	list = (KeySym *) NULL;
	mod_count = 0;
	string = (unsigned char *)astring;
	bytes_string = strlen(string)+1;
	startcall(display);
	if (isdeleted())
		return;
	XRebindKeysym(display, keysym, list, mod_count, string, bytes_string);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * Set up a rebinding for Shift Mod1 A.
	 */
	modlist[0] = MODKS1;
	modlist[1] = MODKS2;

	keysym = XK_A;
	list = modlist;
	mod_count = 2;
	string = (unsigned char *)modstring;
	bytes_string = strlen(string)+1;
	startcall(display);
	if (isdeleted())
		return;
	XRebindKeysym(display, keysym, list, mod_count, string, bytes_string);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * Check the unmodified case.
	 */
	ev.display = display;
	ev.keycode = XKeysymToKeycode(display, XK_A);
	debug(1, "code=%d", ev.keycode);
	ev.state = 0;
	XLookupString(&ev, rstring, MAXRES, &ksr, (XComposeStatus *) NULL);
	trace("String returned >%s<", rstring);

	if (strcmp(astring,  rstring) != 0) {
		report("%s() bound XK_A to \"%s\" instead of \"%s\".", TestName, rstring, astring);
		FAIL;
	} else
		CHECK;


	/*
	 * Now check the modified case.
	 */
	ev.display = display;
	ev.state = ShiftMask|Mod1Mask;
	XLookupString(&ev, rstring, MAXRES, &ksr, (XComposeStatus *) NULL);
	trace("String returned >%s<", rstring);

	if (strcmp(modstring,  rstring) != 0) {
		report("%s() bound XK_A to \"%s\" instead of \"%s\".", TestName, rstring, modstring);
		FAIL;
	} else
		CHECK;

	/*
	 * Attempt to reset the state of the keyboard.
	 */
	XSetModifierMapping(display, origmap);
	XChangeKeyboardMapping(display, minkc, kpk, origkeymap, maxkc-minkc+1);

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XRebindKeysym-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When keysym does not exist, then a call to XRebindKeysym
 * rebinds the meaning of the keysym argument.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
