/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rfrshkybrd.m,v 1.9 92/06/11 17:39:09 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XRefreshKeyboardMapping(event_map)
 *   XMappingEvent	*event_map;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XRefreshKeyboardMapping	1
char    *TestName = "XRefreshKeyboardMapping";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XRefreshKeyboardMapping function
 */
static XMappingEvent	*event_map;


#define XK_LATIN1
#include	"keysymdef.h"
#undef XK_LATIN1

static void
cpymap(dest, src)
	XModifierKeymap *dest;
	XModifierKeymap *src;
{
	int i;
	int n;

	for(i=0,n=src->max_keypermod*8; i<n; i++)
		dest->modifiermap[i] = src->modifiermap[i];
	dest->max_keypermod = src->max_keypermod;
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	event_map = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XRefreshKeyboardMapping-1.(A)
 * When the request component of the event_map argument is
 * MappingKeyboard, then a call to XRefreshKeyboardMapping
 * modifies the keymap information based on that specified by
 * the event_map argument.
 */
static void
t001()
{
XEvent		ev;
KeyCode		kc;
KeySym		ks, res, old, new;
XKeyEvent	ke;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	old = XK_a;
	new = XK_b;

	if((kc = XKeysymToKeycode(Dsp, old)) == 0) {
		delete("XKeysymToKeycode() returned 0 for KeySym %lu.", (long) old);
		return;
	} else
		CHECK;

	XChangeKeyboardMapping(Dsp, kc, 1, &new, 1);
	XSync(Dsp, False);
	
	if( getevent(Dsp, &ev) == 0) {
		delete("Did not get an event after an XChangeKeyboardMapping()");
		XChangeKeyboardMapping(Dsp, kc, 1, &old, 1);
		return;
	} else
		CHECK;

	if(ev.type != MappingNotify) {
		delete("Got a %s event instead of a %s.", eventname(ev.type), eventname(MappingNotify));
		XChangeKeyboardMapping(Dsp, kc, 1, &old, 1);
		return;
	} else
		CHECK;

	if(ev.xmapping.request != MappingKeyboard) {
		delete("The request field was %d instead of %d (MappingKeyboard)",
				ev.xmapping.request, MappingKeyboard);
		XChangeKeyboardMapping(Dsp, kc, 1, &old, 1);
		return;
	} else
		CHECK;

	event_map = &ev.xmapping;
	startcall(Dsp);
	if (isdeleted())
		return;
	XRefreshKeyboardMapping(event_map);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if((res = XKeycodeToKeysym(Dsp, kc, 0)) == NoSymbol) {
		delete("XKeycodeToKeysym() returned NoSymbol for KeyCode %lu.", (long) kc);
		XChangeKeyboardMapping(Dsp, kc, 1, &old, 1);
		return;
	} else
		CHECK;

	if( res != new) {
		report("%s() mapped KeyCode %lu to KeySym %lu instead of %lu.", TestName, (long) kc, (long) res, (long) new);
		FAIL;
	} else
		CHECK;

	XChangeKeyboardMapping(Dsp, kc, 1, &old, 1);
	CHECKPASS(6);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XRefreshKeyboardMapping-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the request component of the event_map argument is
 * MappingModifier, then a call to XRefreshKeyboardMapping
 * modifies the keymap information based on that specified by
 * the event_map argument.
 */
static void
t002()
{
int		i;
XEvent		ev;
KeyCode		*kcptr;
KeyCode		*tkcptr;
XModifierKeymap	*savemap = XGetModifierMapping(Dsp);
XModifierKeymap	*mmap;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(savemap == (XModifierKeymap *) NULL) {
		delete("XGetModifierMapping() returned NULL.");
		return;
	} else
		CHECK;

	mmap = XNewModifiermap(savemap->max_keypermod);

	if(mmap == (XModifierKeymap *) NULL) {
		delete("XNewModifiermap() returned NULL.");
		return;
	} else
		CHECK;

	cpymap(mmap, savemap);

	if(XSetModifierMapping(Dsp, mmap) != MappingSuccess) {
		delete("XSetModifierMapping() did not return MappingSuccess.");
		return;
	} else
		CHECK;

	XSync(Dsp, False);
	
	if( getevent(Dsp, &ev) == 0) {
		delete("Did not get an event after an XSetModifierMapping()");
		XSetModifierMapping(Dsp, savemap);
		return;
	} else
		CHECK;

	if(ev.type != MappingNotify) {
		delete("Got a %s event instead of a %s.", eventname(ev.type), eventname(MappingNotify));
		XSetModifierMapping(Dsp, savemap);
		return;
	} else
		CHECK;

	if(ev.xmapping.request != MappingModifier) {
		delete("The request field was %d instead of %d (MappingModifier)",
				ev.xmapping.request, MappingModifier);
		XSetModifierMapping(Dsp, savemap);
		return;
	} else
		CHECK;

	event_map = &ev.xmapping;
	startcall(Dsp);
	if (isdeleted())
		return;
	XRefreshKeyboardMapping(event_map);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSetModifierMapping(Dsp, savemap);

	XFreeModifiermap(mmap);
	XFreeModifiermap(savemap);
	CHECKUNTESTED(6);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XRefreshKeyboardMapping-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the request component of the event_map argument is
 * other than MappingKeyboard or MappingModifier, then a call
 * to XRefreshKeyboardMapping does not modify the keymap
 * information.
 */
static void
t003()
{
int		i;
int		set;
XEvent		ev;
XModifierKeymap	*savemap = XGetModifierMapping(Dsp);
XModifierKeymap	*mmap1;
KeyCode		minkc;
KeyCode		maxkc;
KeyCode		*kcp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(savemap == (XModifierKeymap *) NULL) {
		delete("XGetModifierMapping() returned NULL.");
		return;
	} else
		CHECK;

	mmap1 = XNewModifiermap(savemap->max_keypermod);

	if(mmap1 == (XModifierKeymap *) NULL) {
		delete("XNewModifiermap() returned NULL.");
		return;
	} else
		CHECK;

	cpymap(mmap1, savemap);

	if(XSetModifierMapping(Dsp, mmap1) != MappingSuccess) {
		delete("XSetModifierMapping() did not return MappingSuccess.");
		return;
	} else
		CHECK;

	XSync(Dsp, False);

	if( getevent(Dsp, &ev) == 0) {
		delete("Did not get an event after an XSetModifierMapping()");
		XSetModifierMapping(Dsp, savemap);
		return;
	} else
		CHECK;

	if(ev.type != MappingNotify) {
		delete("Got a %s event instead of a %s.", eventname(ev.type), eventname(MappingNotify));
		XSetModifierMapping(Dsp, savemap);
		return;
	} else
		CHECK;

	ev.xmapping.request = MappingModifier + MappingKeyboard + 1;
	event_map = &ev.xmapping;
	startcall(Dsp);
	if (isdeleted())
		return;
	XRefreshKeyboardMapping(event_map);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSetModifierMapping(Dsp, savemap);

	XFreeModifiermap(mmap1);
	XFreeModifiermap(savemap);
	CHECKUNTESTED(5);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
