/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rmdstrydtb.m,v 1.4 92/06/11 17:39:17 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XrmDestroyDatabase(database)
 *   XrmDatabase database;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XrmDestroyDatabase	1
char    *TestName = "XrmDestroyDatabase";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XrmDestroyDatabase function
 */
static XrmDatabase database;



static XrmDatabase
xrm_create_database(data)
char *data;
{
	/* Create a new database for testing with */
	return(XrmGetStringDatabase(data));
}

static void
xrm_fill_value(value, data)
XrmValue *value;
char *data;
{
	value->addr = (caddr_t)data;
	value->size = (unsigned int)(strlen(data)+1);
}

static int
xrm_check_entry(dbase, fullspec, fullclass, type, val)
XrmDatabase dbase;
char *fullspec;
char *fullclass;
char *type;
char *val;
{
	int ret_val;
	char	*type_ret;
	XrmValue	value_ret;

	ret_val=0;
	type_ret=(char *)NULL;
	value_ret.size=0;
	value_ret.addr=(caddr_t)NULL;

	if(XrmGetResource(dbase, fullspec, fullclass, &type_ret, &value_ret)
		==False) {
		report("XrmGetResource failed to find database entry");
		report("Specifier was: %s", fullspec);
		ret_val++;
	} else {
		if (type_ret==NULL || strcmp(type_ret, type)) {
			report("XrmGetResource returned unexpected type information.");
			report("Specifier was: %s", fullspec);
			report("Expected type: '%s'", type);
			report("Returned type: '%s'",
				(type_ret==NULL)?"<NULL POINTER>":type_ret);
			ret_val++;
		}

		if (( value_ret.addr==(caddr_t)NULL)
			|| (strncmp((char *)value_ret.addr, val, strlen(val)))) {
			report("XrmGetResource returned unexpected value information.");
			report("Specifier was: %s", fullspec);
			report("Expected value: '%s'", val);
			if(value_ret.addr == (caddr_t)NULL) {
				report("Returned value: <NULL POINTER>");
			} else {
				report("Returned value: '%.*s' (%u bytes)",
					value_ret.size, (char *)value_ret.addr, value_ret.size);
			}
			ret_val++;
		}
	}

	return(ret_val);
}

static int
xrm_tabulate(from, into)
char *from, *into;
{
	int i,j;

	j = strlen(from);
	for(i=0; i<j; i++) {
		if(from[i]=='T') {
			into[i]='\t';
		} else {
			into[i]=from[i];
		}
	}
	return(j);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	database = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/* Including from file ../rmptrsrc/fn.mc */
/* End of included file ../rmptrsrc/fn.mc */

/*
 * Assertion XrmDestroyDatabase-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XrmDestroyDatabase destroys the specified
 * database.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a database. */
	database=xrm_create_database("test:one");
	if(database == (XrmDatabase)NULL) {
		delete("Could not create test database.");
		return;
	} else
		CHECK;

/* Call xname to destroy the database. */
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmDestroyDatabase(database);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	CHECKUNTESTED(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmDestroyDatabase-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When database is NULL, then a call to XrmDestroyDatabase
 * returns immediately.
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call xname with a NULL database. */
	database = (XrmDatabase)NULL;
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmDestroyDatabase(database);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that xname returns. */
/* (It is not possible to test that return is immediate). */
	CHECK;
	CHECKUNTESTED(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmDestroyDatabase-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XrmDestroyDatabase frees the memory associated
 * with the database.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = rmstartup;
void	(*tet_cleanup)() = cleanup;
