/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rmqgtsrchr.m,v 1.4 92/06/11 17:41:20 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Bool
 *   XrmQGetSearchResource(list, name, class, type_return, value)
 *   XrmHashTable *list;
 *   XrmName name;
 *   XrmClass class;
 *   XrmRepresentation *type_return;
 *   XrmValue *value;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XrmQGetSearchResource	1
char    *TestName = "XrmQGetSearchResource";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XrmQGetSearchResource function
 */
static XrmHashTable *list;
static XrmName name;
static XrmClass class;
static XrmRepresentation *type_return;
static XrmValue *value;


static	XrmRepresentation type;
static	XrmValue val;
static 	XrmDatabase	dbase;

#define XQGSR_ARRAY_SIZE 10
static	XrmHashTable	sl[XQGSR_ARRAY_SIZE];
static	XrmName	nl[XQGSR_ARRAY_SIZE];
static	XrmClass	cl[XQGSR_ARRAY_SIZE];

/* Database data */
#define XQGSR_T1_DATA	5
static	char	*t1_data[XQGSR_T1_DATA] = {
	"applic.window.background:blue",
	"Applic.window.BackGround:cyan",
	"applic.Window.background:magenta",
	"Applic.Window.BackGround:orange",
	"Applic.Window.Unimportant:pink" };

/* Searchlist data */
#define XQGSR_T1_SEARCH	2
static	char	*t1_search[XQGSR_T1_SEARCH][2] = {
	{ "applic", "Applic" },
	{ "window",	"Window" } };

/* SearchResource tests */
#define XQGSR_T1_TESTS	3
static char *t1_tests[XQGSR_T1_TESTS][3] = {
	{ "background",	"BackGround",	"blue"	},
	{ "border",	"BackGround",	"cyan"	},
	{ "misc",	"Unimportant",	"pink"	} };

#define XQGSR_T2_TESTS  1
static char *t2_tests[XQGSR_T2_TESTS][2] = {
	{ "nomatch", "Nomatch" } };


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	list = sl;
	name = 0;
	class = 0;
	type_return = &type;
	value = &val;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XrmQGetSearchResource-1.(A)
 * When name and class fully identify a resource which is
 * contained in the specified database levels list, then a
 * call to XrmQGetSearchResource returns the type in
 * type_return and the value in value_return which first match,
 * and returns True.
 */
static void
t001()
{
int a;
XrmRepresentation exp_type;
Bool ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a database containing testable resource/value pairs. */
	dbase=(XrmDatabase)NULL;
	for(a=0; a<XQGSR_T1_DATA; a++) {
		CHECK;
		XrmPutLineResource(&dbase, t1_data[a]);
	}

#ifdef TESTING
	XrmPutFileDatabase(dbase, "xqgsr_one");
#endif

/* Call XrmQGetSearchList to build up the search list. */
	for(a=0; a<XQGSR_T1_SEARCH; a++) {
		CHECK;
		nl[a]=XrmStringToName(t1_search[a][0]);
		cl[a]=XrmStringToClass(t1_search[a][1]);
	}
	nl[XQGSR_T1_SEARCH]=(XrmName)0;
	cl[XQGSR_T1_SEARCH]=(XrmClass)0;

	if(!XrmQGetSearchList(dbase, nl, cl, sl, XQGSR_ARRAY_SIZE)) {
		delete("XrmQGetSearchList unexpectedly exhausted the allocated space for table.");
		return;
	} else
		CHECK;

	exp_type = XrmStringToRepresentation("String");
	for(a=0; a<XQGSR_T1_TESTS; a++) {

/* Call xname to retrieve information. */
		name = XrmStringToName(t1_tests[a][0]);
		class= XrmStringToClass(t1_tests[a][1]);
		type = (XrmRepresentation)0;
		val.addr = (caddr_t)NULL;
		val.size = 0;
		startcall(Dsp);
		if (isdeleted())
			return;
		ret = XrmQGetSearchResource(list, name, class, type_return, value);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* Verify the retrieved information was correct. */
		if(ret != True) {
			FAIL;
			report("%s returned False when expecting a database match.",
				TestName);
		} else {
			if (type != exp_type) {
				char *rt;

				FAIL;
				report("%s returned unexpected type.", TestName);
				report("Name (class): %s (%s)", t1_tests[a][0], t1_tests[a][1]);
				report("Expected type: String");
				rt=XrmRepresentationToString(type);
				report("Returned type: %s", (rt==NULL?"<NULL STRING":rt));
			} else
				CHECK;

			if (val.addr==NULL ||
				strncmp((char*)val.addr, t1_tests[a][2],
					strlen(t1_tests[a][2]))) {
				FAIL;
				report("%s returned unexpected value", TestName);
				report("Name (class): %s (%s)", t1_tests[a][0], t1_tests[a][1]);
				report("Expected value: %s", t1_tests[a][2]);
				if (val.addr==NULL) {
					report("Returned value: <NULL POINTER>");
				} else {
					report("Returned value: %.*s",
						val.size,
						(char *)val.addr);
				}
			} else
				CHECK;
		}
	}

	CHECKPASS(1+XQGSR_T1_DATA+XQGSR_T1_SEARCH+XQGSR_T1_TESTS*2);

	XrmDestroyDatabase(dbase);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmQGetSearchResource-2.(A)
 * When name and class fully identify a resource which is not
 * contained in the specified database levels list, then a
 * call to XrmQGetSearchResource returns False.
 */
static void
t002()
{
int a;
Bool ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a database containing testable resource/value pairs. */
	dbase=(XrmDatabase)NULL;
	for(a=0; a<XQGSR_T1_DATA; a++) {
		CHECK;
		XrmPutLineResource(&dbase, t1_data[a]);
	}

#ifdef TESTING
	XrmPutFileDatabase(dbase, "xqgsr_two");
#endif

/* Call XrmQGetSearchList to build up the search list. */
	for(a=0; a<XQGSR_T1_SEARCH; a++) {
		CHECK;
		nl[a]=XrmStringToName(t1_search[a][0]);
		cl[a]=XrmStringToClass(t1_search[a][1]);
	}
	nl[XQGSR_T1_SEARCH]=(XrmName)0;
	cl[XQGSR_T1_SEARCH]=(XrmClass)0;

	if(!XrmQGetSearchList(dbase, nl, cl, sl, XQGSR_ARRAY_SIZE)) {
		delete("XrmQGetSearchList unexpectedly exhausted the allocated space for table.");
		return;
	} else
		CHECK;

	for(a=0; a<XQGSR_T2_TESTS; a++) {

/* Call xname to retrieve information. */
		name = XrmStringToName(t2_tests[a][0]);
		class= XrmStringToClass(t2_tests[a][1]);
		type = (XrmRepresentation)0;
		val.addr = (caddr_t)NULL;
		val.size = 0;
		startcall(Dsp);
		if (isdeleted())
			return;
		ret = XrmQGetSearchResource(list, name, class, type_return, value);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* Verify there was no match. */
		if(ret != False) {
			char *rt;

			FAIL;
			report("%s returned True when expecting no match.",
				TestName);
			report("Name (class): %s (%s)", t2_tests[a][0], t2_tests[a][1]);
			rt=XrmRepresentationToString(type);
			report("Returned type: %s", (rt==NULL?"<NULL STRING":rt));
			if (val.addr==NULL) {
				report("Returned value: <NULL POINTER>");
			} else {
				report("Returned value: %.*s",
					val.size,
					(char *)val.addr);
			}
		} else
			CHECK;
	}

	CHECKPASS(1+XQGSR_T1_DATA+XQGSR_T1_SEARCH+XQGSR_T2_TESTS);

	XrmDestroyDatabase(dbase);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = rmstartup;
void	(*tet_cleanup)() = cleanup;
