/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rmstrtqrk.m,v 1.4 92/06/11 17:41:59 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XrmQuark
 *   XrmStringToQuark(string)
 *   char *string;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XrmStringToQuark	1
char    *TestName = "XrmStringToQuark";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XrmStringToQuark function
 */
static char *string;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	string = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XrmStringToQuark-1.(A)
 * A call to XrmStringToQuark returns a quark allocated to
 * represent string.
 */
static void
t001()
{
char *s = "stq_one";
XrmQuark ret;
char *rep;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call xname to allocate a quark for a string. */
	string = s;
	startcall(Dsp);
	if (isdeleted())
		return;
	ret = XrmStringToQuark(string);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Call XrmQuarkToString to obtain representation for the quark. */
	rep = XrmQuarkToString(ret);

#ifdef TESTING
	rep = "barfed";
#endif

/* Verify the quark represents the string. */
	if((rep==(char *)NULL) || strcmp(s,rep)) {
		FAIL;
		report("%s did not allocate a quark representing the string",
			TestName);
		report("Returned quark was: %d", (int)ret);
		report("Expected representation: %s", s);
		report("Returned representation: %s",
			(rep==(char *)NULL?"NULL pointer":rep));
	} else
		CHECK;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmStringToQuark-2.(A)
 * When a quark already exists for string, then a call to
 * XrmStringToQuark returns that quark.
 */
static void
t002()
{
char *s = "stq_two";
XrmQuark ret1, ret2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call xname to allocate a quark for a string. */
	string = s;
	startcall(Dsp);
	if (isdeleted())
		return;
	ret1 = XrmStringToQuark(string);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Call xname to allocate a quark for the string again. */
	startcall(Dsp);
	if (isdeleted())
		return;
	ret2 = XrmStringToQuark(string);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that the quarks were the same. */
	if (ret1 != ret2) {
		FAIL;
		report("%s did not return the same quark to represent",
			TestName);
		report("the same string.");
		report("1st quark return: %d", (int) ret1);
		report("2nd quark return: %d", (int) ret2);
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = rmstartup;
void	(*tet_cleanup)() = cleanup;
