/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rmstrtqrkl.m,v 1.4 92/06/11 17:42:06 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XrmStringToQuarkList(string, quarks_return)
 *   char *string;
 *   XrmQuarkList quarks_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XrmStringToQuarkList	1
char    *TestName = "XrmStringToQuarkList";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XrmStringToQuarkList function
 */
static char *string;
static XrmQuarkList quarks_return;


#define XSTQL_MAX_RETURN 10
static XrmQuark xstql_ret[XSTQL_MAX_RETURN];

static char *xstql_s1[]={
	"xstql_test",
	"ena",
	"deo",
	"tria"};

static char *xstql_s1sep[]={
	".",
	".",
	".",
	""};

static int xstql_ns1=NELEM(xstql_s1);

static char *xstql_s2[]={
	"XStql_test",
	"une",
	"deux",
	"trois",
	"erm",
	"five"};
static char *xstql_s2sep[]={
	"*",
	".",
	"*",
	"*",
	".",
	""};
static int xstql_ns2=NELEM(xstql_s2);

static char *xstql_s3[]={
	"XStqL_test.",
	"jack.",
	"and.",
	"jill"};
static int xstql_ns3=NELEM(xstql_s3);

static void
xstqlclear(value)
XrmQuark value;
{
	int i;

/* Clear the return array, to avoid rogue results. */
	for(i=0; i<XSTQL_MAX_RETURN; i++) 
		xstql_ret[i]=value;

}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	string = 0;
	quarks_return = (XrmQuarkList)xstql_ret;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XrmStringToQuarkList-1.(A)
 * When string is a valid null-terminated resource name format
 * string, then a call to XrmStringToQuarkList returns a quark
 * list in quarks_return corresponding to the components of
 * string.
 */
static void
t001()
{
int  t;
char *buffer1;
char *buffer2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	buffer1 = (char *)malloc( (size_t)256);
	if (buffer1==(char *)NULL) {
		delete("Could not malloc 256 bytes for buffer1.");
		return;
	} else
		CHECK;

	buffer2 = (char *)malloc( (size_t)4096);
	if (buffer2==(char *)NULL) {
		delete("Could not malloc 4096 bytes for buffer2.");
		return;
	} else
		CHECK;

	*buffer1='\0'; /* Make the buffers empty strings. */
	*buffer2='\0';

/* Create the test string. */
	for(t=0; t<xstql_ns1; t++) {
		(void) strcat(buffer1, xstql_s1[t]);
		(void) strcat(buffer1, xstql_s1sep[t]);
	}
	trace("Testing with '%s'", buffer1);

	xstqlclear((XrmQuark)0);

/* Call xname to obtain quark list. */
	string= buffer1;
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmStringToQuarkList(string, quarks_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

#ifdef TESTING
	xstql_ret[0]=0;
	xstql_ret[1]=XrmStringToQuark("TESTING");
#endif

/* Verify that the quark list returned was as expected. */
	for(t=0; t<xstql_ns1; t++) {
		char *ts;

		ts = XrmQuarkToString( xstql_ret[t] );
		if (ts == (char *)NULL) {
			FAIL;
			report("Quark array[%d]=%d. Represents: NO STRING (expecting '%s')",
				t, xstql_ret[t], xstql_s1[t]);
			strcat(buffer2,"<NO STRING>");
		} else
		{
			if(strcmp(xstql_s1[t], ts)) {
				FAIL;
				report("Quark array[%d]=%d. Represents: '%s' (expecting '%s')",
					t, xstql_ret[t], ts, xstql_s1[t]);
			} else {
				CHECK;
				trace("Quark array[%d]=%d. Represents: '%s' as expected.",
					t, xstql_ret[t], ts );
			}
			(void) strcat(buffer2, ts);
		}
		(void) strcat(buffer2, " ");
	}

	if (fail) {
		report("%s did not split the string into the quarks as expected",
			TestName);
		report("String passed to %s: '%s'", TestName, buffer1);
		report("Quark array represents: %s", buffer2);
	} else
		CHECK;

	CHECKPASS(2+xstql_ns1+1);
	free(buffer1);
	free(buffer2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmStringToQuarkList-2.(A)
 * On a call to XrmStringToQuarkList, the string is separated
 * into components according to the positions of periods and
 * asterisks.
 */
static void
t002()
{
int  t;
char *buffer1;
char *buffer2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	buffer1 = (char *)malloc( (size_t)256);
	if (buffer1==(char *)NULL) {
		delete("Could not malloc 256 bytes for buffer1.");
		return;
	} else
		CHECK;

	buffer2 = (char *)malloc( (size_t)4096);
	if (buffer2==(char *)NULL) {
		delete("Could not malloc 4096 bytes for buffer2.");
		return;
	} else
		CHECK;

	*buffer1='\0'; /* Make the buffers empty strings. */
	*buffer2='\0';

/* Create the test string. */
	for(t=0; t<xstql_ns2; t++) {
		(void) strcat(buffer1, xstql_s2[t]);
		(void) strcat(buffer1, xstql_s2sep[t]);
	}
	trace("Testing with '%s'", buffer1);

	xstqlclear((XrmQuark)0);

/* Call xname to obtain quark list. */
	string= buffer1;
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmStringToQuarkList(string, quarks_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

#ifdef TESTING
	xstql_ret[0]=0;
	xstql_ret[1]=XrmStringToQuark("TESTING");
#endif

/* Verify that the quark list returned was as expected. */
	for(t=0; t<xstql_ns2; t++) {
		char *ts;

		ts = XrmQuarkToString( xstql_ret[t] );
		if (ts == (char *)NULL) {
			FAIL;
			report("Quark array[%d]=%d. Represents: NO STRING (expecting '%s')",
				t, xstql_ret[t], xstql_s2[t]);
			strcat(buffer2,"<NO STRING>");
		} else
		{
			if(strcmp(xstql_s2[t], ts)) {
				FAIL;
				report("Quark array[%d]=%d. Represents: '%s' (expecting '%s')",
					t, xstql_ret[t], ts, xstql_s2[t]);
			} else {
				CHECK;
				trace("Quark array[%d]=%d. Represents: '%s' as expected.",
					t, xstql_ret[t], ts );
			}
			(void) strcat(buffer2, ts);
		}
		(void) strcat(buffer2, " ");
	}

	if (fail) {
		report("%s did not split the string into the quarks as expected",
			TestName);
		report("String passed to %s: '%s'", TestName, buffer1);
		report("Quark array represents: '%s'", buffer2);
	} else
		CHECK;

	CHECKPASS(2+xstql_ns2+1);
	free(buffer1);
	free(buffer2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmStringToQuarkList-3.(A)
 * On a call to XrmStringToQuarkList, the quarks_return list
 * is terminated with a zero.
 */
static void
t003()
{
char *buffer1;
XrmQuark unq;
int t;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	buffer1 = (char *)malloc( (size_t)256);
	if (buffer1==(char *)NULL) {
		delete("Could not malloc 256 bytes for buffer1.");
		return;
	} else
		CHECK;

	*buffer1='\0';

/* Create the test string. */
	for(t=0; t<xstql_ns3; t++) {
		(void) strcat(buffer1, xstql_s3[t]);
	}
	trace("Testing with '%s'", buffer1);

/* Set return buffer with unique quark value. */
	unq=XrmUniqueQuark();
	xstqlclear(unq);

/* Call xname to obtain quark list. */
	string= buffer1;
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmStringToQuarkList(string, quarks_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that the quark list was zero terminated. */
	t=0;
	while(t<XSTQL_MAX_RETURN
		&& xstql_ret[t] != unq
		&& xstql_ret[t] != (XrmQuark)0) {
		t++;
	}

	if (t==XSTQL_MAX_RETURN) {
		int i;
		FAIL;
		report("%s did not return the quark array as expected.",
			TestName);
		report("At least %d quarks returned.", XSTQL_MAX_RETURN);
		for (i=0; i<XSTQL_MAX_RETURN; i++)
			report("Quark array[%d]=%d", i, xstql_ret[i]);
	} else {
		if (xstql_ret[t] == unq) {
			FAIL;
			report("%s did not terminate the array with a zero.",
				TestName);
		} else
			CHECK;
	}

	CHECKPASS(2);
	free(buffer1);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = rmstartup;
void	(*tet_cleanup)() = cleanup;
