/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rttbffrs.m,v 1.6 92/06/11 17:42:30 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XRotateBuffers(display, rotate)
 *   Display	*display;
 *   int	rotate;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XRotateBuffers	1
char    *TestName = "XRotateBuffers";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XRotateBuffers function
 */
static Display	*display;
static int	rotate;


#include	"Xatom.h"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	rotate = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XRotateBuffers-1.(A)
 * A call to XRotateBuffers rotates the cut buffers by rotate
 * modulo 8.
 */
static void
t001()
{
char	*bp;
char	*rbp;
int	nr;
int	len;
int 	i;
int 	j;
struct	bstrct {
	char	*data;
	int	len;
}	bfrs[8], *bptr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(i=0, bptr=bfrs; i<8; i++, bptr++) {

		len = 1+i*123;
		bptr->len = len;

		if((bptr->data = (char *)malloc(len)) == (char *) NULL) {
			delete("malloc() returned NULL.");
			return;
		} else
			CHECK;

		for(j=len, bp=bptr->data; j>0; *bp++ = (j) %  (256 - i), j--);

		XStoreBuffer(display, bptr->data, len, i);
	}


	rotate = -51;
	startcall(display);
	if (isdeleted())
		return;
	XRotateBuffers(display, rotate);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for(i=0, bptr=bfrs; i<8; i++, bptr++) {
		
		rbp = XFetchBuffer(display, &nr, (8 + i + (rotate % 8)) % 8);

		if(rbp == (char *) NULL) {
			delete("Buffer %d was not set.", i);
			return;
		} else {
			CHECK;

			if(bptr->len != nr) {
				report("%s() did set buffer %d to contain %d bytes instead of %d.", TestName, i, nr, bptr->len);
				FAIL;
			} else {
				CHECK;

				if(memcmp(rbp, bptr->data, nr) != 0) {
					report("%s() set buffer %d to contain the wrong data.", TestName, i);
					FAIL;
				} else
					CHECK;
			}

			free(bptr->data);
			XFree(rbp);
		}
	}

	CHECKPASS(8 + 8 * 3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XRotateBuffers-2.(A)
 * When any of the 8 cut buffers have not been created, then a
 * BadMatch error occurs.
 */
static void
t002()
{
char	*tstr = "XTest Multi buffer string";
int	len = 1 + strlen(tstr);
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(i=0; i < 8; i++)
		XStoreBuffer(display, tstr, len, i);

	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER4);
	rotate = 1;
	startcall(display);
	if (isdeleted())
		return;
	XRotateBuffers(display, rotate);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadMatch)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
