/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: shrnkrgn.m,v 1.6 92/06/11 17:42:51 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XShrinkRegion(r, dx, dy)
 *   Region	r;
 *   int	dx;
 *   int	dy;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XShrinkRegion	1
char    *TestName = "XShrinkRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XShrinkRegion function
 */
static Region	r;
static int	dx;
static int	dy;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	r = 0;
	dx = 0;
	dy = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XShrinkRegion-1.(A)
 * When dx and dy are zero, then a call to XShrinkRegion does
 * not change the size of region r.
 */
static void
t001()
{
Region			R1;
Region			R2;
static XRectangle	rect = { 7,5, 23, 45};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( ((R1 = XCreateRegion()) == (Region) NULL) ||
	    ((R2 = XCreateRegion()) == (Region) NULL) ){
		delete("XCreateRegion() returned NULL.");
		return;
	} else
		CHECK;

	XUnionRectWithRegion(&rect, R1, R1);
	XUnionRectWithRegion(&rect, R2, R2);

	r = R1;
	dx = dy = 0;
	startcall(Dsp);
	if (isdeleted())
		return;
	XShrinkRegion(r, dx, dy);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( XEqualRegion(R1, R2) != True ) {
		report("%s() changed a region when dx and dy were zero.", TestName);
		FAIL;
	} else
		PASS;

	XDestroyRegion(R1);	
	XDestroyRegion(R2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XShrinkRegion-2.(A)
 * When dx and dy are both positive, and a point x, y is
 * outside region r, then call to XShrinkRegion changes the
 * size of the region r such that the point x, y remains
 * outside region r, and some point x1, y1 previously inside
 * region r is now outside region r.
 */
static void
t002()
{
static XRectangle	rect = { 5,7, 100, 200};
Region			R1;
Region			R2;
Region			R3;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = XCreateRegion();
	R2 = XCreateRegion();
	R3 = XCreateRegion();

	if( (R1 == (Region) NULL) || (R2 == (Region) NULL) || R3 == (Region) NULL) {
		delete("XCreateRegion() returned NULL.");
		return;
	} else
		CHECK;

	XUnionRectWithRegion(&rect, R1, R1);
	XUnionRegion(R1, R1, R2);

	dx = dy = 1;
	r = R2;
	startcall(Dsp);
	if (isdeleted())
		return;
	XShrinkRegion(r, dx, dy);	
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XXorRegion(R1, R2, R3);

	if(XEmptyRegion(R3) == True) {
		report("%s() did not change the size of the source region.", TestName);
		FAIL;
		return;
	} else
		CHECK;

	XUnionRegion(R2, R3, R3);

	if( XEqualRegion(R1, R3) == False) {
		report("%s() did not produce a shrunk region contained in its source.", TestName);
		FAIL;
	} else
		CHECK;

	XDestroyRegion(R1);
	XDestroyRegion(R2);
	XDestroyRegion(R3);

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XShrinkRegion-3.(A)
 * When dx and dy are both negative, and a point x, y is in
 * region r, then call to XShrinkRegion changes the size of
 * the region r such that the point x, y remains in region r,
 * and some point x1, y1 previously outside region r is now
 * inside region r.
 */
static void
t003()
{
static XRectangle	rect = { 5,7, 100, 200};
Region			R1;
Region			R2;
Region			R3;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = XCreateRegion();
	R2 = XCreateRegion();
	R3 = XCreateRegion();

	if( (R1 == (Region) NULL) || (R2 == (Region) NULL) || R3 == (Region) NULL) {
		delete("XCreateRegion() returned NULL.");
		return;
	} else
		CHECK;

	XUnionRectWithRegion(&rect, R1, R1);
	XUnionRegion(R1, R1, R2);

	dx = dy = -1;
	r = R2;
	startcall(Dsp);
	if (isdeleted())
		return;
	XShrinkRegion(r, dx, dy);	
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XXorRegion(R1, R2, R3);

	if(XEmptyRegion(R3) == True) {
		report("%s() did not change the size of the source region.", TestName);
		FAIL;
		return;
	} else
		CHECK;

	XUnionRegion(R2, R3, R3);

	if( XEqualRegion(R2, R3) == False) {
		report("%s() did not produce an enlarged region containing its source.", TestName);
		FAIL;
	} else
		CHECK;

	XDestroyRegion(R1);
	XDestroyRegion(R2);
	XDestroyRegion(R3);

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
