/*
 *      SCCS:  @(#)  crtfntcrsr.m Rel 1.5	    (11/25/91)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1991 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 * 
 */
/*
 * SYNOPSIS:
 *   Cursor
 *   XCreateFontCursor(display, shape)
 *   Display *display;
 *   unsigned int shape;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCreateFontCursor	1
char    *TestName = "XCreateFontCursor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XCreateFontCursor function
 */
static Display *display;
static unsigned int shape;


#include <X11/cursorfont.h>

/*
 * mkcolor() -	return a pointer to a color structure.
 *		flag indicates whether or not color is foreground
 * 		(Copied from rclrcrs.m.)
 */
static XColor *
mkcolor(flag)
{
	static	XColor	fore;
	static	XColor	back;
	static	int	first = 1;

	if (first)
	{
		first = 0;

		fore.pixel = BlackPixel(display, DefaultScreen(display));
		XQueryColor(display, DefaultColormap(display, DefaultScreen(display)), &fore);
		back.pixel = WhitePixel(display, DefaultScreen(display));
		XQueryColor(display, DefaultColormap(display, DefaultScreen(display)), &back);
	}
	return(flag ? &fore : &back);
}

static
void
test_font(symbol)
int	symbol;
{
Cursor	qstat;
Window w;
XVisualInfo *vp;
int     pass = 0, fail = 0;

/* Create windows. */
	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		w = makewin(display, vp);

		shape = symbol;

		trace("Shape used is %x", shape);
/* Call XCreateFontCursor with specified shape. */
	
		startcall(display);
		if (isdeleted())
			return;
		qstat = XCreateFontCursor(display, shape);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* Verify that XCreateFontCursor returns non-zero. */
		if (qstat == 0) {
			report("On a call to XCreateFontCursor,");
			report("wrong value %ld was returned", (long) qstat);
			FAIL;
		} else
			CHECK;

/* Call XDefineCursor and verify that no error occurs. */
		startcall(display);
		if (isdeleted())
			return;
		XDefineCursor(display, w, qstat);
		endcall(display);
		if (geterr() != Success) {
			report("On a call to XDefineCursor,");
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		else
			CHECK;

/* Call XRecolorCursor and verify that no error occurs. */
		startcall(display);
		if (isdeleted())
			return;
		XRecolorCursor(display, qstat, mkcolor(1), mkcolor(0));
		endcall(display);
		if (geterr() != Success) {
			report("On a call to XRecolorCursor,");
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		else
			CHECK;

/* Call XFreeCursor and verify that no error occurs. */
		startcall(display);
		if (isdeleted())
			return;
		XFreeCursor(display, qstat);
		endcall(display);
		if (geterr() != Success) {
			report("On a call to XFreeCursor,");
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		else
			CHECK;
	}

	CHECKPASS(4*nvinf());
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	shape = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XCreateFontCursor-1.(A)
 * When the shape argument is XC_X_cursor, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_X_cursor);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-2.(A)
 * When the shape argument is XC_bottom_left_corner, then a
 * call to XCreateFontCursor creates a Cursor and returns the
 * cursor ID.
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_bottom_left_corner);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-3.(A)
 * When the shape argument is XC_bottom_right_corner, then a
 * call to XCreateFontCursor creates a Cursor and returns the
 * cursor ID.
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_bottom_right_corner);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-4.(A)
 * When the shape argument is XC_top_left_corner, then a call
 * to XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_top_left_corner);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-5.(A)
 * When the shape argument is XC_top_right_corner, then a
 * call to XCreateFontCursor creates a Cursor and returns the
 * cursor ID.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_top_right_corner);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-6.(A)
 * When the shape argument is XC_arrow, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_arrow);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-7.(A)
 * When the shape argument is XC_based_arrow_down, then a
 * call to XCreateFontCursor creates a Cursor and returns the
 * cursor ID.
 */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_based_arrow_down);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-8.(A)
 * When the shape argument is XC_based_arrow_up, then a call
 * to XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t008()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_based_arrow_up);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-9.(A)
 * When the shape argument is XC_double_arrow, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_double_arrow);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-10.(A)
 * When the shape argument is XC_question_arrow, then a call
 * to XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t010()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_question_arrow);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-11.(A)
 * When the shape argument is XC_sb_h_double_arrow, then a
 * call to XCreateFontCursor creates a Cursor and returns the
 * cursor ID.
 */
static void
t011()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_sb_h_double_arrow);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-12.(A)
 * When the shape argument is XC_sb_v_double_arrow, then a
 * call to XCreateFontCursor creates a Cursor and returns the
 * cursor ID.
 */
static void
t012()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_sb_v_double_arrow);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-13.(A)
 * When the shape argument is XC_top_left_arrow, then a call
 * to XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_top_left_arrow);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-14.(A)
 * When the shape argument is XC_bottom_tee, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t014()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_bottom_tee);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-15.(A)
 * When the shape argument is XC_left_tee, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t015()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_left_tee);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-16.(A)
 * When the shape argument is XC_right_tee, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t016()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_right_tee);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-17.(A)
 * When the shape argument is XC_top_tee, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t017()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_top_tee);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-18.(A)
 * When the shape argument is XC_bottom_side, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t018()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_bottom_side);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-19.(A)
 * When the shape argument is XC_left_side, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t019()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_left_side);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-20.(A)
 * When the shape argument is XC_right_side, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t020()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_right_side);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-21.(A)
 * When the shape argument is XC_top_side, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t021()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_top_side);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-22.(A)
 * When the shape argument is XC_ll_angle, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t022()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_ll_angle);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-23.(A)
 * When the shape argument is XC_lr_angle, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t023()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_lr_angle);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-24.(A)
 * When the shape argument is XC_ul_angle, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t024()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_ul_angle);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-25.(A)
 * When the shape argument is XC_ur_angle, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t025()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_ur_angle);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-26.(A)
 * When the shape argument is XC_circle, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t026()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_circle);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-27.(A)
 * When the shape argument is XC_cross, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t027()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_cross);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-28.(A)
 * When the shape argument is XC_crosshair, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t028()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_crosshair);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-29.(A)
 * When the shape argument is XC_center_ptr, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t029()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_center_ptr);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-30.(A)
 * When the shape argument is XC_left_ptr, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t030()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_left_ptr);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateFontCursor-31.(A)
 * When the shape argument is XC_right_ptr, then a call to
 * XCreateFontCursor creates a Cursor and returns the cursor
 * ID.
 */
static void
t031()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	test_font(XC_right_ptr);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	t025, 25,
	t026, 26,
	t027, 27,
	t028, 28,
	t029, 29,
	t030, 30,
	t031, 31,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
